package com.psp.payment.common.converters.populators;

import com.psp.model.TokenisedPaymentModeModel;
import com.psp.payment.common.dto.PspBrand;
import com.psp.payment.common.dto.PspPaymentMode;
import com.psp.payment.common.dto.PspPaymentModeData;
import de.hybris.platform.commercefacades.product.converters.populator.ImagePopulator;
import de.hybris.platform.commercefacades.product.data.ImageData;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;

import javax.annotation.Resource;
import java.util.*;

public class PspPaymentModePopulator implements Populator<List<? extends PaymentModeModel>, PspPaymentModeData> {

    @Resource
    private ImagePopulator imagePopulator;

    @Override
    public void populate(List<? extends PaymentModeModel> paymentModeModelList, PspPaymentModeData paymentModeData) {

        if (CollectionUtils.isNotEmpty(paymentModeModelList) && paymentModeData != null) {
            Map<String, List<PspBrand>> paymentModeMap = new HashMap<>();

            for (PaymentModeModel paymentModeModel : paymentModeModelList) {
                String code = paymentModeModel.getCode();
                String paymentMethod = paymentModeModel.getPaymentMethod();
                String brand = paymentModeModel.getBrand();
                String title = paymentModeModel.getName();
                MediaModel image = paymentModeModel.getImage();
                ComposedTypeModel composedTypeModel = paymentModeModel.getPaymentInfoType();
                boolean isTokenised = paymentModeModel instanceof TokenisedPaymentModeModel ? Boolean.TRUE : Boolean.FALSE;

                List<PspBrand> brandList = paymentModeMap.get(paymentMethod);

                if (brandList == null) {
                    brandList = new ArrayList<>();
                    paymentModeMap.put(paymentMethod, brandList);
                }

                PspBrand currentBrand = new PspBrand();
                currentBrand.setCode(code);
                currentBrand.setName(brand);
                currentBrand.setDisplayName(title);
                currentBrand.setComposedTypeModel(composedTypeModel);
                currentBrand.setTokenised(isTokenised);

                if (image != null) {
                    ImageData imageData = new ImageData();
                    imagePopulator.populate(image, imageData);
                    currentBrand.setImage(imageData);
                }

                brandList.add(currentBrand);
            }

            populateTarget(paymentModeData, paymentModeMap);
        }
    }

    private void populateTarget(PspPaymentModeData paymentModeData, Map<String, List<PspBrand>> paymentModeMap) {
        if (MapUtils.isNotEmpty(paymentModeMap)) {
            List<PspPaymentMode> pspPaymentModeDataList = new ArrayList<>();
            Set<Map.Entry<String, List<PspBrand>>> entrySet = paymentModeMap.entrySet();

            for (Map.Entry<String, List<PspBrand>> entry : entrySet) {
                String paymentMode = entry.getKey();
                List<PspBrand> brandList = entry.getValue();

                PspPaymentMode pspPaymentMode = new PspPaymentMode();
                pspPaymentMode.setCode(paymentMode);
                pspPaymentMode.setBrands(brandList);

                for (PspBrand pspBrand : brandList) {
                    Boolean isTokenised = pspBrand.getTokenised();
                    pspPaymentMode.setTokenised(isTokenised);

                    if (isTokenised) {
                        break;
                    }
                }

                pspPaymentModeDataList.add(pspPaymentMode);
            }

            paymentModeData.setPaymentModes(pspPaymentModeDataList);
        }
    }
}
