package com.psp.payment.common.dao.impl;

import com.psp.payment.common.dao.PspCustomerAccountDao;
import de.hybris.platform.commerceservices.customer.dao.impl.DefaultCustomerAccountDao;
import de.hybris.platform.core.PK;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.search.SearchResult;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspCustomerAccountDaoImpl extends DefaultCustomerAccountDao implements PspCustomerAccountDao {


    private static final String FIND_PAYMENT_INFO_BY_CUSTOMER_QUERY = "SELECT {" + PaymentInfoModel.PK + "} FROM {"
            + PaymentInfoModel._TYPECODE + "} WHERE {" + PaymentInfoModel.USER + "} = ?" + CustomerModel._TYPECODE + " AND {"
            + PaymentInfoModel.PK + "} = ?" + PaymentInfoModel.PK + " AND {" + PaymentInfoModel.DUPLICATE + "} = ?" + PaymentInfoModel.DUPLICATE;

    private static final String FIND_SAVED_PAYMENT_INFOS_BY_CUSTOMER_QUERY = "SELECT {" + PaymentInfoModel.PK
            + "} FROM {" + PaymentInfoModel._TYPECODE + "} WHERE {" + PaymentInfoModel.USER
            + "} = ?" + CustomerModel._TYPECODE + " AND {" + PaymentInfoModel.SAVED + "} = ?" + PaymentInfoModel.SAVED + " AND {" + PaymentInfoModel.DUPLICATE
            + "} = ?" + PaymentInfoModel.DUPLICATE;
    private static final String FIND_PAYMENT_INFOS_BY_CUSTOMER_QUERY = "SELECT {" + PaymentInfoModel.PK + "} FROM {"
            + PaymentInfoModel._TYPECODE + "} WHERE {" + PaymentInfoModel.USER + "} = ?" + CustomerModel._TYPECODE + " AND {"
            + PaymentInfoModel.DUPLICATE + "} = ?" + PaymentInfoModel.DUPLICATE;

    @Override
    public PaymentInfoModel findPaymentInfoByCustomer(CustomerModel customerModel, String code) {
        validateParameterNotNull(customerModel, "Customer must not be null");
        Map<String, Object> queryParams = new HashMap<String, Object>();
        queryParams.put(CustomerModel._TYPECODE, customerModel);
        queryParams.put(PaymentInfoModel.DUPLICATE, Boolean.FALSE);
        queryParams.put(PaymentInfoModel.PK, PK.parse(code));
        SearchResult<PaymentInfoModel> result = getFlexibleSearchService().search(
                FIND_PAYMENT_INFO_BY_CUSTOMER_QUERY, queryParams);
        return result.getCount() > 0 ? result.getResult().get(0) : null;
    }

    @Override
    public List<PaymentInfoModel> findPaymentInfosByCustomer(CustomerModel customerModel, boolean saved) {
        validateParameterNotNull(customerModel, "Customer must not be null");
        Map<String, Object> queryParams = new HashMap<String, Object>();
        queryParams.put(CustomerModel._TYPECODE, customerModel);
        if (saved) {
            queryParams.put(PaymentInfoModel.SAVED, Boolean.TRUE);
        }
        queryParams.put(PaymentInfoModel.DUPLICATE, Boolean.FALSE);
        SearchResult<PaymentInfoModel> result = getFlexibleSearchService().search(
                saved ? FIND_SAVED_PAYMENT_INFOS_BY_CUSTOMER_QUERY : FIND_PAYMENT_INFOS_BY_CUSTOMER_QUERY, queryParams);
        return result.getResult();
    }
}
