package com.psp.payment.common.dao.impl;

import com.psp.payment.common.dao.PspPaymentModeDao;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.order.daos.impl.DefaultPaymentModeDao;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import de.hybris.platform.servicelayer.search.SearchResult;

import java.math.BigDecimal;
import java.util.List;

import static com.psp.constants.PspConstants.QUERY_PARAM_AMOUNT;
import static com.psp.constants.PspConstants.QUERY_PARAM_CMSSITE;
import static com.psp.constants.PspConstants.QUERY_PARAM_CODE;
import static com.psp.constants.PspFlexiSearchQueries.ACTIVE_PAYMENTMODE_FOR_PAYMENT_INFO;
import static com.psp.constants.PspFlexiSearchQueries.ACTIVE_PAYMENTMODE_WITHIN_THRESHOLD;
import static com.psp.constants.PspFlexiSearchQueries.ALL_ACTIVE_PAYMENTMODES_WITHIN_THRESHOLD;

public class PspPaymentModeDaoImpl extends DefaultPaymentModeDao implements PspPaymentModeDao {

    @Override
    public List<PaymentModeModel> findActivePaymentModeForCode(BigDecimal amount, String code) {
        FlexibleSearchQuery query = new FlexibleSearchQuery(ACTIVE_PAYMENTMODE_WITHIN_THRESHOLD);
        query.addQueryParameter(QUERY_PARAM_CODE, code);
        query.addQueryParameter(QUERY_PARAM_AMOUNT, amount);
        SearchResult<PaymentModeModel> searchResult = getFlexibleSearchService().search(query);

        return searchResult.getResult();
    }

    @Override
    public List<PaymentModeModel> findActivePaymentModes(BigDecimal amount, final CMSSiteModel cmsSiteModel) {
        FlexibleSearchQuery query = new FlexibleSearchQuery(ALL_ACTIVE_PAYMENTMODES_WITHIN_THRESHOLD);
        query.addQueryParameter(QUERY_PARAM_AMOUNT, amount);
        query.addQueryParameter(QUERY_PARAM_CMSSITE, cmsSiteModel);
        SearchResult<PaymentModeModel> searchResult = getFlexibleSearchService().search(query);

        return searchResult.getResult();
    }

    @Override
    public List<PaymentModeModel> findActivePaymentModeForPaymentInfo(PaymentInfoModel paymentInfoModel) {
        FlexibleSearchQuery query = new FlexibleSearchQuery(ACTIVE_PAYMENTMODE_FOR_PAYMENT_INFO);
        query.addQueryParameter(QUERY_PARAM_CODE, paymentInfoModel.getItemtype());
        SearchResult<PaymentModeModel> searchResult = getFlexibleSearchService().search(query);

        return searchResult.getResult();
    }
}
