package com.psp.payment.common.facades.user.impl;

import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.services.PspCustomerAccountService;
import com.psp.payment.common.strategies.PspPaymentInfoConvertStrategy;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commercefacades.user.UserFacade;
import de.hybris.platform.commercefacades.user.impl.DefaultUserFacade;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNullStandardMessage;

public class PspUserFacadeImpl extends DefaultUserFacade implements UserFacade {
    @Resource
    private PspCustomerAccountService customerAccountService;

    @Resource
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Override
    public void setDefaultPaymentInfo(CCPaymentInfoData paymentInfoData) {
        validateParameterNotNullStandardMessage("paymentInfoData", paymentInfoData);
        CustomerModel currentCustomer = getCurrentUserForCheckout();
        PaymentInfoModel paymentInfoModel = customerAccountService.getPaymentInfoForCode(currentCustomer, paymentInfoData.getId());
        if (paymentInfoModel != null) {
            customerAccountService.setDefaultPaymentInfo(currentCustomer, paymentInfoModel);
        }
    }

    @Override
    public List<CCPaymentInfoData> getCCPaymentInfos(boolean saved) {
        CustomerModel currentCustomer = getCurrentUserForCheckout();
        List<PaymentInfoModel> paymentInfos = customerAccountService.getPaymentInfos(currentCustomer,
                saved);
        List<CCPaymentInfoData> ccPaymentInfos = new ArrayList<CCPaymentInfoData>();
        PaymentInfoModel defaultPaymentInfoModel = currentCustomer.getDefaultPaymentInfo();

        for (PaymentInfoModel paymentInfoModel : paymentInfos) {
            PspPaymentInfoConvertStrategy pspPaymentInfoConvertStrategy = pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(paymentInfoModel);
            CCPaymentInfoData paymentInfoData = pspPaymentInfoConvertStrategy.convert(paymentInfoModel);

            if (paymentInfoModel.equals(defaultPaymentInfoModel)) {
                paymentInfoData.setDefaultPaymentInfo(true);
                ccPaymentInfos.add(0, paymentInfoData);
            } else {
                ccPaymentInfos.add(paymentInfoData);
            }
        }
        return ccPaymentInfos;
    }

}
