package com.psp.payment.common.services.impl;

import com.psp.payment.common.services.PspAddressService;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;

import javax.annotation.Resource;

/**
 * Implementation for performing address related operations.
 */
public class PspAddressServiceImpl implements PspAddressService {

    @Resource
    private CartService cartService;

    @Resource
    private Converter<AddressData, AddressModel> addressReverseConverter;

    @Resource
    private ModelService modelService;

    @Resource
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Override
    public void saveBillingAddressToCurrentCart(AddressData addressData, boolean isExistingAddress) {
        if (addressData != null) {
            CartModel cartModel = cartService.getSessionCart();
            AddressModel addressModel;

            if (isExistingAddress) {
                addressModel = modelService.clone(cartModel.getDeliveryAddress());
                addressModel.setBillingAddress(Boolean.TRUE);
                addressModel.setShippingAddress(Boolean.FALSE);
                addressModel.setVisibleInAddressBook(Boolean.FALSE);
            } else {
                addressModel = modelService.create(AddressModel.class);
                addressReverseConverter.convert(addressData, addressModel);
            }

            // Safe to cast during checkout
            CustomerModel customerModel = (CustomerModel) cartModel.getUser();
            addressModel.setOwner(cartModel);
            addressModel.setEmail(customerEmailResolutionService.getEmailForCustomer(customerModel));
            cartModel.setPaymentAddress(addressModel);

            modelService.saveAll(addressModel, cartModel);
        }
    }
}
