package com.psp.payment.common.services.impl;

import com.psp.model.ErrorMessageModel;
import com.psp.model.StatusMessageModel;
import com.psp.payment.common.dao.PspErrorMessageDao;
import com.psp.payment.common.dao.PspStatusMessageDao;
import com.psp.payment.common.services.PspErrorMessageService;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateIfSingleResult;
import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspErrorMessageServiceImpl implements PspErrorMessageService {

    @Resource
    private PspErrorMessageDao pspErrorMessageDao;

    @Resource
    private PspStatusMessageDao pspStatusMessageDao;

    @Override
    public String fetchErrorMessageByCode(String code) {
        validateParameterNotNull(code, "Parameter [code] cannot be null.");
        String errorMessage = null;
        List<ErrorMessageModel> errorMessageModelList = pspErrorMessageDao.findErrorMessageByCode(code);

        if (!Collections.EMPTY_LIST.equals(errorMessageModelList)) {
            validateIfSingleResult(errorMessageModelList, ErrorMessageModel.class, ErrorMessageModel.CODE, code);
            errorMessage = errorMessageModelList.get(0).getMessage();
        }

        return errorMessage;
    }

    @Override
    public String fetchErrorMessageByStatus(String status) {
        validateParameterNotNull(status, "Parameter [status] cannot be null.");
        String errorMessage = null;
        List<StatusMessageModel> statusMessageModelList = pspStatusMessageDao.findErrorMessageByStatus(status);

        if (!Collections.EMPTY_LIST.equals(statusMessageModelList)) {
            validateIfSingleResult(statusMessageModelList, StatusMessageModel.class, StatusMessageModel.STATUS, status);
            errorMessage = statusMessageModelList.get(0).getDescription();
        }

        return errorMessage;
    }
}
