package com.psp.payment.common.services.impl;

import com.psp.payment.common.dao.PspPaymentModeDao;
import com.psp.payment.common.services.PspPaymentModeService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.order.impl.DefaultPaymentModeService;
import org.apache.commons.collections.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.*;

public class PspPaymentModeServiceImpl extends DefaultPaymentModeService implements PspPaymentModeService {

    @Resource
    private CartService cartService;

    @Resource
    private PspPaymentModeDao pspPaymentModeDao;

    @Override
    public List<PaymentModeModel> getAvailablePaymentModeForCode(String code) {
        validateParameterNotNullStandardMessage(PaymentModeModel.CODE, code);
        CartModel cartModel = cartService.getSessionCart();

        final List<PaymentModeModel> paymentModeModelList = pspPaymentModeDao.findActivePaymentModeForCode(BigDecimal.valueOf(cartModel.getTotalPrice()), code);

        if (CollectionUtils.isNotEmpty(paymentModeModelList)) {
            validateIfSingleResult(paymentModeModelList, PaymentModeModel.class, PaymentModeModel.CODE, code);
        }

        return paymentModeModelList;
    }

    @Override
    public List<PaymentModeModel> getAvailablePaymentMethods() {
        CartModel cartModel = cartService.getSessionCart();
        BaseSiteModel baseSiteModel = cartModel.getSite();
        CMSSiteModel cmsSiteModel = null;

        if (baseSiteModel != null) {
            cmsSiteModel = (CMSSiteModel) baseSiteModel;
        }

        return pspPaymentModeDao.findActivePaymentModes(BigDecimal.valueOf(cartModel.getTotalPrice()), cmsSiteModel);
    }

    @Override
    public PaymentModeModel getPaymentModeForPaymentInfo(PaymentInfoModel paymentInfoModel) {
        validateParameterNotNull(paymentInfoModel, "Parameter [paymentInfoModel] cannot be null.");
        List<PaymentModeModel> paymentModeModelList = pspPaymentModeDao.findActivePaymentModeForPaymentInfo(paymentInfoModel);
        PaymentModeModel paymentModeModel = null;

        if (CollectionUtils.isNotEmpty(paymentModeModelList)) {
            paymentModeModel = paymentModeModelList.get(0);
        }

        return paymentModeModel;
    }
}
