package com.psp.payment.common.services.impl;

import com.psp.enums.PspPaymentProcessOption;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.services.PspPaymentModeService;
import com.psp.payment.common.services.PspPaymentProcessService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.site.BaseSiteService;
import org.apache.commons.lang.BooleanUtils;

import javax.annotation.Resource;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspPaymentProcessServiceImpl implements PspPaymentProcessService {

    @Resource
    private BaseSiteService baseSiteService;

    @Resource
    private PspPaymentModeService pspPaymentModeService;

    @Override
    public PspOperation findOperationForPaymentMode(PaymentModeModel paymentModeModel) {
        validateParameterNotNull(paymentModeModel, "Parameter [paymentModeModel] cannot be null.");
        Boolean forceDirectSale = paymentModeModel.getForceDirectSale();
        PspOperation pspOperation;

        if (BooleanUtils.isNotTrue(forceDirectSale)) {
            BaseSiteModel currentSite = baseSiteService.getCurrentBaseSite();
            PspPaymentProcessOption pspPaymentProcessOption = currentSite.getPaymentProcess();

            if (PspPaymentProcessOption.DIRECTSALE.equals(pspPaymentProcessOption)) {
                pspOperation = PspOperation.SAL;
            } else {
                pspOperation = PspOperation.RES;
            }
        } else {
            pspOperation = PspOperation.SAL;
        }

        return pspOperation;
    }

    @Override
    public PaymentTransactionType findPaymentTransactionTypeForPaymentInfo(PaymentInfoModel paymentInfoModel, BaseSiteModel baseSiteModel) {
        validateParameterNotNull(paymentInfoModel, "Parameter [paymentInfoModel] cannot be null.");
        validateParameterNotNull(baseSiteModel, "Parameter [baseSiteModel] cannot be null.");
        PaymentModeModel paymentModeModel = pspPaymentModeService.getPaymentModeForPaymentInfo(paymentInfoModel);
        Boolean forceDirectSale = paymentModeModel.getForceDirectSale();

        if (BooleanUtils.isTrue(forceDirectSale)) {
            return PaymentTransactionType.CAPTURE;
        } else {
            PspPaymentProcessOption pspPaymentProcessOption = baseSiteModel.getPaymentProcess();

            if (PspPaymentProcessOption.DIRECTSALE.equals(pspPaymentProcessOption)) {
                return PaymentTransactionType.CAPTURE;
            } else {
                return PaymentTransactionType.AUTHORIZATION;
            }
        }
    }
}
