package com.psp.payment.common.strategies.impl;

import com.psp.payment.common.enums.PspTokenisationStatus;
import com.psp.payment.common.strategies.PspPaymentTransactionStrategy;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.UUID;

public class PspPaymentTransactionStrategyImpl implements PspPaymentTransactionStrategy {

    @Resource
    private ModelService modelService;

    @Resource
    private PaymentService paymentService;

    @Override
    public PaymentTransactionEntryModel savePaymentTransactionEntry(final CustomerModel customerModel, final String requestToken) {
        if (StringUtils.isNotEmpty(requestToken)) {
            final PaymentTransactionModel transaction = modelService.create(PaymentTransactionModel.class);
            final PaymentTransactionType paymentTransactionType = PaymentTransactionType.CREATE_SUBSCRIPTION;
            transaction.setCode(customerModel.getUid() + "_" + UUID.randomUUID());
            transaction.setRequestToken(requestToken);
            modelService.save(transaction);

            final PaymentTransactionEntryModel entry = modelService.create(PaymentTransactionEntryModel.class);
            entry.setType(paymentTransactionType);
            entry.setRequestToken(requestToken);
            entry.setTime(new Date());
            entry.setPaymentTransaction(transaction);
            entry.setTransactionStatus(PspTokenisationStatus.OK.name());
            entry.setTransactionStatusDetails(TransactionStatusDetails.SUCCESFULL.name());
            entry.setCode(paymentService.getNewPaymentTransactionEntryCode(transaction, paymentTransactionType));
            modelService.save(entry);

            return entry;
        }

        return null;
    }
}
