package com.psp.payment.common.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.common.strategies.PspPostPaymentFeedbackStrategy;
import com.psp.payment.directlink.services.PspDirectLinkService;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.processengine.BusinessProcessService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;


public class PspPostPaymentFeedbackStrategyImpl implements PspPostPaymentFeedbackStrategy {

    private static final String REVIEW_DECISION_EVENT_NAME = "_ReviewDecision";

    @Resource
    private BusinessProcessService businessProcessService;

    @Resource
    private PspDirectLinkService pspDirectLinkService;

    @Resource
    private ModelService modelService;

    @Resource
    private FlexibleSearchService flexibleSearchService;

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Override
    public void handleFeedback(Map<String, String> parameters) {
        String payId = parameters.get(PspConstants.KEY_PAYID);
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionModel.setRequestId(payId);
        paymentTransactionModel.setRequestToken(payId);
        paymentTransactionModel = (PaymentTransactionModel) flexibleSearchService.getModelByExample(paymentTransactionModel);

        OrderModel order = (OrderModel) paymentTransactionModel.getOrder();
        String transactionStatus = pspDirectLinkService.getTransactionStatus(parameters);
        for (final PaymentTransactionModel transaction : order.getPaymentTransactions()) {
            List<PaymentTransactionEntryModel> transactionEntries = transaction.getEntries();
            for (int index = transactionEntries.size() - 1; index >= 0; index--) {
                final PaymentTransactionEntryModel entry = transactionEntries.get(index);
                if (pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(order.getPaymentInfo(), order.getSite()).equals(entry.getType())
                        && PspTransactionStatus.UNCERTAIN.name().equals(entry.getTransactionStatus())) {
                    entry.setTransactionStatus(transactionStatus);
                    modelService.save(entry);
                    businessProcessService.triggerEvent(order.getCode() + REVIEW_DECISION_EVENT_NAME);
                }
            }
        }
    }
}
