package com.psp.payment.common.utils;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Properties;

public class PspExtensionTrackingUtils {

    private static final Logger LOGGER = Logger.getLogger(PspExtensionTrackingUtils.class.getName());

    private String trackingDetailsLength;

    private String releaseDate;

    private String extensionName;

    public PspExtensionTrackingUtils() {
        InputStream resourceAsStream = this.getClass().getResourceAsStream("/releaseinfo.properties");
        Properties properties = new Properties();
        try {
            properties.load(resourceAsStream);
            setExtensionName(properties.getProperty("psp.extension.tracking.name"));
            setReleaseDate(properties.getProperty("psp.extension.release.date"));
            setTrackingDetailsLength(properties.getProperty("psp.extension.tracking.key.length"));
        } catch (IOException e) {
            logError();
        }
    }

    public String fetchTrackingDetails() {
        String trackingKey = null;

        if (StringUtils.isNotEmpty(getExtensionName()) && StringUtils.isNotEmpty(getReleaseDate())
                && StringUtils.isNotEmpty(getTrackingDetailsLength())) {
            try {
                // Just to make sure that an error is logged if release date is invalid
                int trackingMaxLength = Integer.parseInt(getTrackingDetailsLength());

                StringBuilder trackingDetailsBuilder = new StringBuilder();
                trackingDetailsBuilder.append(getExtensionName());
                trackingDetailsBuilder.append(getReleaseDate());

                if (trackingDetailsBuilder.length() <= trackingMaxLength) {
                    trackingKey = trackingDetailsBuilder.toString();
                    LOGGER.debug("Tracking key: " + trackingKey);
                } else {
                    logError();
                }
            } catch (NumberFormatException exception) {
                logError();
            }
        } else {
            logError();
        }

        return trackingKey;
    }

    private void logError() {
        LOGGER.error("Cannot parse psp extension configuration. Contact support team to resolve this issue.");
    }

    public String getTrackingDetailsLength() {
        return trackingDetailsLength;
    }

    public void setTrackingDetailsLength(String trackingDetailsLength) {
        this.trackingDetailsLength = trackingDetailsLength;
    }

    public String getReleaseDate() {
        return releaseDate;
    }

    public void setReleaseDate(String releaseDate) {
        this.releaseDate = releaseDate;
    }

    public String getExtensionName() {
        return extensionName;
    }

    public void setExtensionName(String extensionName) {
        this.extensionName = extensionName;
    }
}
