package com.psp.payment.directlink.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.directlink.dto.PspAuthorizationResult;
import com.psp.payment.directlink.response.PspDirectLinkResponse;
import com.psp.payment.directlink.services.PspDirectLinkService;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import de.hybris.platform.converters.Populator;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Currency;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/**
 * Populator for ootb style result from the directlink response.
 */
public class PspAuthorizationResultPopulator implements Populator<PspDirectLinkResponse, PspAuthorizationResult> {

    @Resource
    private PspDirectLinkService pspDirectLinkService;

    @Override
    public void populate(PspDirectLinkResponse source, PspAuthorizationResult target) {
        validateParameterNotNull(source, "Parameter [DirectlinkResponse] source cannot be null");
        validateParameterNotNull(target, "Parameter [PspAuthorizationResult] target cannot be null");

        target.setOrderId(source.getParameters().get(PspConstants.KEY_DL_ORDERID));

        String amount = source.getParameters().get(PspConstants.KEY_RESULT_AMOUNT);

        if (StringUtils.isEmpty(amount)) {
            amount = "0.0";
        }

        target.setTotalAmount(new BigDecimal(amount));

        String currency = source.getParameters().get(PspConstants.KEY_CURRENCY);

        if (StringUtils.isNotEmpty(currency)) {
            target.setCurrency(Currency.getInstance(currency));
        }

        // TODO: is this correct mapping?
        target.setRequestId(source.getParameters().get(PspConstants.KEY_PAYID));

        target.setAuthorizationCode(source.getParameters().get(PspConstants.KEY_RESULT_ACCEPTANCE));
        target.setRequestToken(source.getParameters().get(PspConstants.KEY_PAYID));
        target.setBrand(source.getParameters().get(PspConstants.KEY_RESULT_BRAND));
        target.setPm(source.getParameters().get(PspConstants.KEY_RESULT_PAYMENTMETHOD));
        String eciCode = source.getParameters().get(PspConstants.KEY_RESULT_ECI);

        if (StringUtils.isNotEmpty(eciCode)) {
            target.setEci(eciCode);
        }

        target.setNcError(source.getParameters().get(PspConstants.KEY_NCERROR));
        target.setNcErrorPlus(source.getParameters().get(PspConstants.KEY_NCERRORPLUS));
        target.setNcStatus(source.getParameters().get(PspConstants.KEY_RESULT_NCSTATUS));
        target.setPspTransactionStatus(pspDirectLinkService.getTransactionStatus(source.getParameters()));
        target.setTransactionStatusDetails(pspDirectLinkService.getTransactionStatusDetails(source.getParameters()));

        String status = source.getParameters().get(PspConstants.KEY_STATUS);
        target.setStatus(status);
        int statusAsInt = Integer.valueOf(status);

        if (statusAsInt == PspPaymentStatus.STATUS_IDENTIFICATION_REQUESTED) {
            target.setHtmlAnswer(source.getParameters().get(PspConstants.KEY_HTML_ANSWER));
        }
    }
}
