package com.psp.payment.directlink.request;

import com.psp.constants.PspConstants;
import org.apache.commons.lang.StringUtils;

import java.util.Map;
import java.util.TreeMap;

public class PspDirectLinkRequest {

    private Map<String, String> parameters;

    public PspDirectLinkRequest() {
        parameters = new TreeMap<>();
    }

    public void setCardNumber(String cardNumber) {
        setParameter(PspConstants.KEY_CARDNO, cardNumber);
    }

    public void setExpiryDate(int expiryMonth, int expiryYear) {
        setParameter(PspConstants.KEY_EXPIRY_DATE, expiryMonth + "/" + expiryYear);
    }

    public void setCv2(String cv2) {
        setParameter(PspConstants.KEY_CVC, cv2);
    }

    public void setFlag3d(String flag3d) {
        setParameter(PspConstants.KEY_FLAG3D, flag3d);
    }

    public void setHttpAccept(String httpAccept) {
        setParameter(PspConstants.KEY_HTTP_ACCEPT, httpAccept);
    }

    public void setHttpUserAgent(String httpUserAgent) {
        setParameter(PspConstants.KEY_HTTP_USER_AGENT, httpUserAgent);
    }

    public void setWin3ds(String win3ds) {
        setParameter(PspConstants.KEY_WIN3DS, win3ds);
    }

    public void setAcceptUrl(String acceptUrl) {
        setParameter(PspConstants.KEY_ACCEPTURL, acceptUrl);
    }

    public void setDeclineUrl(String declineUrl) {
        setParameter(PspConstants.KEY_DECLINEURL, declineUrl);
    }

    public void setExceptionUrl(String exceptionUrl) {
        setParameter(PspConstants.KEY_EXCEPTIONURL, exceptionUrl);
    }

    public void setParamPlus(String paramPlus) {
        setParameter(PspConstants.KEY_PARAMPLUS, paramPlus);
    }

    public void setComPlus(String comPlus) {
        setParameter(PspConstants.KEY_COMPLUS, comPlus);
    }

    public void setLanguage(String language) {
        setParameter(PspConstants.KEY_LANGUAGE, language);
    }

    public void setBrowserAcceptHeader(String browserAcceptHeader) {
        setParameter(PspConstants.BROWSER_ACCEPT_HEADER, browserAcceptHeader);
    }

    public void setBrowserColorDepth(String browserColorDepth) {
        setParameter(PspConstants.BROWSER_COLOR_DEPTH, browserColorDepth);
    }

    public void setBrowserJavaEnabled(String browserJavaEnabled) {
        setParameter(PspConstants.BROWSER_JAVA_ENABLED, browserJavaEnabled);
    }

    public void setBrowserLanguage(String browserLanguage) {
        setParameter(PspConstants.BROWSER_LANGUAGE, browserLanguage);
    }

    public void setBrowserScreenHeight(String browserScreenHeight) {
        setParameter(PspConstants.BROWSER_SCREEN_HEIGHT, browserScreenHeight);
    }

    public void setBrowserScreenWidth(String browserScreenWidth) {
        setParameter(PspConstants.BROWSER_SCREEN_WIDTH, browserScreenWidth);
    }

    public void setBrowserTimeZone(String browserTimeZone) {
        setParameter(PspConstants.BROWSER_TIME_ZONE, browserTimeZone);
    }

    public void setBrowserUserAgent(String browserUserAgent) {
        setParameter(PspConstants.BROWSER_USER_AGENT, browserUserAgent);
    }

    public void setChallengeWindowSize(String challengeWindowSize) {
        setParameter(PspConstants.CHALLENGE_WINDOW_SIZE, challengeWindowSize);
    }

    public void setRemoteAddr(String remoteAddr) {
        setParameter(PspConstants.REMOTE_ADDR, remoteAddr);
    }

    /**
     * setter for order id, validates for length and spl chars
     *
     * @param orderId
     */
    public void setOrderId(String orderId) {
        if (orderId.length() > 40) {
            throw new IllegalArgumentException("Orderid cannot be longer than 30 characters");
        }

        if (!orderId.matches("^[a-zA-Z0-9_-]+")) {
            throw new IllegalArgumentException("Order id cannot contain special characters");
        }

        setParameter(PspConstants.KEY_DL_ORDERID, orderId);
    }

    /**
     * setter for amount. Validates min and max
     *
     * @param amount
     */
    public void setAmount(Integer amount) {

        if (amount == null || amount < 0) {
            throw new IllegalArgumentException("Amount must be a positive number");
        }
        setParameter(PspConstants.KEY_AMOUNT, String.valueOf(amount));
    }

    public void setCurrency(String isoCode) {
        setParameter(PspConstants.KEY_CURRENCY, isoCode);
    }

    public void setAlias(String alias) {
        setParameter(PspConstants.KEY_ALIAS, alias);
    }

    /**
     * setter for psp id, validates for length
     *
     * @param pspid
     */
    public void setPspId(String pspid) {
        if (StringUtils.isEmpty(pspid) || pspid.length() > 30) {
            throw new IllegalArgumentException("PSPID is either null or too long");
        }

        setParameter(PspConstants.KEY_PSPID, pspid);
    }

    /**
     * setter for user id, validates the length
     *
     * @param userId
     */
    public void setUserId(String userId) {
        if (StringUtils.isEmpty(userId) || userId.length() > 20 || userId.length() < 2) {
            throw new IllegalArgumentException("Incorrect UserId ");
        }

        setParameter(PspConstants.KEY_API_USERID, userId);
    }

    public void setPassword(String password) {
        setParameter(PspConstants.KEY_API_PASSWORD, password);
    }

    public void setOperation(String operation) {
        setParameter(PspConstants.KEY_OPERATION, operation);
    }

    public void setEci(String eci) {
        setParameter(PspConstants.KEY_ECI, eci);
    }

    public void setCustomerName(String customerName) {
        setParameter(PspConstants.KEY_CN, customerName);
    }

    public void setEmail(String email) {
        setParameter(PspConstants.KEY_EMAIL, email);
    }

    public void setAddress(String address) {
        setParameter(PspConstants.KEY_OWNER_ADDRESS, address);
    }

    public void setPostCode(String postCode) {
        setParameter(PspConstants.KEY_OWNER_POSTCODE, postCode);
    }

    public void setTown(String town) {
        setParameter(PspConstants.KEY_OWNER_TOWN, town);
    }

    public void setCountry(String country) {
        setParameter(PspConstants.KEY_OWNER_COUNTRY, country);
    }

    public void setPhoneNumber(String phoneNumber) {
        setParameter(PspConstants.KEY_OWNER_TELNO, phoneNumber);
    }

    private void setParameter(String parameter, String value) {
        parameters.put(parameter, value);
    }

    public Map<String, String> getParameters() {
        return parameters;
    }

    public void setParameters(Map<String, String> parameters) {
        this.parameters = parameters;
    }
}

