package com.psp.payment.directquery.services.impl;

import com.psp.payment.common.dto.PspTransactionResponse;
import com.psp.payment.directquery.services.PspDirectQueryService;
import com.psp.payment.integration.clients.PspRestClient;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import javax.annotation.Resource;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.Optional;

public class PspDirectQueryServiceImpl implements PspDirectQueryService {
    private static final Logger LOG = LogManager.getLogger("PspDirectQueryServiceImpl");
    private static final String KEY = "psp.conf.directquery.request.url";

    @Resource
    private Converter<String, Map<String, String>> pspDirectQueryConverter;

    @Resource
    private PspRestClient<PspTransactionResponse> pspRestClient;

    @Resource
    private ConfigurationService configurationService;

    @Override
    public Optional<PspTransactionResponse> readPaymentStatus(String payId) {
        try {
            Map<String, String> request = pspDirectQueryConverter.convert(payId);
            return Optional.ofNullable(pspRestClient.postRequest(readUri(), request, PspTransactionResponse.class));
        } catch (Exception e) {
            LOG.error("Failure to obtain Payment Status. Cause: " + KEY, e);
            return Optional.empty();
        }
    }

    private URI readUri() throws URISyntaxException {
        return new URI(configurationService.getConfiguration().getString(KEY));
    }
}
