package com.psp.payment.flexpayments.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.services.PspDynamicPaymentTemplateService;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.directlink.dto.Psp3DSecureDTO;
import com.psp.payment.directlink.strategies.PspAddressBuilderStrategy;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import com.psp.payment.flexpayments.strategies.PspCreateFlexPaymentRequestStrategy;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.commerceservices.i18n.CommerceCommonI18NService;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.commerceservices.strategies.CustomerNameStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.session.SessionService;
import de.hybris.platform.site.BaseSiteService;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Map;
import java.util.Optional;
import java.util.TreeMap;

import static com.psp.constants.PspConstants.*;
import static java.lang.Boolean.TRUE;

public class PspCreateFlexPaymentRequestStrategyImpl implements PspCreateFlexPaymentRequestStrategy {

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private PspUtils pspUtils;

    @Resource
    private CustomerNameStrategy customerNameStrategy;

    @Resource
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Resource
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Resource
    private CommerceCommonI18NService commerceCommonI18NService;

    @Resource
    private PspAddressBuilderStrategy pspAddressBuilderStrategy;

    @Resource
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Resource
    private PspDynamicPaymentTemplateService pspDynamicPaymentTemplateService;

    @Resource
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource
    private BaseSiteService baseSiteService;

    @Resource
    private SessionService sessionService;

    @Override
    public PspFlexPaymentRequest createFlexPaymentRequest(CartModel cartModel) {
        PspFlexPaymentRequest request = new PspFlexPaymentRequest();

        if (cartModel != null) {
            BaseSiteModel currentBaseSite = baseSiteService.getCurrentBaseSite();
            Configuration configuration = configurationService.getConfiguration();
            String acceptUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_FLEX_ACCEPT_URL));
            String exceptionUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_FLEX_EXCEPTION_URL));
            String cancelUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_FLEX_CANCEL_URL));
            String declineUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_FLEX_DECLINE_URL));
            String backUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, configuration.getString(PARAM_FLEX_BACK_URL));

            request.setPspId(getParameter(PspConstants.PARAM_PSPID));
            request.setOrderId(pspUtils.generateUniqueOrderId(cartModel));

            String paymentMethod = cartModel.getPaymentMode().getPaymentMethod();

            request.setPaymentMethod(paymentMethod);

            if (!StringUtils.equalsIgnoreCase(CREDIT_CARD, paymentMethod)) {
                request.setBrand(cartModel.getPaymentMode().getBrand());
            }

            request.setAcceptUrl(acceptUrl);
            request.setExceptionUrl(exceptionUrl);
            request.setCancelUrl(cancelUrl);
            request.setDeclineUrl(declineUrl);
            request.setBackUrl(backUrl);
            request.setLanguage(commerceCommonI18NService.getCurrentLanguage().getIsocode());
            request.setCurrency(commerceCommonI18NService.getCurrentCurrency().getIsocode());
            setAddressFields(request, cartModel);
            request.setAmount(getAmount(cartModel));
            request.setTrackingKey(pspExtensionTrackingUtils.fetchTrackingDetails());

            Optional<String> optionalTemplate = pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl();
            optionalTemplate.ifPresent(request::setTemplate);

            setSignature(request);
        }

        return request;
    }

    private void setSignature(PspFlexPaymentRequest request) {
        Map<String, String> parameters = new TreeMap<>();
        parameters.put(PspConstants.KEY_FLEX_PSPID, request.getPspId());
        parameters.put(PspConstants.KEY_FLEX_ORDERID, request.getOrderId());
        parameters.put(PspConstants.KEY_FLEX_PAYMENT_METHOD, request.getPaymentMethod());
        parameters.put(PspConstants.KEY_RESULT_BRAND, request.getBrand());
        parameters.put(PspConstants.KEY_ACCEPTURL, request.getAcceptUrl());
        parameters.put(PspConstants.KEY_EXCEPTIONURL, request.getExceptionUrl());
        parameters.put(PspConstants.KEY_DECLINEURL, request.getDeclineUrl());
        parameters.put(PspConstants.KEY_CANCELURL, request.getDeclineUrl());
        parameters.put(PspConstants.KEY_BACKURL, request.getBackUrl());
        parameters.put(PspConstants.KEY_LANGUAGE, request.getLanguage());
        parameters.put(PspConstants.KEY_CURRENCY, request.getCurrency());
        parameters.put(PspConstants.KEY_AMOUNT, request.getAmount().toString());
        parameters.put(PspConstants.KEY_CN, request.getCustomerName());
        parameters.put(PspConstants.KEY_OWNER_ADDRESS, request.getOwnerAddress());
        parameters.put(PspConstants.KEY_OWNER_COUNTRY, request.getOwnerCountry());
        parameters.put(PspConstants.KEY_OWNER_POSTCODE, request.getOwnerZip());
        parameters.put(PspConstants.KEY_OWNER_TELNO, request.getOwnerTelephoneNumber());
        parameters.put(PspConstants.KEY_OWNER_TOWN, request.getOwnerTown());
        parameters.put(PspConstants.KEY_EMAIL, request.getEmail());
        parameters.put(PspConstants.KEY_OPERATION, PspOperation.SAL.name());
        parameters.put(PspConstants.KEY_TRACKING, request.getTrackingKey());
        parameters.put(KEY_TEMPLATE, request.getTemplate());

        String shaSignature = pspUtils.generateChecksum(parameters, getParameter(PspConstants.PARAM_PASSPHRASE));
        request.setShaSign(shaSignature);
    }

    private void setAddressFields(PspFlexPaymentRequest request, CartModel cartModel) {
        AddressModel shippingAddress = cartModel.getDeliveryAddress();
        //Address details of the customer
        if (shippingAddress != null) {
            request.setCustomerName(customerNameStrategy.getName(shippingAddress.getFirstname(), shippingAddress.getLastname()));

            String email = shippingAddress.getEmail();

            if (StringUtils.isEmpty(email)) {
                email = customerEmailResolutionService.getEmailForCustomer(checkoutCustomerStrategy.getCurrentUserForCheckout());
            }

            request.setEmail(email);
            request.setOwnerAddress(pspAddressBuilderStrategy.buildAddress(shippingAddress.getStreetname(), shippingAddress.getStreetnumber()));
            request.setOwnerTown(shippingAddress.getTown());
            request.setOwnerTelephoneNumber(shippingAddress.getPhone1());
            if (shippingAddress.getCountry() != null) {
                request.setOwnerCountry(shippingAddress.getCountry().getIsocode());
            }
            request.setOwnerZip(shippingAddress.getPostalcode());
        }
    }

    private BigDecimal getAmount(CartModel cartModel) {
        final Double totalPrice = cartModel.getTotalPrice();
        final Double totalTax = (cartModel.getNet() && cartModel.getStore() != null && cartModel.getStore()
                .getExternalTaxEnabled()) ? cartModel.getTotalTax() : Double.valueOf(0d);
        final BigDecimal totalPriceWithoutTaxBD = new BigDecimal(totalPrice == null ? 0d : totalPrice).setScale(2,
                RoundingMode.HALF_EVEN);
        final BigDecimal totalPriceBD = new BigDecimal(totalTax == null ? 0d : totalTax).setScale(2,
                RoundingMode.HALF_EVEN).add(totalPriceWithoutTaxBD);

        return totalPriceBD.scaleByPowerOfTen(2);
    }

    public String getParameter(String parameterName) {
        return configurationService.getConfiguration().getString(parameterName);
    }
}
