package com.psp.payment.fraud.impl;

import com.psp.payment.fraud.PspFraudService;
import de.hybris.platform.basecommerce.enums.FraudStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.fraud.impl.DefaultFraudService;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.fraud.impl.FraudSymptom;
import de.hybris.platform.fraud.model.FraudReportModel;
import de.hybris.platform.fraud.model.FraudSymptomScoringModel;
import de.hybris.platform.orderhistory.model.OrderHistoryEntryModel;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.time.TimeService;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

public class PspFraudServiceImpl extends DefaultFraudService implements PspFraudService {
    @Resource
    private ModelService modelService;

    @Resource
    private TimeService timeService;

    @Override
    public FraudReportModel createFraudReport(String providerName, FraudServiceResponse response, OrderModel order, FraudStatus status) {
        final FraudReportModel fraudReport = createFraudReport(providerName, order, status);
        int reportNumber = 0;
        if (order.getFraudReports() != null && !order.getFraudReports().isEmpty()) {
            reportNumber = order.getFraudReports().size();
        }
        fraudReport.setCode(order.getCode() + "_FR" + reportNumber);
        List<FraudSymptomScoringModel> symptoms = null;
        for (final FraudSymptom symptom : response.getSymptoms()) {
            if (symptoms == null) {
                symptoms = new ArrayList<FraudSymptomScoringModel>();
            }
            FraudSymptomScoringModel symptomScoring = createSymptomScoringItem(fraudReport,symptom);
            symptoms.add(symptomScoring);
        }
        fraudReport.setFraudSymptomScorings(symptoms);
        return fraudReport;
    }

    private FraudSymptomScoringModel createSymptomScoringItem(FraudReportModel fraudReport, FraudSymptom symptom) {
        final FraudSymptomScoringModel symptomScoring = modelService.create(FraudSymptomScoringModel.class);
        symptomScoring.setFraudReport(fraudReport);
        symptomScoring.setName(symptom.getSymptom());
        symptomScoring.setExplanation(symptom.getExplanation());
        symptomScoring.setScore(symptom.getScore());

        return symptomScoring;
    }

    private FraudReportModel createFraudReport(String providerName, OrderModel order, FraudStatus status) {
        final FraudReportModel fraudReport = modelService.create(FraudReportModel.class);
        fraudReport.setOrder(order);
        fraudReport.setStatus(status);
        fraudReport.setProvider(providerName);
        fraudReport.setTimestamp(timeService.getCurrentTime());
        return fraudReport;
    }

    @Override
    public OrderHistoryEntryModel createHistoryLog(String providerName, OrderModel order, FraudStatus status, String code) {
        final String description;
        if (status.equals(FraudStatus.OK)) {
            description = "Fraud check [" + providerName + "]: OK";
        } else {
            description = "Fraud check [" + providerName + "]: " + status.toString() + ". Check the fraud report :" + code;
        }
        OrderHistoryEntryModel historyEntry = modelService.create(OrderHistoryEntryModel.class);
        historyEntry.setTimestamp(timeService.getCurrentTime());
        historyEntry.setOrder(order);
        historyEntry.setDescription(description);
        return historyEntry;
    }
}
