package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import de.hybris.platform.core.enums.OrderStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.processengine.action.AbstractSimpleDecisionAction;

import javax.annotation.Resource;


/**
 * This action implements payment authorization/capture.
 */
public class PspCheckAuthorizeOrderPaymentAction extends AbstractSimpleDecisionAction<OrderProcessModel> {

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Override
    public Transition executeAction(final OrderProcessModel process) {
        final OrderModel order = process.getOrder();

        if (order != null && order.getPaymentInfo() != null) {

            for (final PaymentTransactionModel transaction : order.getPaymentTransactions()) {
                for (final PaymentTransactionEntryModel entry : transaction.getEntries()) {
                    if (pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(order.getPaymentInfo(), order.getSite()).equals(entry.getType())
                            && (TransactionStatus.ACCEPTED.name().equals(entry.getTransactionStatus())
                            || TransactionStatus.REVIEW.name().equals(entry.getTransactionStatus()))
                            || PspTransactionStatus.UNCERTAIN.name().equals(entry.getTransactionStatus())) {
                        order.setStatus(OrderStatus.PAYMENT_AUTHORIZED);
                        modelService.save(order);
                        return Transition.OK;
                    }
                }
            }
        }

        return Transition.NOK;
    }

}
