package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.enums.PspTransactionStatus;
import com.psp.payment.common.services.PspPaymentProcessService;
import de.hybris.platform.core.enums.OrderStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.processengine.action.AbstractAction;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class PspCheckTransactionReviewStatusAction extends AbstractAction<OrderProcessModel> {

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Override
    public final String execute(final OrderProcessModel process) throws Exception {
        return executeAction(process).toString();
    }

    protected Transition executeAction(final OrderProcessModel process) {
        Transition result = Transition.OK;
        final OrderModel order = process.getOrder();

        if (order != null && order.getPaymentInfo() != null) {
            for (final PaymentTransactionModel transaction : order.getPaymentTransactions()) {
                result = checkPaymentTransaction(transaction, order);

                if (!Transition.OK.equals(result)) {
                    break;
                }
            }
        }

        return result;
    }

    protected Transition checkPaymentTransaction(final PaymentTransactionModel transaction, final OrderModel orderModel) {

        final List<PaymentTransactionEntryModel> transactionEntries = transaction.getEntries();

        for (int index = transactionEntries.size() - 1; index >= 0; index--) {
            final PaymentTransactionEntryModel entry = transactionEntries.get(index);

            if (pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(orderModel.getPaymentInfo(), orderModel.getSite()).equals(entry.getType())) {
                String transactionStatus = entry.getTransactionStatus();
                if (PspTransactionStatus.UNCERTAIN.name().equals(transactionStatus)) {
                    orderModel.setStatus(OrderStatus.SUSPENDED);
                    getModelService().save(orderModel);
                    return Transition.WAIT;
                } else if (PspTransactionStatus.ACCEPTED.name().equals(transactionStatus)
                        || PspTransactionStatus.REVIEW.name().equals(transactionStatus)) {
                    return Transition.OK;
                } else {
                    return Transition.NOK;
                }
            }
        }

        return Transition.OK;
    }


    public enum Transition {
        OK, NOK, WAIT;

        public static Set<String> getStringValues() {
            final Set<String> res = new HashSet<>();
            for (final Transition transitions : Transition.values()) {
                res.add(transitions.toString());
            }
            return res;
        }
    }

    @Override
    public Set<String> getTransitions() {
        return Transition.getStringValues();
    }
}
