package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.services.PspMaintenanceService;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.processengine.action.AbstractSimpleDecisionAction;
import org.apache.log4j.Logger;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.net.URISyntaxException;

import static de.hybris.platform.core.enums.OrderStatus.PAYMENT_CAPTURED;
import static de.hybris.platform.core.enums.OrderStatus.PAYMENT_NOT_CAPTURED;
import static de.hybris.platform.payment.enums.PaymentTransactionType.AUTHORIZATION;
import static de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition.NOK;
import static de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition.OK;

public class PspTakePaymentAction extends AbstractSimpleDecisionAction<OrderProcessModel> {

    public static final Logger LOGGER = Logger.getLogger(PspTakePaymentAction.class);

    @Resource
    private PspMaintenanceService pspMaintenanceService;

    @Resource
    private PspPaymentProcessService pspPaymentProcessService;

    @Override
    public Transition executeAction(OrderProcessModel orderProcessModel) throws Exception {
        Transition transition = OK;
        final OrderModel orderModel = orderProcessModel.getOrder();
        PaymentTransactionType paymentTransactionType = pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(orderModel.getPaymentInfo(), orderModel.getSite());

        if (AUTHORIZATION.equals(paymentTransactionType)) {
            try {
                pspMaintenanceService.captureFunds(orderModel, BigDecimal.valueOf(orderModel.getTotalPrice()));
                setOrderStatus(orderModel, PAYMENT_CAPTURED);
                transition = OK;
            } catch (IllegalArgumentException | IllegalMaintenanceOperation | URISyntaxException exception) {
                LOGGER.error("Cannot capture funds for order: " + orderModel, exception);
                setOrderStatus(orderModel, PAYMENT_NOT_CAPTURED);
                transition = NOK;
            }
        } else {
            setOrderStatus(orderModel, PAYMENT_CAPTURED);
        }

        return transition;
    }
}
