package com.psp.payment.integration.clients;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestOperations;

import javax.annotation.Resource;
import java.net.URI;
import java.util.Map;
import java.util.Set;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspRestClient<T> {

    @Resource
    private RestOperations restTemplate;

    /**
     * Makes a RESTful POST request with encoded parameters.
     *
     * @param uri                the URL for the POST request
     * @param parameters         the parameters to send
     * @param expectedReturnType the expected return type
     * @return response from the POST request
     */
    public T postRequest(URI uri, Map<String, String> parameters, Class<T> expectedReturnType) {
        validateParameterNotNull(uri, "Parameter [uri] cannot be null");
        validateParameterNotNull(parameters, "Parameter [parameters] cannot be null");
        validateParameterNotNull(expectedReturnType, "Parameter [expectedReturnType] cannot be null");

        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

        MultiValueMap<String, String> multiValueMap = createMultiValueMap(parameters);
        HttpEntity<MultiValueMap<String, String>> httpEntity = new HttpEntity<>(multiValueMap, httpHeaders);

        return restTemplate.postForObject(uri, httpEntity, expectedReturnType);
    }

    private MultiValueMap<String, String> createMultiValueMap(Map<String, String> map) {
        MultiValueMap<String, String> multiValueMap = new LinkedMultiValueMap<>();
        Set<String> keys = map.keySet();

        if (CollectionUtils.isNotEmpty(keys)) {
            keys.forEach((key) -> multiValueMap.add(key, map.get(key)));
        }

        return multiValueMap;
    }
}
