package com.psp.payment.integration.utils;

import com.psp.payment.directlink.exception.XmlParseException;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.io.StringReader;
import java.util.Map;
import java.util.TreeMap;

public class PspIntegrationUtils {

    private static final Logger LOGGER = Logger.getLogger(PspIntegrationUtils.class);

    public Map<String, String> convertPlainStringToParameters(String response) {
        Map<String, String> resultParameters = new TreeMap<>();
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        InputSource inputSource = new InputSource(new StringReader(response));

        try {
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.parse(inputSource);
            Node ncResponseNode = document.getFirstChild();
            NamedNodeMap attributes = ncResponseNode.getAttributes();

            for (int i = 0; i < attributes.getLength(); i++) {
                Attr attr = (Attr) attributes.item(i);
                resultParameters.put(StringUtils.upperCase(attr.getName()), attr.getValue());
            }
            //get text content of the child if exists
            Node htmlAnswer = ncResponseNode.getFirstChild();
            if (htmlAnswer != null && StringUtils.isNotEmpty(htmlAnswer.getTextContent())) {
                resultParameters.put(StringUtils.upperCase(htmlAnswer.getNodeName()), htmlAnswer.getTextContent());
            }
            logUsefulData(resultParameters);
        } catch (SAXException | IOException | ParserConfigurationException exception) {
            throw new XmlParseException(exception.getMessage(), exception);
        }

        return resultParameters;
    }

    private void logUsefulData(Map<String, String> resultParameters) {
        LOGGER.info("***Server-to-server Response Parameters***");

        if (resultParameters.containsKey("ORDERID")) {
            LOGGER.info("Order Id: " + resultParameters.get("ORDERID"));
        }

        if (resultParameters.containsKey("NCSTATUS")) {
            LOGGER.info("NCSTATUS: " + resultParameters.get("NCSTATUS"));
        }

        if (resultParameters.containsKey("NCERROR")) {
            LOGGER.info("NCERROR: " + resultParameters.get("NCERROR"));
        }

        if (resultParameters.containsKey("STATUS")) {
            LOGGER.info("STATUS: " + resultParameters.get("STATUS"));
        }

        if (resultParameters.containsKey("SCO_CATEGORY")) {
            LOGGER.info("SCO_CATEGORY: " + resultParameters.get("SCO_CATEGORY"));
        }
    }
}
