package com.psp.payment.maintenance.converters.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspOperation;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import de.hybris.platform.converters.Populator;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.util.Map;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspMaintenanceRequestPopulator implements Populator<PspMaintenanceRequest, Map<String, String>> {

    @Resource
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Override
    public void populate(PspMaintenanceRequest source, Map<String, String> target) {
        validateParameterNotNull(source, "Parameter [source] cannot be null");
        validateParameterNotNull(target, "Parameter [target] cannot be null");

        addParameter(target, PspConstants.KEY_PSPID, source.getPspId());
        addParameter(target, PspConstants.KEY_API_USERID, source.getUserId());
        addParameter(target, PspConstants.KEY_API_PASSWORD, source.getPassword());
        addParameter(target, PspConstants.KEY_PAYID, source.getPayId());
        addParameter(target, PspConstants.KEY_AMOUNT, source.getAmount());
        addParameter(target, PspConstants.KEY_TRACKING, pspExtensionTrackingUtils.fetchTrackingDetails());
        PspOperation operation = source.getOperation();

        if (operation != null) {
            addParameter(target, PspConstants.KEY_OPERATION, operation.name());
        }
    }

    private void addParameter(Map<String, String> target, String key, String value) {
        if (StringUtils.isNotEmpty(value)) {
            target.put(key, value);
        }
    }
}
