package com.psp.payment.maintenance.services.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.integration.clients.PspRestClient;
import com.psp.payment.integration.utils.PspIntegrationUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import com.psp.payment.maintenance.services.PspMaintenanceService;
import com.psp.payment.maintenance.strategies.PspMaintenanceRequestCreationStrategy;
import com.psp.payment.maintenance.strategies.PspMaintenanceResponseParsingStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.dto.converter.Converter;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.TreeMap;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspMaintenanceServiceImpl implements PspMaintenanceService {

    @Resource
    private PspMaintenanceRequestCreationStrategy pspMaintenanceRequestCreationStrategy;

    @Resource
    private Converter<PspMaintenanceRequest, Map<String, String>> pspMaintenanceRequestConverter;

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private PspUtils pspUtils;

    @Resource
    private PspRestClient<String> pspRestClient;

    @Resource
    private PspIntegrationUtils pspIntegrationUtils;

    @Resource
    private Converter<Map<String, String>, PspMaintenanceResponse> pspMaintenanceResponseConverter;

    @Resource
    private PspMaintenanceResponseParsingStrategy pspMaintenanceResponseParsingStrategy;

    @Override
    public PaymentTransactionEntryModel captureFunds(OrderModel orderModel, BigDecimal proposedAmount)
            throws URISyntaxException {
        return processMaintenanceRequest(PspMaintenanceRequestType.CAPTURE, orderModel, proposedAmount);
    }

    @Override
    public PaymentTransactionEntryModel refundFunds(OrderModel orderModel, BigDecimal proposedAmount)
            throws URISyntaxException {
        return processMaintenanceRequest(PspMaintenanceRequestType.REFUND, orderModel, proposedAmount);
    }

    private PaymentTransactionEntryModel processMaintenanceRequest(PspMaintenanceRequestType pspMaintenanceRequestType,
                                                                   OrderModel orderModel,
                                                                   BigDecimal proposedAmount) throws URISyntaxException {
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null");
        validateParameterNotNull(proposedAmount, "Parameter [proposedAmount] cannot be null");

        PspMaintenanceRequest pspMaintenanceRequest = pspMaintenanceRequestCreationStrategy.createMaintenanceRequest(pspMaintenanceRequestType,
                orderModel, proposedAmount);

        Map<String, String> requestParameters = new TreeMap<>();
        pspMaintenanceRequestConverter.convert(pspMaintenanceRequest, requestParameters);

        String shaSignature = pspUtils.generateChecksum(requestParameters,
                configurationService.getConfiguration().getString(PspConstants.PARAM_PASSPHRASE));
        requestParameters.put(PspConstants.KEY_SHASIGN, shaSignature);

        String uri = configurationService.getConfiguration().getString(PspConstants.PARAM_MAINTENANCE_URL);
        String response = pspRestClient.postRequest(new URI(uri), requestParameters, String.class);
        Map<String, String> responseParameters = pspIntegrationUtils.convertPlainStringToParameters(response);

        PspMaintenanceResponse pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse = pspMaintenanceResponseConverter.convert(responseParameters, pspMaintenanceResponse);

        return pspMaintenanceResponseParsingStrategy.parseResponse(pspMaintenanceRequestType, orderModel,
                pspMaintenanceResponse);
    }
}
