package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.strategies.PspGenerateMerchantTransactionCodeStrategy;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import com.psp.payment.maintenance.strategies.PspMaintenancePaymentTransactionCreationStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.collections.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspMaintenancePaymentTransactionCreationStrategyImpl implements PspMaintenancePaymentTransactionCreationStrategy {

    @Resource
    private ModelService modelService;

    @Resource
    private CommonI18NService commonI18NService;

    @Resource
    private PspGenerateMerchantTransactionCodeStrategy pspGenerateMerchantTransactionCodeStrategy;

    @Resource
    private PaymentService paymentService;

    @Override
    public PaymentTransactionEntryModel createPaymentTransaction(PaymentTransactionType paymentTransactionType,
                                                                 OrderModel orderModel,
                                                                 PspMaintenanceResponse pspMaintenanceResponse) {
        validateParameterNotNull(paymentTransactionType, "Parameter [paymentTransactionType] cannot be null");
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null");
        validateParameterNotNull(pspMaintenanceResponse, "Parameter [pspMaintenanceResponse] cannot be null");

        PaymentTransactionEntryModel paymentTransactionEntryModel = null;
        List<PaymentTransactionModel> paymentTransactions = orderModel.getPaymentTransactions();

        if (CollectionUtils.isNotEmpty(paymentTransactions)) {
            PaymentTransactionModel paymentTransactionModel = paymentTransactions.get(0);
            paymentTransactionEntryModel = createPaymentTransactionEntry(paymentTransactionType,
                    paymentTransactionModel, pspMaintenanceResponse);
            paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
            modelService.saveAll(paymentTransactionModel, paymentTransactionEntryModel);
        }

        return paymentTransactionEntryModel;
    }

    private PaymentTransactionEntryModel createPaymentTransactionEntry(PaymentTransactionType paymentTransactionType,
                                                                       PaymentTransactionModel paymentTransactionModel,
                                                                       PspMaintenanceResponse pspMaintenanceResponse) {
        PaymentTransactionEntryModel paymentTransactionEntryModel = modelService.create(PaymentTransactionEntryModel.class);
        String currency = pspMaintenanceResponse.getCurrency();
        paymentTransactionEntryModel.setCurrency(commonI18NService.getCurrency(currency));
        paymentTransactionEntryModel.setType(paymentTransactionType);

        String payId = pspMaintenanceResponse.getPayId();
        paymentTransactionEntryModel.setRequestToken(payId);
        paymentTransactionEntryModel.setRequestId(payId);
        paymentTransactionEntryModel.setSubscriptionID(payId);
        paymentTransactionEntryModel.setAmount(new BigDecimal(pspMaintenanceResponse.getAmount()));
        paymentTransactionEntryModel.setCode(paymentService.getNewPaymentTransactionEntryCode(paymentTransactionModel,
                paymentTransactionType));
        paymentTransactionEntryModel.setTime(new Date());
        paymentTransactionEntryModel.setTransactionStatus(TransactionStatus.ACCEPTED.name());
        paymentTransactionEntryModel.setTransactionStatusDetails(TransactionStatusDetails.SUCCESFULL.name());

        return paymentTransactionEntryModel;
    }
}
