package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.enums.PspMaintenanceRequestType;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.strategies.PspMaintenancePaymentTransactionCreationStrategy;
import com.psp.payment.maintenance.strategies.PspMaintenanceResponseParsingStrategy;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspMaintenanceResponseParsingStrategyImpl implements PspMaintenanceResponseParsingStrategy {

    @Resource
    private PspMaintenancePaymentTransactionCreationStrategy pspMaintenancePaymentTransactionCreationStrategy;

    @Override
    public PaymentTransactionEntryModel parseResponse(PspMaintenanceRequestType pspMaintenanceRequestType,
                                                      OrderModel orderModel, PspMaintenanceResponse pspMaintenanceResponse) {
        validateParameterNotNull(pspMaintenanceRequestType, "Parameter [pspMaintenanceRequestType] cannot be null");
        validateParameterNotNull(orderModel, "Parameter [orderModel] cannot be null");
        validateParameterNotNull(pspMaintenanceResponse, "Parameter [pspMaintenanceResponse] cannot be null");
        PaymentTransactionType paymentTransactionType = null;

        if (PspMaintenanceRequestType.CAPTURE.equals(pspMaintenanceRequestType)
                && (StringUtils.equals(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_PROCESSING), pspMaintenanceResponse.getStatus())
                || StringUtils.equals(String.valueOf(PspPaymentStatus.STATUS_PAYMENT_REQUESTED), pspMaintenanceResponse.getStatus()))) {
            paymentTransactionType = PaymentTransactionType.CAPTURE;
        } else if (PspMaintenanceRequestType.REFUND.equals(pspMaintenanceRequestType)
                && (StringUtils.equals(String.valueOf(PspPaymentStatus.STATUS_REFUND_PENDING), pspMaintenanceResponse.getStatus())
                || StringUtils.equals(String.valueOf(PspPaymentStatus.STATUS_REFUND), pspMaintenanceResponse.getStatus()))) {
            paymentTransactionType = PaymentTransactionType.REFUND_FOLLOW_ON;
        }

        if (paymentTransactionType != null) {
            return pspMaintenancePaymentTransactionCreationStrategy.createPaymentTransaction(paymentTransactionType,
                    orderModel, pspMaintenanceResponse);
        }

        throw new IllegalMaintenanceOperation("Request failed. Please see the logs for more details.");
    }
}
