package com.psp.payment.tokenisation.converters.populators.response;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import de.hybris.platform.converters.Populator;
import org.apache.commons.lang.StringUtils;

import java.util.Map;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/*
 * Populate PaymentInfo and set to PSPCreateSubscriptionResult.
 * */
public class PspPaymentInfoResultPopulator implements Populator<Map<String, String>, PspCreateSubscriptionResult> {

    @Override
    public void populate(final Map<String, String> source, final PspCreateSubscriptionResult target) {
        validateParameterNotNull(source, "Parameter [Map<String, String>] source cannot be null");
        validateParameterNotNull(target, "Parameter [PSPCreateSubscriptionResult] target cannot be null");

        final PaymentInfoData data = new PaymentInfoData();
        String expiryDate = source.get(PspConstants.KEY_RESULT_CARD_EXPIRYDATE);
        String cardNo = source.get(PspConstants.KEY_RESULT_CARD_NUMBER);
        String cardHolderName = source.get(PspConstants.KEY_RESULT_CARD_HOLDER);
        String brand = source.get(PspConstants.KEY_RESULT_CARD_BRAND);
        String cvc = source.get(PspConstants.KEY_RESULT_CARD_CVC);

        if (StringUtils.isNotEmpty(expiryDate)) {
            String expiryMonth = StringUtils.substring(expiryDate, 0, 2);
            String expiryYear = StringUtils.substring(expiryDate, 2, expiryDate.length());

            data.setCardExpirationMonth(Integer.parseInt(expiryMonth));
            data.setCardExpirationYear(Integer.parseInt(expiryYear));
        }

        if (StringUtils.isNotEmpty(cardNo)) {
            data.setCardAccountNumber(cardNo);
        }

        if (StringUtils.isNotEmpty(cardHolderName)) {
            data.setCardAccountHolderName(cardHolderName);
        }

        if (StringUtils.isNotEmpty(brand)) {
            data.setCardCardType(brand);
        }

        if (StringUtils.isNotEmpty(cvc)) {
            data.setCardCvNumber(cvc);
        }

        target.setPaymentInfoData(data);
    }
}
