package com.psp.payment.tokenisation.factories;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.enums.PspTokenisationStatus;
import com.psp.payment.tokenisation.strategies.PspResponseStrategy;

import java.util.Map;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/**
 * Factory for choosing the correct handler based on the tokenisation response.
 */
public class PspResponseStrategyFactory {

    private Map<PspTokenisationStatus, PspResponseStrategy> pspStatusResponseMapping;

    /**
     * Provides the strategy for processing the tokenisation response.
     *
     * @param pspCreateSubscriptionResult the tokenisation response
     * @return the response processing strategy
     */
    public PspResponseStrategy getResponseStrategy(final PspCreateSubscriptionResult pspCreateSubscriptionResult) {
        validateParameterNotNull(pspCreateSubscriptionResult, "PSPCreateSubscriptionResult cannot be null");

        return getPspStatusResponseMapping().get(pspCreateSubscriptionResult.getTokenisationStatus());
    }

    public Map<PspTokenisationStatus, PspResponseStrategy> getPspStatusResponseMapping() {
        return pspStatusResponseMapping;
    }

    public void setPspStatusResponseMapping(Map<PspTokenisationStatus, PspResponseStrategy> pspStatusResponseMapping) {
        this.pspStatusResponseMapping = pspStatusResponseMapping;
    }
}
