package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.payment.common.dto.PspErrorData;
import com.psp.payment.common.enums.PspTokenisationStatus;
import com.psp.payment.tokenisation.strategies.PspResponseStrategy;

public abstract class AbstractPspResponseStrategy implements PspResponseStrategy {

    /**
     * Default handling of the tokenisation response in case no specific
     * strategy is defined.
     *
     * @param result parsed response
     * @return details of action taken based on the response
     */
    @Override
    public PspPaymentSubscriptionResultItem handleResponse(PspCreateSubscriptionResult result) {
        final PspPaymentSubscriptionResultItem paymentSubscriptionResult = new PspPaymentSubscriptionResultItem();
        paymentSubscriptionResult.setSuccess(PspTokenisationStatus.OK.name().equalsIgnoreCase(result.getTokenisationStatus().name()));
        paymentSubscriptionResult.setDecision(result.getTokenisationStatus().name());
        paymentSubscriptionResult.setErrorCode(getErrorCode(result.getErrorData()));

        return paymentSubscriptionResult;
    }

    protected abstract String getErrorCode(PspErrorData errorData);

}
