package com.psp.payment.tokenisation.strategies.impl;

import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import com.psp.payment.tokenisation.strategies.PspCreateSubscriptionRequestStrategy;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.site.BaseSiteService;

import javax.annotation.Resource;
import java.util.Map;
import java.util.TreeMap;

import static com.psp.constants.PspConstants.*;
import static java.lang.Boolean.TRUE;

public class PspCreateSubscriptionRequestStrategyImpl implements PspCreateSubscriptionRequestStrategy {

    @Resource
    private PspUtils pspUtils;

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Resource
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource
    private BaseSiteService baseSiteService;

    @Override
    public PspCreateSubscriptionRequest createSubscriptionRequest(CartModel cartModel, CustomerModel customerModel) {
        final PspCreateSubscriptionRequest request = new PspCreateSubscriptionRequest();

        if (cartModel != null) {
            Map<String, String> parameters = new TreeMap<>();
            String pspId = getParameter(PARAM_PSPID);
            String storePermanently = YES;

            if (checkoutCustomerStrategy.isAnonymousCheckout()) {
                storePermanently = NO;
            }

            String orderId = pspUtils.generateUniqueOrderId(cartModel);
            BaseSiteModel currentBaseSite = baseSiteService.getCurrentBaseSite();
            String acceptUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, getParameter(PARAM_ACCEPT_URL));
            String exceptionUrl = siteBaseUrlResolutionService
                    .getWebsiteUrlForSite(currentBaseSite, TRUE, getParameter(PARAM_EXCEPTION_URL));

            parameters.put(KEY_TOKENISATION_PSPID, pspId);
            parameters.put(KEY_ORDERID, orderId);
            parameters.put(KEY_STORE_PERMANENTLY, storePermanently);
            parameters.put(KEY_PAYMENT_METHOD, cartModel.getPaymentMode().getPaymentMethod());

            if (!CREDIT_CARD.equals(cartModel.getPaymentMode().getBrand())) {
                parameters.put(KEY_RESULT_CARD_BRAND, cartModel.getPaymentMode().getBrand());
            }

            parameters.put(KEY_ACCEPT_URL, acceptUrl);
            parameters.put(KEY_EXCEPTION_URL, exceptionUrl);

            request.setPspId(pspId);
            request.setOrderId(orderId);
            request.setStorePermanently(storePermanently);
            request.setPaymentMethod(cartModel.getPaymentMode().getPaymentMethod());

            if (!CREDIT_CARD.equals(cartModel.getPaymentMode().getBrand())) {
                request.setBrand(cartModel.getPaymentMode().getBrand());
            }

            request.setAcceptUrl(acceptUrl);
            request.setExceptionUrl(exceptionUrl);

            String shaSignature = pspUtils.generateChecksum(parameters, getParameter(PARAM_PASSPHRASE));
            request.setShaSign(shaSignature);
        }

        return request;
    }

    public String getParameter(String parameterName) {
        return configurationService.getConfiguration().getString(parameterName);
    }
}
