package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.strategies.PspPaymentInfoCreateStrategy;
import com.psp.payment.tokenisation.strategies.PspCreditCardTypeBrandMappingStrategy;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import de.hybris.platform.core.enums.CreditCardType;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.util.UUID;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

public class PspCreditCardPaymentInfoCreateStrategyImpl implements PspPaymentInfoCreateStrategy<CreditCardPaymentInfoModel> {

    @Resource
    private ModelService modelService;

    @Resource
    private PspCreditCardTypeBrandMappingStrategy pspCreditCardTypeBrandMappingStrategy;

    @Resource
    private CartService cartService;

    @Override
    public CreditCardPaymentInfoModel saveSubscription(final CustomerModel customerModel,
                                                       final PspCreateSubscriptionResult response) {
        validateParameterNotNull(response, "PspCreateSubscriptionResult cannot be null");
        validateParameterNotNull(customerModel, "CustomerModel cannot be null");

        if (StringUtils.isNotEmpty(response.getPaymentToken()) && null != response.getPaymentInfoData()) {
            final CreditCardPaymentInfoModel cardPaymentInfoModel = createCreditCardPaymentInfo(
                    response.getPaymentToken(), response.getPaymentInfoData(), customerModel, Boolean.FALSE);
            modelService.save(cardPaymentInfoModel);

            return cardPaymentInfoModel;
        }

        return null;
    }

    private CreditCardPaymentInfoModel createCreditCardPaymentInfo(final String requestToken,
                                                                   final PaymentInfoData paymentInfo, final CustomerModel customerModel, final boolean saveInAccount) {
        validateParameterNotNull(paymentInfo, "paymentInfo cannot be null");
        validateParameterNotNull(customerModel, "customerModel cannot be null");

        final CreditCardPaymentInfoModel cardPaymentInfoModel = modelService.create(CreditCardPaymentInfoModel.class);
        cardPaymentInfoModel.setCode(customerModel.getUid() + "_" + UUID.randomUUID());
        cardPaymentInfoModel.setUser(customerModel);
        cardPaymentInfoModel.setSubscriptionId(requestToken);
        cardPaymentInfoModel.setNumber(paymentInfo.getCardAccountNumber());
        cardPaymentInfoModel.setCcOwner(paymentInfo.getCardAccountHolderName());
        cardPaymentInfoModel.setBillingAddress(cartService.getSessionCart().getPaymentAddress());

        CreditCardType creditCardType = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(paymentInfo.getCardCardType());
        cardPaymentInfoModel.setType(creditCardType);

        if (paymentInfo.getCardExpirationMonth() != null && paymentInfo.getCardExpirationMonth() > 0) {
            cardPaymentInfoModel.setValidToMonth(String.valueOf(paymentInfo.getCardExpirationMonth()));
        }
        if (paymentInfo.getCardExpirationYear() != null && paymentInfo.getCardExpirationYear() > 0) {
            cardPaymentInfoModel.setValidToYear(String.valueOf(paymentInfo.getCardExpirationYear()));
        }

        cardPaymentInfoModel.setSaved(saveInAccount);

        return cardPaymentInfoModel;
    }
}
