package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.payment.common.strategies.PspPaymentInfoCreateStrategy;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;
import java.util.UUID;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;


public class PspDirectDebitPaymentInfoCreateStrategyImpl implements PspPaymentInfoCreateStrategy<DirectDebitPaymentInfoModel> {
    @Resource
    private ModelService modelService;

    @Resource
    private CartService cartService;

    @Override
    public DirectDebitPaymentInfoModel saveSubscription(CustomerModel customerModel, PspCreateSubscriptionResult response) {
        validateParameterNotNull(response, "PspCreateSubscriptionResult cannot be null");
        validateParameterNotNull(customerModel, "CustomerModel cannot be null");

        if (StringUtils.isNotEmpty(response.getPaymentToken()) && null != response.getPaymentInfoData()) {
            DirectDebitPaymentInfoModel directDebitPaymentInfo = createDirectDebitPaymentInfo(
                    response.getPaymentToken(), response.getPaymentInfoData(), customerModel, Boolean.FALSE);
            modelService.save(directDebitPaymentInfo);

            return directDebitPaymentInfo;
        }
        return null;
    }

    private DirectDebitPaymentInfoModel createDirectDebitPaymentInfo(final String requestToken,
                                                                     final PaymentInfoData paymentInfo, final CustomerModel customerModel, final boolean saveInAccount) {
        validateParameterNotNull(paymentInfo, "paymentInfo cannot be null");
        validateParameterNotNull(customerModel, "customerModel cannot be null");
        DirectDebitPaymentInfoModel directDebitPaymentInfo = modelService.create(DirectDebitPaymentInfoModel.class);
        directDebitPaymentInfo.setCode(customerModel.getUid() + "_" + UUID.randomUUID());
        directDebitPaymentInfo.setUser(customerModel);
        directDebitPaymentInfo.setSubscriptionId(requestToken);
        directDebitPaymentInfo.setBrand(paymentInfo.getCardCardType());
        directDebitPaymentInfo.setAccountNumber(paymentInfo.getCardAccountNumber());
        directDebitPaymentInfo.setAccountName(paymentInfo.getCardAccountHolderName());
        directDebitPaymentInfo.setBillingAddress(cartService.getSessionCart().getPaymentAddress());
        directDebitPaymentInfo.setSaved(saveInAccount);
        return directDebitPaymentInfo;
    }
}
