package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.strategies.PspPaymentInfoCreateStrategy;
import com.psp.payment.common.strategies.PspPaymentTransactionStrategy;
import com.psp.payment.tokenisation.strategies.PspResponseStrategy;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import org.apache.commons.lang.StringUtils;

import javax.annotation.Resource;

import static de.hybris.platform.servicelayer.util.ServicesUtil.validateParameterNotNull;

/**
 * Contract for handling successful tokenisation response.
 */
public class PspSucessResponseStrategyImpl implements PspResponseStrategy {

    @Resource
    private PspPaymentTransactionStrategy pspPaymentTransactionStrategy;

    @Resource
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Resource
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Resource
    private CartService cartService;

    @Resource
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Override
    public PspPaymentSubscriptionResultItem handleResponse(final PspCreateSubscriptionResult result) {
        validateParameterNotNull(result, "PspCreateSubscriptionResult cannot be null");

        final PspPaymentSubscriptionResultItem paymentSubscriptionResult = new PspPaymentSubscriptionResultItem();
        final CustomerModel customer = checkoutCustomerStrategy.getCurrentUserForCheckout();

        if (StringUtils.isNotEmpty(result.getPaymentToken())) {
            pspPaymentTransactionStrategy.savePaymentTransactionEntry(customer, result.getPaymentToken());
        }

        CartModel cartModel = cartService.getSessionCart();

        PaymentModeModel paymentModeModel = cartModel.getPaymentMode();
        ComposedTypeModel composedTypeModel = paymentModeModel.getPaymentInfoType();

        PspPaymentInfoCreateStrategy pspPaymentInfoCreateStrategy = pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel);


        final PaymentInfoModel paymentInfoModel = pspPaymentInfoCreateStrategy.saveSubscription(customer, result);

        if (null != paymentInfoModel) {
            paymentSubscriptionResult.setStoredCard(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(paymentInfoModel).convert(paymentInfoModel));
            paymentSubscriptionResult.setSuccess(true);
        }

        return paymentSubscriptionResult;
    }
}
