package com.psp.payment.common.converters.populators;

import com.psp.model.DirectDebitPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.converters.impl.AbstractPopulatingConverter;
import de.hybris.platform.core.PK;
import de.hybris.platform.core.model.user.AddressModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectDebitPaymentInfoPopulatorTest {

    private final String ACCOUNT_NUMBER = "0124043XXXBLZ39070024";
    private final String SUBSCRIPTION_ID = "00005000-1651670";
    private final String ACCOUNT_NAME = "test";
    private final String BRAND_NAME = "direct debits";
    private final String PK_STRING = "123";
    @InjectMocks
    private PspDirectDebitPaymentInfoPopulator pspDirectDebitPaymentInfoPopulator;

    @Mock
    private AbstractPopulatingConverter<AddressModel, AddressData> addressConverter;
    @Mock
    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;

    private CCPaymentInfoData ccPaymentInfoData;

    @Before
    public void setUp(){
        ccPaymentInfoData = new CCPaymentInfoData();
        when(directDebitPaymentInfoModel.getAccountName()).thenReturn(ACCOUNT_NAME);
        when(directDebitPaymentInfoModel.getAccountNumber()).thenReturn(ACCOUNT_NUMBER);
        when(directDebitPaymentInfoModel.getSubscriptionId()).thenReturn(SUBSCRIPTION_ID);
        when(directDebitPaymentInfoModel.getBrand()).thenReturn(BRAND_NAME);
        PK pk = PK.parse(PK_STRING);
        when(directDebitPaymentInfoModel.getPk()).thenReturn(pk);
    }
    @Test(expected = IllegalArgumentException.class)
    public void populateWithSourceNull() throws Exception {
        // Given no source

        // When
        pspDirectDebitPaymentInfoPopulator.populate(null, ccPaymentInfoData);

        // Then it throws an exception
    } @Test(expected = IllegalArgumentException.class)
    public void populateWithTargetNull() throws Exception {
        // Given no source

        // When
        pspDirectDebitPaymentInfoPopulator.populate(directDebitPaymentInfoModel, null);

        // Then it throws an exception
    }
    @Test
    public void populateSuccess() {
        // Given both source (without address) and target

        // When
        pspDirectDebitPaymentInfoPopulator.populate(directDebitPaymentInfoModel, ccPaymentInfoData);

        // Then
        assertEquals(ACCOUNT_NAME, ccPaymentInfoData.getAccountHolderName());
        assertEquals(ACCOUNT_NUMBER, ccPaymentInfoData.getCardNumber());
        assertEquals(SUBSCRIPTION_ID, ccPaymentInfoData.getSubscriptionId());
        assertEquals(BRAND_NAME, ccPaymentInfoData.getCardType());
        verify(addressConverter, never()).convert(any());
    }

    @Test
    public void populateWithAddress() {
        // Given both source (with address) and target
        AddressModel addressModel = new AddressModel();
        when(directDebitPaymentInfoModel.getBillingAddress()).thenReturn(addressModel);

        // When
        pspDirectDebitPaymentInfoPopulator.populate(directDebitPaymentInfoModel, ccPaymentInfoData);

        // Then
        assertEquals(ACCOUNT_NAME, ccPaymentInfoData.getAccountHolderName());
        assertEquals(ACCOUNT_NUMBER, ccPaymentInfoData.getCardNumber());
        assertEquals(SUBSCRIPTION_ID, ccPaymentInfoData.getSubscriptionId());
        assertEquals(BRAND_NAME, ccPaymentInfoData.getCardType());
        verify(addressConverter).convert(addressModel);
    }

}
