package com.psp.payment.common.converters.populators;

import com.psp.data.payment.PspFlexPaymentInfoData;
import com.psp.model.PayPalPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentInfoPopulatorTest {

    @InjectMocks
    private PspFlexPaymentInfoPopulator pspFlexPaymentInfoPopulator;

    @Mock
    private Converter<AddressModel, AddressData> addressConverter;

    private PayPalPaymentInfoModel payPalPaymentInfoModel;

    private PspFlexPaymentInfoData paypalPaymentInfoData;

    @Before
    public void setUp() {
        paypalPaymentInfoData = new PspFlexPaymentInfoData();
        payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        payPalPaymentInfoModel.setAcceptanceCode("test123");
        payPalPaymentInfoModel.setPayId("pay-id");
        payPalPaymentInfoModel.setCode("code");
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateWithSourceNull() throws Exception {
        // Given no source

        // When
        pspFlexPaymentInfoPopulator.populate(null, paypalPaymentInfoData);

        // Then it throws an exception
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateWithTargetNull() throws Exception {
        // Given no source

        // When
        pspFlexPaymentInfoPopulator.populate(payPalPaymentInfoModel, null);

        // Then it throws an exception
    }

    @Test
    public void populateSuccess() {
        // Given both source (without address) and target

        // When
        pspFlexPaymentInfoPopulator.populate(payPalPaymentInfoModel, paypalPaymentInfoData);

        // Then
        assertEquals("code", paypalPaymentInfoData.getCode());
        assertEquals("pay-id", paypalPaymentInfoData.getPayid());
        assertEquals("test123", paypalPaymentInfoData.getAcceptance());
        verify(addressConverter, never()).convert(any());
    }

    @Test
    public void populateWithAddress() {
        // Given both source (with address) and target
        AddressModel addressModel = new AddressModel();
        payPalPaymentInfoModel.setBillingAddress(addressModel);

        // When
        pspFlexPaymentInfoPopulator.populate(payPalPaymentInfoModel, paypalPaymentInfoData);

        // Then
        assertEquals("code", paypalPaymentInfoData.getCode());
        assertEquals("pay-id", paypalPaymentInfoData.getPayid());
        assertEquals("test123", paypalPaymentInfoData.getAcceptance());
        verify(addressConverter).convert(addressModel);
    }
}
