package com.psp.payment.common.converters.populators;

import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.flexpayments.strategies.impl.PspFlexPaymentInfoConvertStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspDirectDebitPaymentInfoConvertStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.order.data.OrderData;
import de.hybris.platform.core.model.order.OrderModel;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspOrderPopulatorTest {

    @InjectMocks
    private PspOrderPopulator pspOrderPopulator;

    @Mock
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Mock
    private PspFlexPaymentInfoConvertStrategyImpl pspPaypalPaymentInfoConvertStrategy;

    @Mock
    private PspDirectDebitPaymentInfoConvertStrategyImpl pspDirectDebitPaymentInfoConvertStrategy;

    @Mock
    private OrderModel orderModel;

    @Mock
    private OrderData orderData;

    @Mock
    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;
    @Mock
    private PayPalPaymentInfoModel payPalPaymentInfoModel;

    @Test
    public void addPaymentInformationHandlesPayPal() {
        // Given
        when(orderModel.getPaymentInfo()).thenReturn(payPalPaymentInfoModel);
        when (pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(payPalPaymentInfoModel)).thenReturn(pspPaypalPaymentInfoConvertStrategy);

        // When
        pspOrderPopulator.addPaymentInformation(orderModel, orderData);

        // Then
        verify(pspPaypalPaymentInfoConvertStrategy).convert(payPalPaymentInfoModel);
    }
    @Test
    public void addPaymentInformationHandlesDirectDebits() {
        // Given
        when(orderModel.getPaymentInfo()).thenReturn(directDebitPaymentInfoModel);
        when (pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(directDebitPaymentInfoModel)).thenReturn(pspDirectDebitPaymentInfoConvertStrategy);

        // When
        pspOrderPopulator.addPaymentInformation(orderModel, orderData);

        // Then
        verify(pspDirectDebitPaymentInfoConvertStrategy).convert(directDebitPaymentInfoModel);
    }

}
