package com.psp.payment.common.converters.populators;

import com.psp.model.TokenisedPaymentModeModel;
import com.psp.payment.common.dto.PspBrand;
import com.psp.payment.common.dto.PspPaymentMode;
import com.psp.payment.common.dto.PspPaymentModeData;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.product.converters.populator.ImagePopulator;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentModePopulatorTest {

    @InjectMocks
    private PspPaymentModePopulator pspPaymentModePopulator;

    @Mock
    private ImagePopulator imagePopulator;

    @Mock
    private PaymentModeModel paymentModeModel;

    @Mock
    private TokenisedPaymentModeModel tokenisedPaymentModeModel;

    @Mock
    private ComposedTypeModel composedTypeModel;

    private List<PaymentModeModel> paymentModeModelList;

    private List<TokenisedPaymentModeModel> tokenisedPaymentModeModelList;

    @Before
    public void setup() {
        paymentModeModelList = getFlexPaymentModeModelList();
        tokenisedPaymentModeModelList = getTokenisedPaymentModeModelList();
    }

    @Test
    public void paymentModeDataIsPopulatedForFlexPayments() {
        // Given
        PspPaymentModeData paymentModeData = new PspPaymentModeData();

        // When
        pspPaymentModePopulator.populate(paymentModeModelList, paymentModeData);

        // Then
        List<PspPaymentMode> paymentModes = paymentModeData.getPaymentModes();
        List<PspBrand> brandList = paymentModes.get(0).getBrands();
        PspBrand brand = brandList.get(0);

        assertEquals(1, paymentModes.size());
        assertEquals("PayPal-PM", paymentModes.get(0).getCode());
        assertEquals(1, brandList.size());
        assertEquals("paypal-paypal", brand.getCode());
        assertEquals("PayPal-Name", brand.getDisplayName());
        assertEquals("PayPal-Brand", brand.getName());
        assertEquals(composedTypeModel, brand.getComposedTypeModel());
        assertNull(brand.getImage());
        assertFalse(paymentModes.get(0).getTokenised());
    }

    @Test
    public void paymentModeDataIsPopulatedForTokenisedPayments() {
        // Given
        PspPaymentModeData paymentModeData = new PspPaymentModeData();

        // When
        pspPaymentModePopulator.populate(tokenisedPaymentModeModelList, paymentModeData);

        // Then
        List<PspPaymentMode> paymentModes = paymentModeData.getPaymentModes();
        assertTrue(paymentModes.get(0).getTokenised());
    }

    private List<PaymentModeModel> getFlexPaymentModeModelList() {
        when(paymentModeModel.getCode()).thenReturn("paypal-paypal");
        when(paymentModeModel.getPaymentMethod()).thenReturn("PayPal-PM");
        when(paymentModeModel.getName()).thenReturn("PayPal-Name");
        when(paymentModeModel.getBrand()).thenReturn("PayPal-Brand");
        when(paymentModeModel.getPaymentInfoType()).thenReturn(composedTypeModel);

        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModelList.add(paymentModeModel);

        return paymentModeModelList;
    }

    private List<TokenisedPaymentModeModel> getTokenisedPaymentModeModelList() {
        when(tokenisedPaymentModeModel.getCode()).thenReturn("creditcard-creditcard");
        when(tokenisedPaymentModeModel.getPaymentMethod()).thenReturn("CC-PM");
        when(tokenisedPaymentModeModel.getName()).thenReturn("CC-Name");
        when(tokenisedPaymentModeModel.getBrand()).thenReturn("CC-Brand");
        when(tokenisedPaymentModeModel.getPaymentInfoType()).thenReturn(composedTypeModel);

        List<TokenisedPaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModelList.add(tokenisedPaymentModeModel);

        return paymentModeModelList;
    }
}
