package com.psp.payment.common.dao.impl;


import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyMap;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.when;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import de.hybris.platform.servicelayer.search.SearchResult;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.junit.Test;
import org.mockito.Answers;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.InjectMocks;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCustomerAccountDaoImplTest {
    @InjectMocks
    private PspCustomerAccountDaoImpl pspCustomerAccountDao;
    @Mock
    private FlexibleSearchService flexibleSearchService;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private SearchResult<PaymentInfoModel> searchResult;

    @Mock
    private List<PaymentInfoModel> paymentInfoModelList;

    private CustomerModel customerModel;

    @Before
    public void setUp() throws Exception {
        customerModel = new CustomerModel();
        PaymentInfoModel paymentInfoModel = new PaymentInfoModel();
        paymentInfoModelList = Arrays.asList(paymentInfoModel);
        doReturn(searchResult).when(flexibleSearchService).search(anyString(),anyMap());
        doReturn(paymentInfoModelList).when(searchResult).getResult();
        doReturn(1).when(searchResult).getCount();
    }

    @Test
    public void findPaymentInfoByCustomer() throws Exception {
        // Given nothing

        // When
        PaymentInfoModel paymentInfoModel = pspCustomerAccountDao.findPaymentInfoByCustomer(customerModel, "1234");

        //Then
        assertTrue(paymentInfoModelList.contains(paymentInfoModel));
        //assertNull(paymentInfoModel);

    }
    @Test
    public void findPaymentInfoByCustomerNoResult() throws Exception {
        // Given
        doReturn(0).when(searchResult).getCount();

        // When
        PaymentInfoModel paymentInfoModel = pspCustomerAccountDao.findPaymentInfoByCustomer(customerModel, "1234");

        //Then
        assertNull(paymentInfoModel);

    }

    @Test
    public void findPaymentInfosByCustomer() throws Exception {
        // Given nothing

        // When
        List<PaymentInfoModel> paymentInfoModels = pspCustomerAccountDao.findPaymentInfosByCustomer(customerModel, false);
        //Then
        assertEquals(paymentInfoModels, paymentInfoModelList);
    }

    @Test
    public void findPaymentInfosByCustomerSaved() throws Exception {
        // Given nothing

        // When
        List<PaymentInfoModel> paymentInfoModels = pspCustomerAccountDao.findPaymentInfosByCustomer(customerModel, true);
        //Then
        assertEquals(paymentInfoModels, paymentInfoModelList);
    }




}