package com.psp.payment.common.dao.impl;

import com.psp.model.PayPalPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import de.hybris.platform.servicelayer.search.SearchResult;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentModeDaoImplTest {

    @Spy
    @InjectMocks
    private PspPaymentModeDaoImpl pspPaymentModeDao;

    @Mock
    private FlexibleSearchService flexibleSearchService;

    @Mock
    private SearchResult<PaymentModeModel> searchResult;

    @Mock
    private List<PaymentModeModel> paymentModeModelList;

    @Before
    public void setup() {
        doReturn(searchResult).when(flexibleSearchService).search(any(FlexibleSearchQuery.class));
        when(searchResult.getResult()).thenReturn(paymentModeModelList);
    }

    @Test
    public void flexSearchToFindAllPaymentModes() {
        // When
        List<PaymentModeModel> result = pspPaymentModeDao.findActivePaymentModes(BigDecimal.valueOf(20.00),new CMSSiteModel());

        // Then
        assertEquals(paymentModeModelList, result);
}

    @Test
    public void flexSearchToFindSpecificPaymentMode() {
        // When
        List<PaymentModeModel> result = pspPaymentModeDao.findActivePaymentModeForCode(BigDecimal.valueOf(20.00), "1234");

        // Then
        assertEquals(paymentModeModelList, result);
    }

    @Test
    public void flexSearchToFindSpecificPaymentModeForPaymentInfo() {
        // Given
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        payPalPaymentInfoModel.setCode("PayPalPaymentInfo");

        // When
        List<PaymentModeModel> result = pspPaymentModeDao.findActivePaymentModeForPaymentInfo(payPalPaymentInfoModel);

        // Then
        assertEquals(paymentModeModelList, result);
    }
}
