package com.psp.payment.common.facades.payment.impl;

import com.psp.payment.common.dto.PspPaymentModeData;
import com.psp.payment.common.services.PspPaymentModeService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.paymentstandard.model.StandardPaymentModeModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentModeFacadeImplTest {

    @InjectMocks
    private PspPaymentModeFacadeImpl pspPaymentFacade;

    @Mock
    private PspPaymentModeService pspPaymentModeService;

    @Mock
    private Converter<List<PaymentModeModel>, PspPaymentModeData> pspPaymentModeConverter;

    @Mock
    private StandardPaymentModeModel paymentModeModel;

    @Test
    public void correctPaymentMethodsAreReturned() {
        // Given
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModel.setPaymentMethod("Cards");
        paymentModeModel.setBrand("CreditCard");
        paymentModeModel.setName("Credit/Debit Cards");
        paymentModeModelList.add(paymentModeModel);

        when(pspPaymentModeService.getAvailablePaymentMethods()).thenReturn(paymentModeModelList);
        when(pspPaymentModeConverter.convert(any(), any())).thenReturn(null);

        // When
        pspPaymentFacade.getPaymentModes();

        // Then
        verify(pspPaymentModeService).getAvailablePaymentMethods();
        verify(pspPaymentModeConverter).convert(any(), any());
    }

    @Test
    public void noPaymentMethodsAreReturned() {
        // Given
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();

        when(pspPaymentModeService.getAvailablePaymentMethods()).thenReturn(paymentModeModelList);

        // When
        pspPaymentFacade.getPaymentModes();

        // Then
        verify(pspPaymentModeService).getAvailablePaymentMethods();
        verify(pspPaymentModeConverter, never()).convert(any(), any());
    }

    @Test
    public void correctPaymentMethodIsReturned() {
        // Given
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModel.setPaymentMethod("Cards");
        paymentModeModel.setBrand("CreditCard");
        paymentModeModel.setName("Credit/Debit Cards");
        paymentModeModelList.add(paymentModeModel);

        when(pspPaymentModeService.getAvailablePaymentModeForCode("string")).thenReturn(paymentModeModelList);

        // When
        pspPaymentFacade.getPaymentModeForCode("string");

        // Then
        verify(pspPaymentModeService).getAvailablePaymentModeForCode("string");
        verify(pspPaymentModeConverter).convert(any(), any());
    }

    @Test
    public void noPaymentMethodIsReturned() {
        // Given
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();

        when(pspPaymentModeService.getAvailablePaymentModeForCode("string")).thenReturn(paymentModeModelList);

        // When
        pspPaymentFacade.getPaymentModeForCode("string");

        // Then
        verify(pspPaymentModeService).getAvailablePaymentModeForCode("string");
        verify(pspPaymentModeConverter, never()).convert(any(), any());
    }
}
