package com.psp.payment.common.facades.user.impl;


import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.services.PspCustomerAccountService;
import com.psp.payment.tokenisation.strategies.impl.PspCreditCardPaymentInfoConvertStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspDirectDebitPaymentInfoConvertStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspUserFacadeImplTest {
    @Spy
    @InjectMocks
    private PspUserFacadeImpl pspUserFacade;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private PspCustomerAccountService customerAccountService;

    private CCPaymentInfoData paymentInfoData;

    private CustomerModel customerModel;

    private CreditCardPaymentInfoModel creditCardPaymentInfoModel;

    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;

    private List<PaymentInfoModel> paymentInfoModels;

    @Mock
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Mock
    private PspCreditCardPaymentInfoConvertStrategyImpl pspCreditCardPaymentInfoConvertStrategy;

    @Mock
    private PspDirectDebitPaymentInfoConvertStrategyImpl pspDirectDebitPaymentInfoConvertStrategy;


    private PaymentInfoModel paymentInfoModel;

    @Before
    public void setUp() throws Exception {
        paymentInfoData = new CCPaymentInfoData();
        pspUserFacade.setCheckoutCustomerStrategy(checkoutCustomerStrategy);
        customerModel = new CustomerModel();
        paymentInfoModel = new PaymentInfoModel();
        paymentInfoModels = new ArrayList<PaymentInfoModel>();
        creditCardPaymentInfoModel = new CreditCardPaymentInfoModel();
        directDebitPaymentInfoModel = new DirectDebitPaymentInfoModel();
        paymentInfoModels.add(creditCardPaymentInfoModel);
        paymentInfoModels.add(directDebitPaymentInfoModel);
        when(customerAccountService.getPaymentInfos(customerModel, true)).thenReturn(paymentInfoModels);
        doReturn(customerModel).when(checkoutCustomerStrategy).getCurrentUserForCheckout();
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(creditCardPaymentInfoModel)).thenReturn(pspCreditCardPaymentInfoConvertStrategy);
        when(pspCreditCardPaymentInfoConvertStrategy.convert(creditCardPaymentInfoModel)).thenReturn(paymentInfoData);
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(directDebitPaymentInfoModel)).thenReturn(pspDirectDebitPaymentInfoConvertStrategy);
        when(pspDirectDebitPaymentInfoConvertStrategy.convert(directDebitPaymentInfoModel)).thenReturn(paymentInfoData);


    }

    @Test(expected = IllegalArgumentException.class)
    public void setDeafaultPaymentInfoWithNullValue() {
        // Given nothing

        // When
        pspUserFacade.setDefaultPaymentInfo(null);

        // Then exception
    }

    @Test
    public void setDefaultPaymentInfoWithNoExistingPaymentInfo() {
        //Given
        when(customerAccountService.getPaymentInfoForCode(any(CustomerModel.class), anyString())).thenReturn(null);

        //When
        pspUserFacade.setDefaultPaymentInfo(paymentInfoData);

        //then
        verify(customerAccountService, never()).setDefaultPaymentInfo(any(CustomerModel.class), any(PaymentInfoModel.class));
    }

    @Test
    public void setPaymentInfoSuccess() {
        // Given
        when(customerAccountService.getPaymentInfoForCode(any(CustomerModel.class), anyString())).thenReturn(paymentInfoModel);

        // When
        pspUserFacade.setDefaultPaymentInfo(paymentInfoData);

        // Then
        verify(customerAccountService).setDefaultPaymentInfo(any(CustomerModel.class), any(PaymentInfoModel.class));
    }

    @Test
    public void getCCPaymentInfosWithDefaultCreditCard() {
        // Given
        customerModel.setDefaultPaymentInfo(creditCardPaymentInfoModel);

        // When
        final List<CCPaymentInfoData> list = pspUserFacade.getCCPaymentInfos(true);
        //Then
        assertEquals(Boolean.TRUE, Boolean.valueOf(list.iterator().next().isDefaultPaymentInfo()));
    }

    @Test
    public void getCCPaymentInfosWithDefaultDirectDebitPaymentInfo() {
        //Given
        customerModel.setDefaultPaymentInfo(directDebitPaymentInfoModel);

        //When
        final List<CCPaymentInfoData> list = pspUserFacade.getCCPaymentInfos(true);

        //Then
        assertEquals(Boolean.TRUE, Boolean.valueOf(list.iterator().next().isDefaultPaymentInfo()));
    }

    @Test
    public void testGetCCPaymentInfosNoDefaultInfo() {
        // Given no default paymentInfo

        //When
        List<CCPaymentInfoData> list = pspUserFacade.getCCPaymentInfos(true);
        //Then
        assertEquals(Boolean.FALSE, Boolean.valueOf(list.iterator().next().isDefaultPaymentInfo()));
    }


}