package com.psp.payment.common.factories;


import static org.junit.Assert.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.strategies.PspPaymentInfoConvertStrategy;
import com.psp.payment.flexpayments.strategies.impl.PspFlexPaymentInfoConvertStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspCreditCardPaymentInfoConvertStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspDirectDebitPaymentInfoConvertStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import org.junit.runner.RunWith;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.InjectMocks;

import java.util.Map;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentInfoConvertStrategyFactoryTest {
    @InjectMocks
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Mock
    private PspFlexPaymentInfoConvertStrategyImpl pspPaypalPaymentInfoConvertStrategy;

    @Mock
    private PspDirectDebitPaymentInfoConvertStrategyImpl pspDirectDebitPaymentInfoConvertStrategy;

    @Mock
    private PspCreditCardPaymentInfoConvertStrategyImpl pspCreditCardPaymentInfoConvertStrategy;
    @Mock
    private PayPalPaymentInfoModel payPalPaymentInfoModel;
    @Mock
    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;
    @Mock
    private CreditCardPaymentInfoModel creditCardPaymentInfoModel;
    @Mock
    private Map<String, PspPaymentInfoConvertStrategy> paymentInfoConvertStrategyMapping;


    @Test
    public void getPaypalPaymentInfoConvertStrategy() throws Exception {
        // Given
        when(payPalPaymentInfoModel.getItemtype()).thenReturn(PayPalPaymentInfoModel._TYPECODE);
        when(paymentInfoConvertStrategyMapping.get(PayPalPaymentInfoModel._TYPECODE)).thenReturn(pspPaypalPaymentInfoConvertStrategy);

        // When
        PspPaymentInfoConvertStrategy result = pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(payPalPaymentInfoModel);

        //Then
        assertTrue(result instanceof PspFlexPaymentInfoConvertStrategyImpl);

    }

    @Test
    public void getDirectDebitPaymentInfoConvertStrategy() {
        // Given
        when(directDebitPaymentInfoModel.getItemtype()).thenReturn(DirectDebitPaymentInfoModel._TYPECODE);
        when(paymentInfoConvertStrategyMapping.get(DirectDebitPaymentInfoModel._TYPECODE)).thenReturn(pspDirectDebitPaymentInfoConvertStrategy);

        // When
        PspPaymentInfoConvertStrategy result = pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(directDebitPaymentInfoModel);

        // Then
        assertTrue(result instanceof  PspDirectDebitPaymentInfoConvertStrategyImpl);
    }

    @Test
    public void getCreditCardPaymentInfoConvertStrategy() {
        // Given
        when(creditCardPaymentInfoModel.getItemtype()).thenReturn(CreditCardPaymentInfoModel._TYPECODE);
        when(paymentInfoConvertStrategyMapping.get(CreditCardPaymentInfoModel._TYPECODE)).thenReturn(pspCreditCardPaymentInfoConvertStrategy);

        // When
        PspPaymentInfoConvertStrategy result = pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(creditCardPaymentInfoModel);

        // Then
        assertTrue(result instanceof  PspCreditCardPaymentInfoConvertStrategyImpl);
    }



}