package com.psp.payment.common.factories;

import com.psp.model.GenericFlexPaymentInfoModel;
import com.psp.payment.common.strategies.PspPaymentInfoCreateStrategy;
import com.psp.payment.flexpayments.strategies.impl.PspGenericFlexPaymentInfoCreateStrategyImpl;
import com.psp.payment.flexpayments.strategies.impl.PspPayPalPaymentInfoCreateStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentInfoCreateStrategyFactoryTest {

    @InjectMocks
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Mock
    private ComposedTypeModel composedTypeModel;

    @Mock
    private Map<String, PspPaymentInfoCreateStrategy> paymentInfoCreateStrategyMapping;

    @Mock
    private PspPayPalPaymentInfoCreateStrategyImpl pspPayPalPaymentInfoCreateStrategy;

    @Mock
    private PspGenericFlexPaymentInfoCreateStrategyImpl pspGenericFlexPaymentInfoCreateStrategy;

    @Test
    public void payPalPaymentInfoCreateStrategyIsReturned() {
        // Given
        when(composedTypeModel.getCode()).thenReturn("code");
        when(paymentInfoCreateStrategyMapping.get("code")).thenReturn(pspPayPalPaymentInfoCreateStrategy);

        // When
        PspPaymentInfoCreateStrategy result = pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel);

        // Then
        assertEquals(pspPayPalPaymentInfoCreateStrategy, result);
    }

    @Test
    public void genericCreateStrategyIsGivenOnMappingNotFound() {
        // Given
        when(composedTypeModel.getCode()).thenReturn("code");
        when(paymentInfoCreateStrategyMapping.get(GenericFlexPaymentInfoModel._TYPECODE)).thenReturn(pspGenericFlexPaymentInfoCreateStrategy);

        // When
        PspPaymentInfoCreateStrategy result = pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(composedTypeModel);

        // Then
        assertEquals(pspGenericFlexPaymentInfoCreateStrategy, result);
    }
}
