package com.psp.payment.common.services.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commerceservices.customer.CustomerEmailResolutionService;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

/**
 * Unit test for performing address related operations.
 */
@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspAddressServiceImplTest {

    @InjectMocks
    private PspAddressServiceImpl pspAddressService;

    @Mock
    private CartService cartService;

    @Mock
    private ModelService modelService;

    @Mock
    private CustomerEmailResolutionService customerEmailResolutionService;

    @Mock
    private Converter<AddressData, AddressModel> addressReverseConverter;

    private AddressModel shippingAddressModel;

    private AddressModel billingAddressModel;

    private CartModel cartModel;

    private AddressData addressData;

    private CustomerModel customerModel;

    @Before
    public void setUp() {
        shippingAddressModel = new AddressModel();
        billingAddressModel = new AddressModel();
        customerModel = new CustomerModel();
        cartModel = new CartModel();
        cartModel.setUser(customerModel);
        cartModel.setDeliveryAddress(shippingAddressModel);
        addressData = new AddressData();

        when(modelService.clone(shippingAddressModel)).thenReturn(billingAddressModel);
        when(modelService.create(AddressModel.class)).thenReturn(billingAddressModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(addressReverseConverter.convert(addressData, shippingAddressModel)).thenReturn(billingAddressModel);
        when(customerEmailResolutionService.getEmailForCustomer(customerModel)).thenReturn("email");
    }

    @Test
    public void saveAddressForNewBillingAddress() {
        // When
        pspAddressService.saveBillingAddressToCurrentCart(addressData, Boolean.FALSE);

        // Then
        verify(modelService).create(AddressModel.class);
        verify(addressReverseConverter).convert(addressData, billingAddressModel);
        verify(modelService, never()).clone(shippingAddressModel);
        verify(modelService).saveAll(billingAddressModel, cartModel);
        assertEquals("email", billingAddressModel.getEmail());
        assertEquals(cartModel, billingAddressModel.getOwner());
        assertEquals(billingAddressModel, cartModel.getPaymentAddress());
    }

    @Test
    public void saveAddressForBillingSameAsShippingAddress() {
        // When
        pspAddressService.saveBillingAddressToCurrentCart(addressData, Boolean.TRUE);

        // Then
        verify(modelService).clone(shippingAddressModel);
        verify(modelService, never()).create(AddressModel.class);
        verify(addressReverseConverter, never()).convert(addressData, billingAddressModel);
        verify(modelService).saveAll(billingAddressModel, cartModel);
        assertEquals("email", billingAddressModel.getEmail());
        assertEquals(cartModel, billingAddressModel.getOwner());
        assertEquals(billingAddressModel, cartModel.getPaymentAddress());
    }

    @Test
    public void testSaveAddressInternalCallsAreIgnored() {
        // Given nothing

        // When
        pspAddressService.saveBillingAddressToCurrentCart(null, Boolean.FALSE);

        // Then
        verify(addressReverseConverter, never()).convert(any(AddressData.class));
        verify(modelService, never()).saveAll(any(AddressModel.class), any(CartModel.class));
        verify(modelService, never()).clone(shippingAddressModel);
    }
}
