package com.psp.payment.common.services.impl;


import com.psp.payment.common.dao.PspCustomerAccountDao;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.user.CustomerModel;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Mockito.verify;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCustomerAccountServiceImplTest {

    @InjectMocks
    private PspCustomerAccountServiceImpl pspCustomerAccountService;

    @Mock
    private PspCustomerAccountDao customerAccountDao;

    private CustomerModel customerModel;

    @Before
    public void setUp() throws Exception {
        customerModel = new CustomerModel();
    }

    @Test(expected = IllegalArgumentException.class)
    public void getPaymentInfoForCodeWithNullCustomer() {
        // When
        pspCustomerAccountService.getPaymentInfoForCode(null, null);

        //Then an exception is thrown
    }

    @Test
    public void getPaymentInfoForCodeSuccess() {
        // When
        pspCustomerAccountService.getPaymentInfoForCode(customerModel, StringUtils.EMPTY);

        // Then
        verify(customerAccountDao).findPaymentInfoByCustomer(customerModel, StringUtils.EMPTY);
    }

    @Test(expected = IllegalArgumentException.class)
    public void paymentInfosAreNotSearchedWhenCustomerNotProvided() {
        // When
        pspCustomerAccountService.getPaymentInfos(null, Boolean.TRUE);

        // Then an exception is thrown
    }

    @Test
    public void paymentInfosAreSearchedInDB() {
        // When
        pspCustomerAccountService.getPaymentInfos(customerModel, Boolean.TRUE);

        // Then
        verify(customerAccountDao).findPaymentInfosByCustomer(customerModel, Boolean.TRUE);
    }
}