package com.psp.payment.common.services.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCustomerServiceImplTest {

    @InjectMocks
    private PspCustomerServiceImpl pspCustomerServiceImpl;

    @Mock
    private ModelService modelService;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private Converter<AddressData, CustomerModel> customerNameFromAddressConverter;

    private CustomerModel customerModel;

    @Before
    public void setup() {
        customerModel = new CustomerModel();
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(customerModel);
    }

    @Test
    public void customerNameIsSaved() {
        // Given
        AddressData addressData = new AddressData();

        // When
        pspCustomerServiceImpl.setCustomerNameFromAddress(addressData);

        // Then
        verify(customerNameFromAddressConverter).convert(addressData, customerModel);
        verify(modelService).save(customerModel);
    }

    @Test
    public void customerNameIsNotSavedWhenAddressNotProvided() {
        // When
        pspCustomerServiceImpl.setCustomerNameFromAddress(null);

        // Then
        verify(customerNameFromAddressConverter, never()).convert(any(AddressData.class), any(CustomerModel.class));
        verify(modelService, never()).save(any(CustomerModel.class));
    }
}
