package com.psp.payment.common.services.impl;

import com.psp.payment.common.services.PspDynamicPaymentTemplateService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.site.BaseSiteService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Optional;

import static java.lang.Boolean.FALSE;
import static java.lang.Boolean.TRUE;
import static org.apache.logging.log4j.core.config.plugins.Plugin.EMPTY;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDynamicPaymentTemplateServiceImplTest {

    @InjectMocks
    private PspDynamicPaymentTemplateService pspDynamicPaymentTemplateService = new PspDynamicPaymentTemplateServiceImpl();

    @Mock
    private CMSSiteModel cmsSiteModel;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Before
    public void setup() {
        when(baseSiteService.getCurrentBaseSite()).thenReturn(cmsSiteModel);
        when(cmsSiteModel.getPaymentTemplateFlag()).thenReturn(TRUE);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(cmsSiteModel, TRUE, "relativeUrl"))
                .thenReturn("absoluteUrl");
    }

    @Test
    public void noTemplateFoundForMissingSessionSite() {
        // Given
        when(baseSiteService.getCurrentBaseSite()).thenReturn(null);

        // When
        Optional<String> result = pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl();

        // Then
        assertFalse(result.isPresent());
    }

    @Test
    public void noTemplateFoundIfFunctionalityIsDisabled() {
        // Given
        when(cmsSiteModel.getPaymentTemplateFlag()).thenReturn(FALSE);

        // When
        Optional<String> result = pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl();

        // Then
        assertFalse(result.isPresent());
    }

    @Test
    public void noTemplateFoundForMissingUrl() {
        // Given
        when(cmsSiteModel.getPaymentTemplateUrl()).thenReturn(EMPTY);

        // When
        Optional<String> result = pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl();

        // Then
        assertFalse(result.isPresent());
    }

    @Test
    public void noTemplateFoundForMissingS() {
        // Given
        when(cmsSiteModel.getPaymentTemplateUrl()).thenReturn("relativeUrl");

        // When
        Optional<String> result = pspDynamicPaymentTemplateService.fetchPaymentTemplateUrl();

        // Then
        assertTrue(result.isPresent());
    }
}
