package com.psp.payment.common.services.impl;

import com.psp.model.PayPalPaymentInfoModel;
import com.psp.payment.common.dao.PspPaymentModeDao;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.exceptions.AmbiguousIdentifierException;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentModeServiceImplTest {

    @InjectMocks
    private PspPaymentModeServiceImpl pspPaymentModeService;

    @Mock
    private PspPaymentModeDao pspPaymentModeDao;

    @Mock
    private CartService cartService;

    private CMSSiteModel cmsSiteModel;

    @Before
    public void setup() {
        CartModel cartModel = new CartModel();
        cmsSiteModel = new CMSSiteModel();
        cartModel.setTotalPrice(20.00);
        cartModel.setSite(cmsSiteModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
    }

    @Test
    public void paymentModeIsFound() {
        // Given
        String code = "code";
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModelList.add(new PaymentModeModel());

        when(pspPaymentModeDao.findActivePaymentModeForCode(BigDecimal.valueOf(20.00), code)).thenReturn(paymentModeModelList);

        // When
        pspPaymentModeService.getAvailablePaymentModeForCode(code);

        // Then
        verify(pspPaymentModeDao).findActivePaymentModeForCode(BigDecimal.valueOf(20.00), code);
    }

    @Test(expected = IllegalArgumentException.class)
    public void codeNotProvided() {
        // When
        pspPaymentModeService.getAvailablePaymentModeForCode(null);

        // Then an exception is thrown
    }

    @Test(expected = AmbiguousIdentifierException.class)
    public void moreThanOnePaymentModeFound() {
        // Given
        String code = "code";
        List<PaymentModeModel> paymentModeModelList = new ArrayList<>();
        paymentModeModelList.add(new PaymentModeModel());
        paymentModeModelList.add(new PaymentModeModel());

        when(pspPaymentModeDao.findActivePaymentModeForCode(BigDecimal.valueOf(20.00), code)).thenReturn(paymentModeModelList);

        // When
        pspPaymentModeService.getAvailablePaymentModeForCode(code);

        // Then an exception is thrown
    }

    @Test
    public void allPaymentModesAreFetched() {
        // When
        pspPaymentModeService.getAvailablePaymentMethods();

        // Then
        verify(pspPaymentModeDao).findActivePaymentModes(BigDecimal.valueOf(20.00), cmsSiteModel);
    }

    @Test
    public void paymentModeIsFetchedForPaymentInfo() {
        // Given
        PayPalPaymentInfoModel payPalPaymentInfoModel = new PayPalPaymentInfoModel();

        // When
        pspPaymentModeService.getPaymentModeForPaymentInfo(payPalPaymentInfoModel);

        // Then
        verify(pspPaymentModeDao).findActivePaymentModeForPaymentInfo(payPalPaymentInfoModel);
    }

    @Test(expected = IllegalArgumentException.class)
    public void paymentModeIsNotFetchedWhenPaymentInfoNotProvided() {
        // When
        pspPaymentModeService.getPaymentModeForPaymentInfo(null);

        // Then an exception is thrown
    }
}
