package com.psp.payment.common.strategies.impl;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;

import static org.mockito.Mockito.when;
import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentTransactionStrategyImplTest {
	
	@InjectMocks
	private PspPaymentTransactionStrategyImpl pspPaymentTransactionStrategyImpl;
	
	@Mock
	private ModelService modelService;
	
	@Mock
	private PaymentService paymentService;
	
	@Mock
	private CustomerModel customerModel;
	
	private PaymentTransactionModel transaction;
	
	private PaymentTransactionEntryModel entry;
	
	@Before
	public void setUp(){
		transaction=new PaymentTransactionModel();
		entry=new PaymentTransactionEntryModel();
	}
	
	@Test
	public void testSavePaymentTransactionEntrySuccess(){
		when(modelService.create(PaymentTransactionModel.class)).thenReturn(transaction);
		when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(entry);
		entry = pspPaymentTransactionStrategyImpl.savePaymentTransactionEntry(customerModel, "34F5302C-85D7-4F35-BDF5-103CCEC2FB61");
		assertEquals("34F5302C-85D7-4F35-BDF5-103CCEC2FB61", entry.getRequestToken());
		assertEquals("34F5302C-85D7-4F35-BDF5-103CCEC2FB61", entry.getPaymentTransaction().getRequestToken());
	}

	@Test
	public void testSavePaymentTransactionEntryNullValue(){
		when(modelService.create(PaymentTransactionModel.class)).thenReturn(transaction);
		when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(entry);
		PaymentTransactionEntryModel entry = pspPaymentTransactionStrategyImpl.savePaymentTransactionEntry(customerModel, null);
		assertEquals(entry,null);
	}
	
	@Test
	public void testSavePaymentTransactionEntryEmptyValue(){
		when(modelService.create(PaymentTransactionModel.class)).thenReturn(transaction);
		when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(entry);
		PaymentTransactionEntryModel entry = pspPaymentTransactionStrategyImpl.savePaymentTransactionEntry(customerModel, "");
		assertEquals(entry,null);
	}
}
