package com.psp.payment.common.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.utils.PspUtils;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.anyMap;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspSignatureValidationStrategyImplTest {

    @Spy
    @InjectMocks
    private PspSignatureValidationStrategyImpl pspSignatureValidationStrategyImpl = new PspSignatureValidationStrategyImpl();

    @Mock
    private PspUtils pspUtils;

    private Map<String, String> parameters = new TreeMap<>();

    @Before
    public void setup() {
        parameters.put("Alias.AliasId", "12345");
        parameters.put("Alias.NCError", "0");
        parameters.put("Alias.NCErrorCardNo", "0");
        parameters.put("Alias.NCErrorCN", "0");
        parameters.put("Alias.NCErrorCVC", "0");
        parameters.put("Alias.NCErrorED", "0");
        parameters.put("Alias.OrderId", "100000-0");
        parameters.put("Alias.Status", "0");
        parameters.put("Alias.StorePermanently", "N");
        parameters.put("Card.Brand", "Mastercard");
        parameters.put("Card.CardHolderName", "Test");
        parameters.put("Card.CardNumber", "XXXXXXXXXXXX9999");
        parameters.put("Card.Cvc", "XXX");
        parameters.put("Card.ExpiryDate", "0318");
        parameters.put(PspConstants.KEY_SHASIGN, "00C3C3328759DD5BEDD12B16BB5DD6F6811525B82D6D4E0E11B43D7340612E48");

        when(pspUtils.generateChecksum(anyMap(), anyString())).thenReturn("00C3C3328759DD5BEDD12B16BB5DD6F6811525B82D6D4E0E11B43D7340612E48");
        doReturn("essie1234essie1234").when(pspSignatureValidationStrategyImpl).getParameter(PspConstants.PARAM_PASSPHRASE);
    }

    @Test
    public void testValidateSignatureWithValidData() {
        // Given parameters

        // When
        boolean result = pspSignatureValidationStrategyImpl.validateSignature(parameters, PspConstants.KEY_SHASIGN);

        // Then
        assertTrue(result);
    }

    @Test
    public void testValidateSignatureWithInvalidData() {
        // Given parameters with fake SHA signature
        parameters.put(PspConstants.KEY_SHASIGN, "fake");

        // When
        boolean result = pspSignatureValidationStrategyImpl.validateSignature(parameters, PspConstants.KEY_SHASIGN);

        // Then
        assertFalse(result);
    }

    @Test
    public void testValidateSignatureWithBlankData() {
        // Given
        Map<String, String> specificParameters = new TreeMap<>();

        // When
        boolean result = pspSignatureValidationStrategyImpl.validateSignature(specificParameters, PspConstants.KEY_SHASIGN);

        // Then
        assertFalse(result);
    }

    @Test
    public void testValidateSignatureWithNoData() {
        // Given nothing

        // When
        boolean result = pspSignatureValidationStrategyImpl.validateSignature(null, null);

        // Then
        assertFalse(result);
    }
}
