package com.psp.payment.common.utils;

import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.io.InputStream;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.mockito.Mockito.doReturn;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspExtensionTrackingUtilsTest {

    @Spy
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    @Before
    public void setUp() {
        doReturn("HYB").when(pspExtensionTrackingUtils).getExtensionName();
        doReturn("010317").when(pspExtensionTrackingUtils).getReleaseDate();
        doReturn("10").when(pspExtensionTrackingUtils).getTrackingDetailsLength();
    }

    @Test
    public void trackingKeyIsGenerated() {
        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertEquals("HYB010317", trackingKey);
    }

    @Test
    public void generationFailureForMissingName() {
        // Given
        doReturn(null).when(pspExtensionTrackingUtils).getExtensionName();

        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertNull(trackingKey);
    }

    @Test
    public void generationFailureForMissingDate() {
        // Given
        doReturn(null).when(pspExtensionTrackingUtils).getReleaseDate();

        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertNull(trackingKey);
    }

    @Test
    public void generationFailureForMissingLength() {
        // Given
        doReturn(null).when(pspExtensionTrackingUtils).getTrackingDetailsLength();

        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertNull(trackingKey);
    }

    @Test
    public void generationFailureForKeyGreaterThanLength() {
        // Given
        doReturn("1").when(pspExtensionTrackingUtils).getTrackingDetailsLength();

        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertNull(trackingKey);
    }

    @Test
    public void generationFailureForUnparsableLength() {
        // Given
        doReturn("abc").when(pspExtensionTrackingUtils).getTrackingDetailsLength();

        // When
        String trackingKey = pspExtensionTrackingUtils.fetchTrackingDetails();

        // Then
        assertNull(trackingKey);
    }
}
