package com.psp.payment.directlink.commands.impl;

import com.psp.payment.directlink.dto.PspAuthorizationResult;
import com.psp.payment.directlink.dto.PspSubscriptionAuthorizationRequest;
import com.psp.payment.directlink.request.PspDirectLinkRequest;
import com.psp.payment.directlink.response.PspDirectLinkResponse;
import com.psp.payment.directlink.services.PspDirectLinkService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.payment.commands.result.AuthorizationResult;
import de.hybris.platform.payment.dto.BillingInfo;
import de.hybris.platform.payment.dto.CardInfo;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;
import java.util.Currency;

import static org.junit.Assert.assertNull;
import static org.mockito.BDDMockito.given;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspSubscriptionAuthorizationCommandTest {

    private static final String CURRENCY_CODE = "GBP";
    private static final String MERCHANT_TRANSACTION_CODE = "merchantTransactionCode";
    private static final String SUBSCRIPTION_ID = "subscriptionID";
    private static final String PAYMENT_PROVIDER = "paymentProvider";
    private static final String ORDER_ID = "00006001";
    private static final String CV_2 = "cv2";

    @InjectMocks
    private PspSubscriptionAuthorizationCommand pspSubscriptionAuthorizationCommand;

    @Mock
    private PspDirectLinkService pspDirectLinkService;

    @Mock
    private Converter<PspSubscriptionAuthorizationRequest, PspDirectLinkRequest> pspDirectLinkRequestConverter;

    @Mock
    private Converter<PspDirectLinkResponse, PspAuthorizationResult> pspAuthorizationResultConverter;

    private PspSubscriptionAuthorizationRequest pspSubscriptionAuthorizationRequest;

    private PspDirectLinkRequest pspDirectLinkRequest;

    private AuthorizationResult pspAuthorizationResult;

    @Before
    public void setUp() throws Exception {
        Currency currency = Currency.getInstance(CURRENCY_CODE);
        BillingInfo shippingInfo = new BillingInfo();
        CardInfo card = new CardInfo();
        pspSubscriptionAuthorizationRequest = new PspSubscriptionAuthorizationRequest(MERCHANT_TRANSACTION_CODE, SUBSCRIPTION_ID, currency,
                BigDecimal.ZERO, shippingInfo, CV_2,
                PAYMENT_PROVIDER, ORDER_ID, card);
        pspDirectLinkRequest = new PspDirectLinkRequest();
        pspDirectLinkRequest.setOrderId(ORDER_ID);
    }

    @Test
    public void performWithEmptyRequest() throws Exception {
        // Given
        given(pspDirectLinkRequestConverter.convert(any(PspSubscriptionAuthorizationRequest.class))).willReturn(pspDirectLinkRequest);

        // When
        pspAuthorizationResult = pspSubscriptionAuthorizationCommand.perform(pspSubscriptionAuthorizationRequest);

        // Then
        verify(pspDirectLinkService).executeDirectLinkRequest(pspDirectLinkRequest);
        assertNull(pspAuthorizationResult);

    }

    @Test
    public void performWithNullRequest() throws Exception {
        // Given nothing

        // When
        pspAuthorizationResult = pspSubscriptionAuthorizationCommand.perform(null);

        // Then
        verify(pspDirectLinkService, never()).executeDirectLinkRequest(null);
        assertNull(pspAuthorizationResult);
    }
}