package com.psp.payment.directlink.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.directlink.dto.PspAuthorizationResult;
import com.psp.payment.directlink.response.PspDirectLinkResponse;
import com.psp.payment.directlink.services.PspDirectLinkService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspAuthorizationResultPopulatorTest {

    private static final String ORDER_ID = "ORDER-1";
    private static final String HTML_ANSWER = "1234";
    private static final String AMOUNT = "10";
    private static final String CURRENCY_CODE = "GBP";
    private static final String NC_ERROR = "0";
    private static final String STATUS_CODE = "9";
    private static final String STATUS_IDENTIFICATION_REQUESTED = "46";
    private static final String SECURE_NC_ERROR = "55555555";
    @InjectMocks
    private PspAuthorizationResultPopulator pspAuthorizationResultPopulator;

    @Mock
    private PspDirectLinkService pspDirectLinkService;

    private PspDirectLinkResponse pspDirectLinkResponse;

    private PspAuthorizationResult pspAuthorizationResult;

    @Before
    public void setUp() {
        pspDirectLinkResponse = new PspDirectLinkResponse();
        pspDirectLinkResponse.setParameters(getDirectLinkResponseParameters(STATUS_CODE, null));
        pspAuthorizationResult = new PspAuthorizationResult();

        when(pspDirectLinkService.getTransactionStatus(getDirectLinkResponseParameters(STATUS_CODE, null))).thenReturn(TransactionStatus.ACCEPTED.name());
        when(pspDirectLinkService.getTransactionStatusDetails(getDirectLinkResponseParameters(STATUS_CODE, null))).thenReturn(TransactionStatusDetails.SUCCESFULL);
    }

    @Test
    public void testPopulate() {
        // When
        pspAuthorizationResultPopulator.populate(pspDirectLinkResponse, pspAuthorizationResult);

        // Then
        assertTrue(pspAuthorizationResult.getOrderId().equals(ORDER_ID));
    }

    @Test
    public void testPopulate3DSecure() {
        // Given
        PspDirectLinkResponse directLink3DSecureResponse = new PspDirectLinkResponse();
        directLink3DSecureResponse.setParameters(getDirectLinkResponseParameters(STATUS_IDENTIFICATION_REQUESTED, HTML_ANSWER));

        // When
        pspAuthorizationResultPopulator.populate(directLink3DSecureResponse, pspAuthorizationResult);

        // Then
        assertTrue(pspAuthorizationResult.getHtmlAnswer().equals(HTML_ANSWER));
    }

    @Test(expected = IllegalArgumentException.class)
    public void testPopulateSourceNull() {
        // When
        pspAuthorizationResultPopulator.populate(null, pspAuthorizationResult);

        // Then an exception will be thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void testPopulateTargetNull() {
        // When
        pspAuthorizationResultPopulator.populate(pspDirectLinkResponse, null);

        // Then an exception will be thrown
    }

    private Map<String, String> getDirectLinkResponseParameters(String status, String html) {
        Map<String, String> parameters = new TreeMap<>();
        parameters.put(PspConstants.KEY_DL_ORDERID, ORDER_ID);
        parameters.put((PspConstants.KEY_RESULT_AMOUNT), AMOUNT);
        parameters.put((PspConstants.KEY_CURRENCY), CURRENCY_CODE);
        parameters.put(PspConstants.KEY_NCERROR, NC_ERROR);
        parameters.put(PspConstants.KEY_STATUS, status);
        parameters.put(PspConstants.KEY_HTML_ANSWER, html);
        return parameters;
    }
}