package com.psp.payment.directlink.strategies.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import org.apache.commons.lang.StringUtils;
import org.junit.Test;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

@UnitTest
public class PspAddressBuilderStrategyImplTest {

    private PspAddressBuilderStrategyImpl pspAddressBuilderStrategy = new PspAddressBuilderStrategyImpl();

    private String addressLine1;

    private String addressLine2;

    @Test
    public void testBuildAddressWhenBothAddressProvided() {
        // Given
        addressLine1 = "The Varnish Works";
        addressLine2 = "3 Bravingtons Walk";

        // When
        String result = pspAddressBuilderStrategy.buildAddress(addressLine1, addressLine2);

        // Then
        String expectedResult = "The Varnish Works, 3 Bravingtons Walk";
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testBuildAddressWhenAddress1Provided() {
        // Given
        addressLine1 = "The Varnish Works";

        // When
        String result = pspAddressBuilderStrategy.buildAddress(addressLine1, addressLine2);

        // Then
        String expectedResult = "The Varnish Works";
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testBuildAddressWhenAddress2Provided() {
        // Given
        addressLine2 = "3 Bravingtons Walk";

        // When
        String result = pspAddressBuilderStrategy.buildAddress(addressLine1, addressLine2);

        // Then
        String expectedResult = "3 Bravingtons Walk";
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testBuildAddressWhenNothingIsProvided() {
        // Given nothing

        // When
        String result = pspAddressBuilderStrategy.buildAddress(addressLine1, addressLine2);

        // Then
        String expectedResult = StringUtils.EMPTY;
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testBuildAddressWhenBlankLine1AndLine2AreProvided() {
        // Given
        addressLine1 = StringUtils.EMPTY;
        addressLine2 = StringUtils.EMPTY;

        // When
        String result = pspAddressBuilderStrategy.buildAddress(addressLine1, addressLine2);

        // Then
        String expectedResult = StringUtils.EMPTY;
        assertThat(expectedResult, is(result));
    }
}
