package com.psp.payment.directquery.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectQueryPopulatorTest {
    private static final String PAY_ID = "Pay Id";
    private static final String PSP_ID = "Psp Id";
    private static final String PASSWORD = "Password";
    private static final String USER_ID = "User Id";


    @InjectMocks
    private Populator<String, Map<String, String>> populator = new PspDirectQueryPopulator();

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    private Map<String, String> request;

    @Before
    public void setUp() {
        request = new HashMap<>();

        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PSPID)).thenReturn(PSP_ID);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_API_USER_PASSWORD)).thenReturn(PASSWORD);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_API_USER_ID)).thenReturn(USER_ID);
    }

    @Test
    public void payId() {
        // When
        populator.populate(PAY_ID, request);

        // Then
        assertEquals(PAY_ID, request.get(PspConstants.KEY_PAYID));
    }

    @Test
    public void pspId() {
        // When
        populator.populate(PAY_ID, request);

        // Then
        assertEquals(PSP_ID, request.get(PspConstants.KEY_PSPID));
    }

    @Test
    public void password() {
        // When
        populator.populate(PAY_ID, request);

        // Then
        assertEquals(PASSWORD, request.get(PspConstants.KEY_API_PASSWORD));
    }

    @Test
    public void userId() {
        populator.populate(PAY_ID, request);

        assertEquals(USER_ID, request.get(PspConstants.KEY_API_USERID));
    }


    @Test
    public void trackingKeyIsPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn("HYB010317");

        // When
        populator.populate(PAY_ID, request);

        // Then
        assertEquals("HYB010317", request.get(PspConstants.KEY_TRACKING));
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn(null);

        // When
        populator.populate(PAY_ID, request);

        // Then
        assertNull(request.get(PspConstants.KEY_TRACKING));
    }
}
