package com.psp.payment.directquery.services.impl;

import com.psp.payment.common.dto.PspTransactionResponse;
import com.psp.payment.directquery.services.PspDirectQueryService;
import com.psp.payment.integration.clients.PspRestClient;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.dto.converter.ConversionException;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectQueryServiceImplTest {
    private static final String PAY_ID = "Pay Id";
    private static final String KEY = "psp.conf.directquery.request.url";
    private static final String VALID_URL = "http://anyUri";
    private static final String INVALID_URI = "http://whatever.com?s=^IXIC";

    @InjectMocks
    private PspDirectQueryService pspDirectQueryService = new PspDirectQueryServiceImpl();

    @Mock
    private Converter<String, Map<String, String>> pspDirectQueryConverter;

    @Mock
    private PspRestClient<PspTransactionResponse> pspRestClient;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private PaymentTransactionModel paymentTransactionModel;

    private PspTransactionResponse pspTransactionResponse;

    @Before
    public void setUp() throws URISyntaxException {
        Map<String, String> directQueryRequest = new HashMap<>();
        URI uri = new URI(VALID_URL);
        pspTransactionResponse = new PspTransactionResponse();

        when(configurationService.getConfiguration().getString(KEY)).thenReturn(VALID_URL);
        when(pspDirectQueryConverter.convert(PAY_ID)).thenReturn(directQueryRequest);
        when(pspRestClient.postRequest(uri, directQueryRequest, PspTransactionResponse.class)).thenReturn(pspTransactionResponse);
    }

    @Test
    public void noUriForEndpoint() {
        // Given
        when(configurationService.getConfiguration().getString(KEY)).thenReturn(null);

        // When
        Optional<PspTransactionResponse> response = pspDirectQueryService.readPaymentStatus(PAY_ID);

        // Then
        assertFalse(response.isPresent());
    }

    @Test
    public void invalidUriForEndpoint() {
        // Given
        when(configurationService.getConfiguration().getString(KEY)).thenReturn(INVALID_URI);

        // When
        Optional<PspTransactionResponse> response = pspDirectQueryService.readPaymentStatus(PAY_ID);

        // Then
        assertFalse(response.isPresent());
    }

    @Test
    public void populatorFailure() {
        // Given
        when(pspDirectQueryConverter.convert(PAY_ID)).thenThrow(new ConversionException("test"));

        // When
        Optional<PspTransactionResponse> response = pspDirectQueryService.readPaymentStatus(PAY_ID);

        // Then
        assertFalse(response.isPresent());
    }

    @Test
    public void getsResponse() {
        // When
        PspTransactionResponse response = pspDirectQueryService.readPaymentStatus(PAY_ID).get();

        // Then
        assertEquals(pspTransactionResponse, response);
    }
}
