package com.psp.payment.flexpayments.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentCallbackUrlPopulatorTest {

    private static final String RESPONSE_URL = "https://apparel-uk.local:9002/yacceleratorstorefront/checkout/psp/response";
    @InjectMocks
    private PspFlexPaymentCallbackUrlPopulator pspFlexPaymentCallbackUrlPopulator;
    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Before
    public void setup() {
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void populateWithValidData() {

        //given
        pspFlexPaymentRequest.setAcceptUrl(RESPONSE_URL);
        pspFlexPaymentRequest.setDeclineUrl(RESPONSE_URL);
        pspFlexPaymentRequest.setExceptionUrl(RESPONSE_URL);
        pspFlexPaymentRequest.setCancelUrl(RESPONSE_URL);

        // When
        pspFlexPaymentCallbackUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_ACCEPTURL), is(RESPONSE_URL));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_EXCEPTIONURL), is(RESPONSE_URL));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_DECLINEURL), is(RESPONSE_URL));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_CANCELURL), is(RESPONSE_URL));


    }

    @Test
    public void populateWithEmptyData() {
        // Given
        pspFlexPaymentRequest.setAcceptUrl(StringUtils.EMPTY);
        pspFlexPaymentRequest.setDeclineUrl(StringUtils.EMPTY);
        pspFlexPaymentRequest.setExceptionUrl(StringUtils.EMPTY);
        pspFlexPaymentRequest.setCancelUrl(StringUtils.EMPTY);


        // When
        pspFlexPaymentCallbackUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_ACCEPTURL), is(StringUtils.EMPTY));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_EXCEPTIONURL), is(StringUtils.EMPTY));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_DECLINEURL), is(StringUtils.EMPTY));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_CANCELURL), is(StringUtils.EMPTY));

    }

    @Test
    public void populateWithNoData() {
        // Given nothing

        // When
        pspFlexPaymentCallbackUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertNull(paymentData.getParameters().get(PspConstants.KEY_ACCEPTURL));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_EXCEPTIONURL));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_DECLINEURL));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_CANCELURL));
    }

}