package com.psp.payment.flexpayments.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentCustomerInfoPopulatorTest {
    private static final String PSP_ID = "1000";
    private static final String CUSTOMER_NAME = "Name";
    private static final String OWNER_TELEPHONE_NUMBER = "1234567890";
    private static final String OWNER_COUNTRY = "UK";
    @InjectMocks
    private PspFlexPaymentCustomerInfoPopulator pspFlexPaymentCustomerInfoPopulator;

    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Before
    public void setup() {
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void populateWithValidData() {

        //given
        pspFlexPaymentRequest.setPspId(PSP_ID);
        pspFlexPaymentRequest.setCustomerName(CUSTOMER_NAME);
        pspFlexPaymentRequest.setOwnerTelephoneNumber(OWNER_TELEPHONE_NUMBER);
        pspFlexPaymentRequest.setOwnerCountry(OWNER_COUNTRY);

        pspFlexPaymentCustomerInfoPopulator.populate(pspFlexPaymentRequest, paymentData);
        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_PSPID), is(PSP_ID));

    }

    @Test
    public void populateWithEmptyData() {
        // Given
        pspFlexPaymentRequest.setPspId(StringUtils.EMPTY);

        // When
        pspFlexPaymentCustomerInfoPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_PSPID), is(StringUtils.EMPTY));
    }

    @Test
    public void populateWithNoData() {
        // Given nothing

        // When
        pspFlexPaymentCustomerInfoPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertNull(paymentData.getParameters().get(PspConstants.KEY_FLEX_PSPID));
    }

}