package com.psp.payment.flexpayments.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentOrderInfoPopulatorTest {
    private static final String CURRENCY = "GBP";
    private static final String PAYMENT_METHOD = "Paypal";
    private static final String ORDER_ID = "100000-0";

    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @InjectMocks
    private PspFlexPaymentOrderInfoPopulator pspFlexPaymentOrderInfoPopulator;

    @Before
    public void setUp() {
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        paymentData = new PaymentData();


    }

    @Test
    public void populateWithValidData() {

        //given
        pspFlexPaymentRequest.setAmount(BigDecimal.ZERO);
        pspFlexPaymentRequest.setCurrency(CURRENCY);
        pspFlexPaymentRequest.setPaymentMethod(PAYMENT_METHOD);
        pspFlexPaymentRequest.setOrderId(ORDER_ID);

        pspFlexPaymentOrderInfoPopulator.populate(pspFlexPaymentRequest, paymentData);
        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_ORDERID), is(ORDER_ID));

    }

    @Test
    public void populateWithEmptyData() {
        // Given
        pspFlexPaymentRequest.setOrderId(StringUtils.EMPTY);

        // When
        pspFlexPaymentOrderInfoPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_ORDERID), is(StringUtils.EMPTY));
    }

    @Test
    public void populateWithNoData() {
        // Given nothing

        // When
        pspFlexPaymentOrderInfoPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertNull(paymentData.getParameters().get(PspConstants.KEY_FLEX_ORDERID));
    }
}
