package com.psp.payment.flexpayments.populators;

import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;
import static org.mockito.BDDMockito.given;
import static org.mockito.Matchers.anyString;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentPostUrlPopulatorTest {
    private static final String POST_URL = "http://www.test.com";
    @InjectMocks
    private PspFlexPaymentPostUrlPopulator pspFlexPaymentPostUrlPopulator;

    private PspFlexPaymentRequest pspFlexPaymentRequest = new PspFlexPaymentRequest();
    private PaymentData paymentData = new PaymentData();

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Test
    public void populateWithValidData() {
        // Given
        given(configurationService.getConfiguration().getString(anyString())).willReturn(POST_URL);

        // When
        pspFlexPaymentPostUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getPostUrl());
        assertThat(paymentData.getPostUrl(), is(POST_URL));
    }

    @Test
    public void populateWithEmptyData() {
        // Given
        given(configurationService.getConfiguration().getString(anyString())).willReturn(StringUtils.EMPTY);

        // When
        pspFlexPaymentPostUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertThat(paymentData.getPostUrl(), is(StringUtils.EMPTY));
    }

    @Test
    public void populateWithNoData() {
        // Given
        given(configurationService.getConfiguration().getString(anyString())).willReturn(null);

        // When
        pspFlexPaymentPostUrlPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNull(paymentData.getPostUrl());
    }

}