package com.psp.payment.flexpayments.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentSignaturePopulatorTest {
    private static final String SHA_SIGN = "27F8153B841E058072D11876FE7A4AB96ED1281A1B03C4E24F110FE23F887AEB";
    @InjectMocks
    private PspFlexPaymentSignaturePopulator pspFlexPaymentSignaturePopulator ;
    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Before
    public void setup() {
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void populateWithValidData() {

        //given
        pspFlexPaymentRequest.setShaSign(SHA_SIGN);

        pspFlexPaymentSignaturePopulator.populate(pspFlexPaymentRequest, paymentData);
        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_SHA_SIGNATURE),is(pspFlexPaymentRequest.getShaSign()));
    }

    @Test
    public void populateWithEmptyData() {
        // Given
        pspFlexPaymentRequest.setShaSign(StringUtils.EMPTY);


        // When
        pspFlexPaymentSignaturePopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().get(PspConstants.KEY_FLEX_SHA_SIGNATURE),is(StringUtils.EMPTY));

    }

    @Test
    public void testPopulateWithNoData() {
        // Given nothing

        // When
        pspFlexPaymentSignaturePopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertNull(paymentData.getParameters().get(PspConstants.KEY_FLEX_SHA_SIGNATURE));

    }


}