package com.psp.payment.flexpayments.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.flexpayments.dto.PspFlexPaymentRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFlexPaymentTrackingPopulatorTest {

    @InjectMocks
    private PspFlexPaymentTrackingPopulator pspFlexPaymentTrackingPopulator;

    private PspFlexPaymentRequest pspFlexPaymentRequest;

    private PaymentData paymentData;

    @Before
    public void setup() {
        pspFlexPaymentRequest = new PspFlexPaymentRequest();
        paymentData = new PaymentData();
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateFailsOnInvalidSource() {
        // When
        pspFlexPaymentTrackingPopulator.populate(null, paymentData);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateFailsOnInvalidTarget() {
        // When
        pspFlexPaymentTrackingPopulator.populate(pspFlexPaymentRequest, null);

        // Then an exception is thrown
    }

    @Test
    public void trackingKeyIsPopulated() {
        // Given
        pspFlexPaymentRequest.setTrackingKey("HYB010317");

        // When
        pspFlexPaymentTrackingPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertEquals(1, paymentData.getParameters().size());
        assertEquals("HYB010317", paymentData.getParameters().get(PspConstants.KEY_TRACKING));
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        pspFlexPaymentRequest.setTrackingKey(null);

        // When
        pspFlexPaymentTrackingPopulator.populate(pspFlexPaymentRequest, paymentData);

        // Then
        assertEquals(0, paymentData.getParameters().size());
    }
}
