package com.psp.payment.flexpayments.strategies.impl;

import com.psp.model.GenericFlexPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspGenericFlexPaymentInfoCreateStrategyImplTest {

    @InjectMocks
    private PspGenericFlexPaymentInfoCreateStrategyImpl pspGenericFlexPaymentInfoCreateStrategyImpl;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private CartModel cartModel;

    @Mock
    private CustomerModel customerModel;

    @Mock
    private ModelService modelService;

    private GenericFlexPaymentInfoModel genericFlexPaymentInfoModel;

    @Before
    public void setup() {
        genericFlexPaymentInfoModel = new GenericFlexPaymentInfoModel();

        when(cartModel.getUser()).thenReturn(customerModel);
        when(customerModel.getUid()).thenReturn("uid");
        when(cartModel.getPaymentMode().getBrand()).thenReturn("brand");
        when(modelService.create(GenericFlexPaymentInfoModel.class)).thenReturn(genericFlexPaymentInfoModel);
    }

    @Test
    public void modelIsCreated() {

        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(cartModel);

        // Then
        verify(modelService).create(GenericFlexPaymentInfoModel.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void exceptionIsThrownOnInvalidInputs() {
        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(null);

        // Then an exception is thrown
    }

    @Test
    public void customerIsSaved() {
        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(cartModel);

        // Then
        assertEquals(customerModel, genericFlexPaymentInfoModel.getUser());
    }

    @Test
    public void brandIsSaved() {
        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(cartModel);

        // Then
        assertEquals("brand", genericFlexPaymentInfoModel.getBrand());
    }

    @Test
    public void allModifiedModelsAreSaved() {
        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(cartModel);

        // Then
        verify(modelService).saveAll(genericFlexPaymentInfoModel, cartModel);
    }

    @Test
    public void codeIsSavedStartingWithCustomerUid() {
        // When
        pspGenericFlexPaymentInfoCreateStrategyImpl.saveSubscription(cartModel);

        // Then
        assertTrue(genericFlexPaymentInfoModel.getCode().startsWith("uid"));
    }
}
