package com.psp.payment.flexpayments.strategies.impl;


import com.psp.model.IDealPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspIDealPaymentInfoCreateStrategyImplTest {
    @InjectMocks
    private PspIDealPaymentInfoCreateStrategyImpl pspIDealPaymentInfoCreateStrategy;
    @Mock
    private ModelService modelService;

    private IDealPaymentInfoModel iDealPaymentInfoModel;
    private CartModel cartModel;

    private CustomerModel customerModel;

    private AddressModel addressModel;

    private PaymentModeModel paymentModeModel;

    @Before
    public void setup() {
        customerModel = new CustomerModel();
        customerModel.setUid("test");
        addressModel = new AddressModel();
        cartModel = new CartModel();
        cartModel.setUser(customerModel);
        cartModel.setPaymentAddress(addressModel);
        iDealPaymentInfoModel = new IDealPaymentInfoModel();
        paymentModeModel = new PaymentModeModel();
        paymentModeModel.setBrand("iDeal");
        cartModel.setPaymentMode(paymentModeModel);
    }

    @Test
    public void iDealPaymentInfoIsPopulatedAsExpected() {
        // Given
        when(modelService.create(IDealPaymentInfoModel.class)).thenReturn(iDealPaymentInfoModel);

        // When
        pspIDealPaymentInfoCreateStrategy.saveSubscription(cartModel);

        // Then
        assertTrue(iDealPaymentInfoModel.getCode().startsWith(customerModel.getUid() + "_"));
        assertEquals(customerModel, iDealPaymentInfoModel.getUser());
        assertEquals(addressModel, iDealPaymentInfoModel.getBillingAddress());
        assertEquals(iDealPaymentInfoModel, cartModel.getPaymentInfo());
        verify(modelService).saveAll(iDealPaymentInfoModel, cartModel);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void shouldThrowUnsupportedOperationException() {
        // Given
        //nothing

        // When
        pspIDealPaymentInfoCreateStrategy.saveSubscription(customerModel,null);


        // Then
        //throws exception

    }

}