package com.psp.payment.flexpayments.strategies.impl;

import com.psp.model.PayPalPaymentInfoModel;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPayPalPaymentInfoCreateStrategyImplTest {

    @InjectMocks
    private PspPayPalPaymentInfoCreateStrategyImpl pspPayPalPaymentInfoCreateStrategy;

    @Mock
    private ModelService modelService;

    @Mock
    private PayPalPaymentInfoModel payPalPaymentInfoModel;

    private CartModel cartModel;

    private CustomerModel customerModel;

    private AddressModel addressModel;

    private PaymentModeModel paymentModeModel;

    @Before
    public void setup() {
        customerModel = new CustomerModel();
        customerModel.setUid("test");
        addressModel = new AddressModel();
        cartModel = new CartModel();
        cartModel.setUser(customerModel);
        cartModel.setPaymentAddress(addressModel);
        payPalPaymentInfoModel = new PayPalPaymentInfoModel();
        paymentModeModel = new PaymentModeModel();
        paymentModeModel.setBrand("Paypal");
        cartModel.setPaymentMode(paymentModeModel);
    }

    @Test
    public void payPalPaymentInfoIsPopulatedAsExpected() {
        // Given
        when(modelService.create(PayPalPaymentInfoModel.class)).thenReturn(payPalPaymentInfoModel);

        // When
        pspPayPalPaymentInfoCreateStrategy.saveSubscription(cartModel);

        // Then
        assertTrue(payPalPaymentInfoModel.getCode().startsWith(customerModel.getUid() + "_"));
        assertEquals(customerModel, payPalPaymentInfoModel.getUser());
        assertEquals(addressModel, payPalPaymentInfoModel.getBillingAddress());
        assertEquals(payPalPaymentInfoModel, cartModel.getPaymentInfo());
        verify(modelService).saveAll(payPalPaymentInfoModel, cartModel);
    }
}
