package com.psp.payment.fraud.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.enums.FraudStatus;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.fraud.impl.FraudSymptom;
import de.hybris.platform.fraud.model.FraudReportModel;
import de.hybris.platform.fraud.model.FraudSymptomScoringModel;
import de.hybris.platform.orderhistory.model.OrderHistoryEntryModel;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.time.TimeService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFraudServiceImplTest {

    private static String PROVIDER_NAME = "psp";

    @InjectMocks
    private PspFraudServiceImpl fraudService;

    private FraudServiceResponse response;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private OrderModel order;
    @Mock
    private FraudStatus status;
    @Mock
    private ModelService modelService;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private TimeService timeService;

    @Mock
    private FraudReportModel fraudReport;

    @Mock
    private OrderHistoryEntryModel orderHistoryEntryModel;

    @Mock
    private FraudSymptomScoringModel symptomScoring;

    private FraudSymptom symptom;

    @Before
    public void setUp() throws Exception {
        symptom = new FraudSymptom("PspFraudScoreSymptom", 50);
        response = new FraudServiceResponse(PROVIDER_NAME);
        response.addSymptom(symptom);
        when(modelService.create(FraudReportModel.class)).thenReturn(fraudReport);
        when(modelService.create(FraudSymptomScoringModel.class)).thenReturn(symptomScoring);
        when(modelService.create(OrderHistoryEntryModel.class)).thenReturn(orderHistoryEntryModel);
    }


    @Test
    public void createFraudReport() throws Exception {
        // Given nothing

        // When
        FraudReportModel reportModel = fraudService.createFraudReport(PROVIDER_NAME, response, order, FraudStatus.CHECK);

        //Then
        assertNotNull(reportModel.getFraudSymptomScorings());

    }

    @Test
    public void createHistoryLog() throws Exception {
        // Given

        // When
        OrderHistoryEntryModel orderHistoryEntryModel = fraudService.createHistoryLog(PROVIDER_NAME, order, FraudStatus.CHECK, "123");

        //Then
        assertNotNull(orderHistoryEntryModel);

    }
    @Test
    public void createHistoryLogWithFraudOKStatus() throws Exception {
        // Given

        // When
        OrderHistoryEntryModel orderHistoryEntryModel = fraudService.createHistoryLog(PROVIDER_NAME, order, FraudStatus.OK, "123");

        //Then
        assertNotNull(orderHistoryEntryModel);

    }


}