package com.psp.payment.fraud.impl;
/**
 * Created by mkandalie on 13/10/2016.
 */

import com.psp.payment.fraud.symptom.impl.PspFraudScoreSymptom;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.user.UserModel;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import org.apache.commons.lang.NotImplementedException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.InjectMocks;

import java.util.Arrays;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFraudServiceProviderTest {

    @InjectMocks
    private PspFraudServiceProvider pspFraudServiceProvider;

    private final String ORDER_CODE = "123";
    private final String PAYMENT_PROVIDER = "psp";

    private OrderModel order;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private PaymentTransactionModel paymentTransactionModel;

    private PspFraudScoreSymptom pspFraudScoreSymptom;

    @Mock
    private UserModel userModel;

    @Before
    public void setUp(){
        paymentTransactionModel = getTransaction(TransactionStatusDetails.REVIEW_NEEDED.name());
        order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode(ORDER_CODE);
        paymentTransactionModel.setOrder(order);
        pspFraudScoreSymptom = new PspFraudScoreSymptom();
        pspFraudServiceProvider.setSymptomList(Arrays.asList(pspFraudScoreSymptom));
    }


    @Test
    public void recognizeOrderFraudSymptoms() throws Exception {
        // Given
        //nothing

        // When
        FraudServiceResponse fraudServiceResponse = pspFraudServiceProvider.recognizeOrderFraudSymptoms(order);
        //Then
        Assert.assertEquals(50, fraudServiceResponse.getScore(),0);
    }

    @Test(expected = NotImplementedException.class)
    public void recognizeUserActivitySymptoms() throws Exception {
        // Given
        // nothing

        // When
        pspFraudServiceProvider.recognizeUserActivitySymptoms(userModel);

        //Then
        // nothing to verify

    }

    private PaymentTransactionModel getTransaction(String status) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionEntryModel.setTransactionStatusDetails(status);
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));
        return paymentTransactionModel;
    }


}