package com.psp.payment.fraud.symptom.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.fraud.impl.FraudServiceResponse;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;

import static org.junit.Assert.assertTrue;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspFraudScoreSymptomTest {
    private final String ORDER_CODE = "123";
    private final String PAYMENT_PROVIDER = "psp";
    @InjectMocks
    private PspFraudScoreSymptom pspFraudScoreSymptom;

    private FraudServiceResponse response;

    private OrderModel order;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private PaymentTransactionModel paymentTransactionModel;

    @Before
    public void setUp() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(TransactionStatusDetails.REVIEW_NEEDED.name());
        response = new FraudServiceResponse(PAYMENT_PROVIDER);
        order = new OrderModel();
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        order.setCode(ORDER_CODE);
        paymentTransactionModel.setOrder(order);
    }

    @Test
    public void recognizeSymptomWithFraudScore() throws Exception {
        // Given nothing
        // When
        response = pspFraudScoreSymptom.recognizeSymptom(response, order);
        //Then
        assertTrue(0 < response.getScore());
    }

    @Test
    public void recognizeSymptomWithNoFraudScore() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(TransactionStatusDetails.SUCCESFULL.name());
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        paymentTransactionModel.setOrder(order);
        // When
        response = pspFraudScoreSymptom.recognizeSymptom(response, order);
        //Then
        assertTrue(0 == response.getScore());

    }
    @Test
    public void recognizeSymptomWithReview() throws Exception {
        // Given
        paymentTransactionModel = getTransaction(TransactionStatusDetails.SUCCESFULL.name());
        paymentTransactionEntryModel.setTransactionStatus(TransactionStatus.REVIEW.name());
        order.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        paymentTransactionModel.setOrder(order);
        // When
        response = pspFraudScoreSymptom.recognizeSymptom(response, order);
        //Then
        assertTrue(0 < response.getScore());

    }

    private PaymentTransactionModel getTransaction(String status) {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionEntryModel.setPaymentTransaction(paymentTransactionModel);
        paymentTransactionEntryModel.setTransactionStatusDetails(status);
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));
        return paymentTransactionModel;
    }

}