package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.payment.common.services.PspPaymentProcessService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;


@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCheckAuthorizeOrderPaymentActionTest {

    @InjectMocks
    private PspCheckAuthorizeOrderPaymentAction checkAuthorizeOrderPayment;

    @Mock
    private ModelService modelService;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    private OrderProcessModel businessProcessModel;

    private OrderModel order;

    private CreditCardPaymentInfoModel creditCardPaymentInfoModel;

    private PaymentTransactionModel paymentTransaction;

    private PaymentTransactionEntryModel entry;

    @Before
    public void setup() {
        businessProcessModel = new OrderProcessModel();
        order = new OrderModel();
        businessProcessModel.setOrder(order);
        creditCardPaymentInfoModel = new CreditCardPaymentInfoModel();
        paymentTransaction = new PaymentTransactionModel();
        entry = new PaymentTransactionEntryModel();
        entry.setType(PaymentTransactionType.CAPTURE);
        order.setPaymentInfo(creditCardPaymentInfoModel);
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(PaymentTransactionType.CAPTURE);
    }

    @Test
    public void testExecuteActionForAcceptedStatus() {
        // Given
        entry.setTransactionStatus(TransactionStatus.ACCEPTED.name());
        paymentTransaction.setEntries(Arrays.asList(entry));
        order.setPaymentTransactions(Arrays.asList(paymentTransaction));

        // When
        Transition result = checkAuthorizeOrderPayment.executeAction(businessProcessModel);

        // Then
        Transition expectedResult = Transition.OK;
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testExecuteActionForReviewStatus() {
        // Given
        entry.setTransactionStatus(TransactionStatus.REVIEW.name());
        paymentTransaction.setEntries(Arrays.asList(entry));
        order.setPaymentTransactions(Arrays.asList(paymentTransaction));

        // When
        Transition result = checkAuthorizeOrderPayment.executeAction(businessProcessModel);

        // Then
        Transition expectedResult = Transition.OK;
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testExecuteActionForAnyOtherStatus() {
        // Given
        entry.setTransactionStatus(TransactionStatus.REJECTED.name());
        paymentTransaction.setEntries(Arrays.asList(entry));
        order.setPaymentTransactions(Arrays.asList(paymentTransaction));

        // When
        Transition result = checkAuthorizeOrderPayment.executeAction(businessProcessModel);

        // Then
        Transition expectedResult = Transition.NOK;
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testExecuteActionForAnyOtherType() {
        // Given
        entry.setType(PaymentTransactionType.AUTHORIZATION);
        entry.setTransactionStatus(TransactionStatus.ACCEPTED.name());
        paymentTransaction.setEntries(Arrays.asList(entry));
        order.setPaymentTransactions(Arrays.asList(paymentTransaction));

        // When
        Transition result = checkAuthorizeOrderPayment.executeAction(businessProcessModel);

        // Then
        Transition expectedResult = Transition.NOK;
        assertThat(expectedResult, is(result));
    }
}
