package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.payment.common.services.PspPaymentProcessService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import static com.psp.enums.PspTransactionStatus.UNCERTAIN;
import static de.hybris.platform.payment.dto.TransactionStatus.ACCEPTED;
import static de.hybris.platform.payment.dto.TransactionStatus.REVIEW;
import static de.hybris.platform.payment.enums.PaymentTransactionType.AUTHORIZATION;
import static de.hybris.platform.payment.enums.PaymentTransactionType.CAPTURE;
import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCheckTransactionReviewStatusActionTest {

    @InjectMocks
    private PspCheckTransactionReviewStatusAction pspCheckTransactionReviewStatusAction;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    @Mock
    private ModelService modelService;

    private OrderProcessModel orderProcessModel;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    @Before
    public void setup() {
        PaymentTransactionModel paymentTransactionModel = new PaymentTransactionModel();
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        paymentTransactionModel.setEntries(Arrays.asList(paymentTransactionEntryModel));
        OrderModel orderModel = new OrderModel();
        orderModel.setPaymentTransactions(Arrays.asList(paymentTransactionModel));
        PaymentInfoModel paymentInfoModel = new PaymentInfoModel();
        orderModel.setPaymentInfo(paymentInfoModel);
        orderProcessModel = new OrderProcessModel();
        orderProcessModel.setOrder(orderModel);
    }

    @Test
    public void actionGivesOkForCaptureAccepted() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(CAPTURE);
        paymentTransactionEntryModel.setType(CAPTURE);
        paymentTransactionEntryModel.setTransactionStatus(ACCEPTED.name());

        // When
        String result = pspCheckTransactionReviewStatusAction.execute(orderProcessModel);

        // Then
        assertEquals("OK", result);
    }

    @Test
    public void actionGivesOkForAuthorisationAccepted() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(AUTHORIZATION);
        paymentTransactionEntryModel.setType(AUTHORIZATION);
        paymentTransactionEntryModel.setTransactionStatus(ACCEPTED.name());

        // When
        String result = pspCheckTransactionReviewStatusAction.execute(orderProcessModel);

        // Then
        assertEquals("OK", result);
    }

    @Test
    public void actionGivesOkForAuthorisationReview() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(AUTHORIZATION);
        paymentTransactionEntryModel.setType(AUTHORIZATION);
        paymentTransactionEntryModel.setTransactionStatus(REVIEW.name());

        // When
        String result = pspCheckTransactionReviewStatusAction.execute(orderProcessModel);

        // Then
        assertEquals("OK", result);
    }

    @Test
    public void actionGivesWaitForCaptureUncertain() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(CAPTURE);
        paymentTransactionEntryModel.setType(CAPTURE);
        paymentTransactionEntryModel.setTransactionStatus(UNCERTAIN.name());

        // When
        String result = pspCheckTransactionReviewStatusAction.execute(orderProcessModel);

        // Then
        assertEquals("WAIT", result);
    }

    @Test
    public void actionGivesOkForAnyOtherCombination() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(PaymentInfoModel.class), any(BaseSiteModel.class))).thenReturn(CAPTURE);
        paymentTransactionEntryModel.setType(AUTHORIZATION);
        paymentTransactionEntryModel.setTransactionStatus(UNCERTAIN.name());

        // When
        String result = pspCheckTransactionReviewStatusAction.execute(orderProcessModel);

        // Then
        assertEquals("OK", result);
    }

    @Test
    public void expectedTransitionsAreThere() {
        // Given
        Set<String> expectedTransitions = new HashSet<>();
        expectedTransitions.add("OK");
        expectedTransitions.add("NOK");
        expectedTransitions.add("WAIT");

        // When
        Set<String> transitions = pspCheckTransactionReviewStatusAction.getTransitions();

        // Then
        assertEquals(expectedTransitions, transitions);
    }
}
