package com.psp.payment.fulfilmentprocess.actions.order;

import com.psp.payment.common.services.PspPaymentProcessService;
import com.psp.payment.maintenance.exceptions.IllegalMaintenanceOperation;
import com.psp.payment.maintenance.services.PspMaintenanceService;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.orderprocessing.model.OrderProcessModel;
import de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.math.BigDecimal;

import static de.hybris.platform.core.enums.OrderStatus.PAYMENT_CAPTURED;
import static de.hybris.platform.core.enums.OrderStatus.PAYMENT_NOT_CAPTURED;
import static de.hybris.platform.payment.enums.PaymentTransactionType.AUTHORIZATION;
import static de.hybris.platform.payment.enums.PaymentTransactionType.CAPTURE;
import static de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition.NOK;
import static de.hybris.platform.processengine.action.AbstractSimpleDecisionAction.Transition.OK;
import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspTakePaymentActionTest {

    @InjectMocks
    private PspTakePaymentAction pspTakePaymentAction;

    @Mock
    private PspMaintenanceService pspMaintenanceService;

    @Mock
    private PspPaymentProcessService pspPaymentProcessService;

    @Mock
    private ModelService modelService;

    private OrderProcessModel orderProcessModel;

    private OrderModel orderModel;

    @Before
    public void setup() {
        orderProcessModel = new OrderProcessModel();
        orderModel = new OrderModel();
        orderModel.setTotalPrice(Double.valueOf("10.00"));
        orderProcessModel.setOrder(orderModel);
    }

    @Test
    public void paymentIsTaken() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(), any())).thenReturn(AUTHORIZATION);

        // When
        Transition result = pspTakePaymentAction.executeAction(orderProcessModel);

        // Then
        assertEquals(OK, result);
        assertEquals(PAYMENT_CAPTURED, orderModel.getStatus());
        verify(modelService).save(orderModel);
        verify(pspMaintenanceService).captureFunds(orderModel, BigDecimal.valueOf(orderModel.getTotalPrice()));
    }

    @Test
    public void paymentIsNotTaken() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(), any())).thenReturn(AUTHORIZATION);
        when(pspMaintenanceService.captureFunds(eq(orderModel), any(BigDecimal.class))).thenThrow(IllegalMaintenanceOperation.class);

        // When
        Transition result = pspTakePaymentAction.executeAction(orderProcessModel);

        // Then
        assertEquals(NOK, result);
        assertEquals(PAYMENT_NOT_CAPTURED, orderModel.getStatus());
        verify(modelService).save(orderModel);
    }

    @Test
    public void successfulButPaymentAlreadyTakenForCapture() throws Exception {
        // Given
        when(pspPaymentProcessService.findPaymentTransactionTypeForPaymentInfo(any(), any())).thenReturn(CAPTURE);

        // When
        Transition result = pspTakePaymentAction.executeAction(orderProcessModel);

        // Then
        assertEquals(OK, result);
        assertEquals(PAYMENT_CAPTURED, orderModel.getStatus());
        verify(modelService).save(orderModel);
        verify(pspMaintenanceService, never()).captureFunds(orderModel, BigDecimal.valueOf(orderModel.getTotalPrice()));
    }
}
