package com.psp.payment.integration.clients;

import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.http.HttpEntity;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.TreeMap;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspRestClientTest {

    @InjectMocks
    private PspRestClient<String> pspRestClient;

    @Mock
    private RestTemplate restTemplate;

    @Test
    public void postRequestTriggersRestCall() throws URISyntaxException {
        // Given
        URI uri = new URI("http://www.google.com");
        Map<String, String> parameters = getRequestParameterMap();

        // When
        pspRestClient.postRequest(uri, parameters, String.class);

        // Then
        verify(restTemplate).postForObject(eq(uri), any(HttpEntity.class), eq(String.class));
    }

    @Test(expected = RestClientException.class)
    public void postRequestThrowsException() throws URISyntaxException {
        // Given
        URI uri = new URI("http://www.google.com");
        Map<String, String> parameters = getRequestParameterMap();

        when(restTemplate.postForObject(eq(uri), any(HttpEntity.class), eq(String.class))).thenThrow(RestClientException.class);

        // When
        pspRestClient.postRequest(uri, parameters, String.class);

        // Then a runtime exception can be thrown for connection timeouts etc.
    }

    @Test(expected = IllegalArgumentException.class)
    public void postRequestCalledWithoutUrl() {
        // Given
        Map<String, String> parameters = getRequestParameterMap();

        // When
        pspRestClient.postRequest(null, parameters, String.class);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void postRequestCalledWithoutParameters() throws URISyntaxException {
        // Given
        URI uri = new URI("http://www.google.com");

        // When
        pspRestClient.postRequest(uri, null, String.class);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void postRequestCalledWithoutReturnType() throws URISyntaxException {
        // Given
        URI uri = new URI("http://www.google.com");
        Map<String, String> parameters = getRequestParameterMap();

        // When
        pspRestClient.postRequest(uri, parameters, null);

        // Then a runtime exception can be thrown for connection timeouts etc.
    }

    private Map<String, String> getRequestParameterMap() {
        Map<String, String> parameters = new TreeMap<>();
        parameters.put("key1", "value1");
        parameters.put("key2", "value2");
        parameters.put("key3", "value3");

        return parameters;
    }
}
