package com.psp.payment.integration.utils;

import com.psp.payment.directlink.exception.XmlParseException;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;

import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspIntegrationUtilsTest {

    @InjectMocks
    private PspIntegrationUtils pspIntegrationUtils;

    private String response;

    @Before
    public void setup() {
        response = "<?xml version=\"1.0\"?><ncresponse currency=\"GBP\" amount=\"18.85\" STATUS=\"91\" ACCEPTANCE=\"\" NCERRORPLUS=\"!\" NCERROR=\"0\" NCSTATUS=\"0\" PAYIDSUB=\"1\" PAYID=\"3011150941\" orderID=\"TestOrder21081\"><HTML_ANSWER>DFGFDGDFGFDGFDG</HTML_ANSWER></ncresponse>";
    }

    @Test
    public void eachAttributeIsPopulatedInUpperCaseKeys() {
        // When
        Map<String, String> result = pspIntegrationUtils.convertPlainStringToParameters(response);

        // Then
        assertEquals("GBP", result.get("CURRENCY"));
        assertEquals("18.85", result.get("AMOUNT"));
        assertEquals("91", result.get("STATUS"));
        assertEquals("", result.get("ACCEPTANCE"));
        assertEquals("!", result.get("NCERRORPLUS"));
        assertEquals("0", result.get("NCERROR"));
        assertEquals("0", result.get("NCSTATUS"));
        assertEquals("1", result.get("PAYIDSUB"));
        assertEquals("3011150941", result.get("PAYID"));
        assertEquals("TestOrder21081", result.get("ORDERID"));
    }

    @Test
    public void noMoreThanGivenAttributesArePopulated() {
        // When
        Map<String, String> result = pspIntegrationUtils.convertPlainStringToParameters(response);

        // Then
        assertEquals(11, result.size());
    }

    @Test(expected = XmlParseException.class)
    public void responseNotPopulatedOnInvalidInputs() {
        // When
        pspIntegrationUtils.convertPlainStringToParameters("fake");

        // Then an exception is thrown
    }
}
