package com.psp.payment.maintenance.converters.populators;

import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import com.psp.payment.maintenance.dto.PspMaintenanceRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.HashMap;
import java.util.Map;

import static com.psp.constants.PspConstants.*;
import static com.psp.payment.common.enums.PspOperation.SAS;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenanceRequestPopulatorTest {

    @InjectMocks
    PspMaintenanceRequestPopulator pspMaintenanceRequestPopulator;

    @Mock
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    private PspMaintenanceRequest source;

    private Map<String, String> target;

    @Before
    public void setup() {
        createSource();
        target = new HashMap<>();
    }

    @Test
    public void pspIdIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getPspId(), target.get(KEY_PSPID));
    }

    @Test
    public void pspIdIsNotPopulated() {
        // Given
        source.setPspId(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_PSPID));
    }

    @Test
    public void userIdIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getUserId(), target.get(KEY_API_USERID));
    }

    @Test
    public void userIdIsNotPopulated() {
        // Given
        source.setUserId(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_API_USERID));
    }

    @Test
    public void passwordIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getPassword(), target.get(KEY_API_PASSWORD));
    }

    @Test
    public void passwordIsNotPopulated() {
        // Given
        source.setPassword(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_API_PASSWORD));
    }

    @Test
    public void payIdIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getPayId(), target.get(KEY_PAYID));
    }

    @Test
    public void payIdIsNotPopulated() {
        // Given
        source.setPayId(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_PAYID));
    }

    @Test
    public void amountIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getAmount(), target.get(KEY_AMOUNT));
    }

    @Test
    public void amountIsNotPopulated() {
        // Given
        source.setAmount(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_AMOUNT));
    }

    @Test
    public void operationIsPopulated() {
        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals(source.getOperation().name(), target.get(KEY_OPERATION));
    }

    @Test
    public void operationIsNotPopulated() {
        // Given
        source.setOperation(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_OPERATION));
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateFailsIfSourceNotProvided() {
        // When
        pspMaintenanceRequestPopulator.populate(null, target);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void populateFailsIfTargetNotProvided() {
        // When
        pspMaintenanceRequestPopulator.populate(source, null);

        // Then an exception is thrown
    }

    @Test
    public void trackingKeyIsPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn("HYB010317");

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertEquals("HYB010317", target.get(KEY_TRACKING));
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn(null);

        // When
        pspMaintenanceRequestPopulator.populate(source, target);

        // Then
        assertNull(target.get(KEY_TRACKING));
    }

    private void createSource() {
        source = new PspMaintenanceRequest();
        source.setPspId("pspid");
        source.setUserId("userid");
        source.setPassword("password");
        source.setPayId("1");
        source.setAmount("1000");
        source.setOperation(SAS);
    }
}
