package com.psp.payment.maintenance.converters.populators;

import com.psp.constants.PspConstants;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenanceResponsePopulatorTest {

    @InjectMocks
    private PspMaintenanceResponsePopulator pspMaintenanceResponsePopulator;

    private Map<String, String> source;

    private PspMaintenanceResponse target;

    @Before
    public void setup() {
        source = new TreeMap<>();
        target = new PspMaintenanceResponse();
    }

    @Test(expected = IllegalArgumentException.class)
    public void responseNotPopulatedWhenTargetNotProvided() {
        // When
        pspMaintenanceResponsePopulator.populate(source, null);

        // Then an exception will be thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void responseNotPopulatedWhenSourceNotProvided() {
        // When
        pspMaintenanceResponsePopulator.populate(null, target);

        // Then an exception will be thrown
    }

    @Test
    public void amountIsPopulated() {
        // Given
        String amount = "10.0";
        source.put(PspConstants.KEY_AMOUNT, amount);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(amount, target.getAmount());
    }

    @Test
    public void payIdIsPopulated() {
        // Given
        String payId = "21673688";
        source.put(PspConstants.KEY_PAYID, payId);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(payId, target.getPayId());
    }

    @Test
    public void acceptanceIsPopulated() {
        // Given
        String acceptance = "!";
        source.put(PspConstants.KEY_RESULT_ACCEPTANCE, acceptance);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(acceptance, target.getAcceptance());
    }

    @Test
    public void currencyIsPopulated() {
        // Given
        String currency = "GBP";
        source.put(PspConstants.KEY_CURRENCY, currency);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(currency, target.getCurrency());
    }

    @Test
    public void ncStatusIsPopulated() {
        // Given
        String ncStatus = "0";
        source.put(PspConstants.KEY_RESULT_NCSTATUS, ncStatus);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(ncStatus, target.getNcStatus());
    }

    @Test
    public void orderIdIsPopulated() {
        // Given
        String orderId = "00174900";
        source.put(PspConstants.KEY_ORDERID, orderId);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(orderId, target.getOrderId());
    }

    @Test
    public void payIdSubIsPopulated() {
        // Given
        String payIdSub = "0";
        source.put(PspConstants.KEY_PAYID_SUB, payIdSub);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(payIdSub, target.getPayIdSub());
    }

    @Test
    public void statusIsPopulated() {
        // Given
        String status = "91";
        source.put(PspConstants.KEY_STATUS, status);

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals(status, target.getStatus());
    }

    @Test
    public void ncErrorDataIsPopulated() {
        // Given
        source.put(PspConstants.KEY_NCERROR, "0");
        source.put(PspConstants.KEY_NCERRORPLUS, "0");

        // When
        pspMaintenanceResponsePopulator.populate(source, target);

        // Then
        assertEquals("0", target.getNcErrorData().getNcError());
        assertEquals("0", target.getNcErrorData().getNcErrorPlus());
    }
}
