package com.psp.payment.maintenance.strategies.impl;

import com.psp.payment.common.strategies.PspGenerateMerchantTransactionCodeStrategy;
import com.psp.payment.maintenance.dto.PspMaintenanceResponse;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commerceservices.order.CommerceCheckoutService;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.PaymentService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.dto.TransactionStatusDetails;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.i18n.CommonI18NService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.store.BaseStoreModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspMaintenancePaymentTransactionCreationStrategyImplTest {

    @InjectMocks
    private PspMaintenancePaymentTransactionCreationStrategyImpl pspMaintenancePaymentTransactionCreationStrategy;

    @Mock
    private ModelService modelService;

    @Mock
    private CommonI18NService commonI18NService;

    @Mock
    private CommerceCheckoutService commerceCheckoutService;

    @Mock
    private PspGenerateMerchantTransactionCodeStrategy pspGenerateMerchantTransactionCodeStrategy;

    @Mock
    private PaymentService paymentService;

    @Mock
    private CurrencyModel currencyModel;

    private PaymentTransactionModel paymentTransactionModel;

    private PaymentTransactionEntryModel paymentTransactionEntryModel;

    private OrderModel orderModel;

    private PspMaintenanceResponse pspMaintenanceResponse;

    @Before
    public void setup() {
        orderModel = new OrderModel();
        BaseStoreModel baseStoreModel = new BaseStoreModel();
        baseStoreModel.setPaymentProvider("ingenico");
        orderModel.setStore(baseStoreModel);
        paymentTransactionModel = new PaymentTransactionModel();
        List<PaymentTransactionModel> paymentTransactionModelList = new ArrayList<>();
        paymentTransactionModelList.add(paymentTransactionModel);
        orderModel.setPaymentTransactions(paymentTransactionModelList);
        paymentTransactionEntryModel = new PaymentTransactionEntryModel();
        createMaintenanceResponse();
        setupMockingBehaviour();
    }

    @Test(expected = IllegalArgumentException.class)
    public void failsWhenTypeIsNotProvided() {
        // When
        pspMaintenancePaymentTransactionCreationStrategy.createPaymentTransaction(null, orderModel, pspMaintenanceResponse);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void failsWhenOrderIsNotProvided() {
        // When
        pspMaintenancePaymentTransactionCreationStrategy.createPaymentTransaction(PaymentTransactionType.CAPTURE, null, pspMaintenanceResponse);

        // Then an exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void failsWhenResponseObjectIsNotProvided() {
        // When
        pspMaintenancePaymentTransactionCreationStrategy.createPaymentTransaction(PaymentTransactionType.CAPTURE, orderModel, null);

        // Then an exception is thrown
    }

    @Test
    public void paymentTransactionIsSavedToOrder() {
        // When
        PaymentTransactionEntryModel result = pspMaintenancePaymentTransactionCreationStrategy.createPaymentTransaction(PaymentTransactionType.CAPTURE, orderModel, pspMaintenanceResponse);

        // Then
        assertEquals(pspMaintenanceResponse.getPayId(), result.getRequestToken());
        assertEquals(pspMaintenanceResponse.getAmount(), result.getAmount().toString());
        assertEquals(TransactionStatus.ACCEPTED.name(), result.getTransactionStatus());
        assertEquals(TransactionStatusDetails.SUCCESFULL.name(), result.getTransactionStatusDetails());
        verify(modelService).saveAll(paymentTransactionModel, paymentTransactionEntryModel);
    }

    private void createMaintenanceResponse() {
        pspMaintenanceResponse = new PspMaintenanceResponse();
        pspMaintenanceResponse.setPayId("payid");
        pspMaintenanceResponse.setAmount("20.00");
    }

    private void setupMockingBehaviour() {
        when(modelService.create(PaymentTransactionModel.class)).thenReturn(paymentTransactionModel);
        when(modelService.create(PaymentTransactionEntryModel.class)).thenReturn(paymentTransactionEntryModel);
        when(commonI18NService.getCurrency(anyString())).thenReturn(currencyModel);
    }
}
