package com.psp.payment.setup;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commerceservices.setup.SetupImpexService;
import de.hybris.platform.core.initialization.SystemSetupContext;
import de.hybris.platform.core.initialization.SystemSetupParameter;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.configuration.Configuration;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyBoolean;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspSystemSetupTest {

    @Spy
    @InjectMocks
    private PspSystemSetup pspSystemSetup;

    @Mock
    private ConfigurationService configurationService;

    @Mock
    private Configuration configuration;

    @Mock
    private SetupImpexService setupImpexService;

    @Mock
    private SystemSetupContext context;

    @Before
    public void setUp() {
        pspSystemSetup.setSetupImpexService(setupImpexService);
        when(configurationService.getConfiguration()).thenReturn(configuration);
    }

    @Test
    public void createProjectDataWithConfigTrue() {
        // Given
        when(configuration.getBoolean(anyString())).thenReturn(Boolean.TRUE);

        // When
        pspSystemSetup.createProjectData(context);

        //Then
        verify(pspSystemSetup, times(7)).importImpexFile(any(SystemSetupContext.class), anyString(), anyBoolean());
    }

    @Test
    public void createProjectDataWithConfigFalse() {
        // Given
        when(configuration.getBoolean(anyString())).thenReturn(Boolean.FALSE);

        // When
        pspSystemSetup.createProjectData(context);

        // Then
        verify(pspSystemSetup, never()).importImpexFile(any(SystemSetupContext.class), anyString(), anyBoolean());
    }

    @Test
    public void getInitializationOptions() {
        // When
        List<SystemSetupParameter> params = pspSystemSetup.getInitializationOptions();

        // Then
        assertTrue(CollectionUtils.isEmpty(params));
    }
}