package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.junit.Before;
import org.junit.Test;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;

@UnitTest
public class PspCallbackUrlPopulatorTest {

    private PspCallbackUrlPopulator pspCallbackUrlPopulator;

    private PspCreateSubscriptionRequest pspCreateSubscriptionRequest;

    private PaymentData paymentData;

    @Before
    public void setUp() {
        pspCallbackUrlPopulator = new PspCallbackUrlPopulator();
        pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void testPopulateWithValidData() {
        // Given
        pspCreateSubscriptionRequest.setAcceptUrl("https://apparel-uk.local:9002/yacceleratorstorefront/checkout/psp/response");
        pspCreateSubscriptionRequest.setExceptionUrl("https://apparel-uk.local:9002/yacceleratorstorefront/checkout/psp/response");

        // When
        pspCallbackUrlPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_ACCEPT_URL), is("https://apparel-uk.local:9002/yacceleratorstorefront/checkout/psp/response"));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_EXCEPTION_URL), is("https://apparel-uk.local:9002/yacceleratorstorefront/checkout/psp/response"));
    }

    @Test
    public void testPopulateWithEmptyData() {
        // Given
        pspCreateSubscriptionRequest.setAcceptUrl("");
        pspCreateSubscriptionRequest.setExceptionUrl("");

        // When
        pspCallbackUrlPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_ACCEPT_URL), is(""));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_EXCEPTION_URL), is(""));
    }

    @Test
    public void testPopulateWithNoData() {
        // Given nothing

        // When
        pspCallbackUrlPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_ACCEPT_URL));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_EXCEPTION_URL));
    }
}
