package com.psp.payment.tokenisation.converters.populators.request;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.core.model.user.CustomerModel;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

@UnitTest
public class PspCustomerNameFromAddressPopulatorTest {

    private Populator<AddressData, CustomerModel> pspCustomerNameFromAddressPopulator = new PspCustomerNameFromAddressPopulator();

    private AddressData addressData;

    private CustomerModel customerModel;

    @Before
    public void setup() {
        addressData = new AddressData();
        customerModel = new CustomerModel();
    }

    @Test
    public void testCustomerNameIsSavedWhenBothNamesProvided() {
        // Given
        addressData.setFirstName("Test");
        addressData.setLastName("User");

        // When
        pspCustomerNameFromAddressPopulator.populate(addressData, customerModel);

        // Then
        String expectedResult = "Test User";
        assertEquals(expectedResult, customerModel.getName());
    }

    @Test
    public void testCustomerNameIsSavedWhenFirstNameProvided() {
        // Given
        addressData.setFirstName("Test");

        // When
        pspCustomerNameFromAddressPopulator.populate(addressData, customerModel);

        // Then
        String expectedResult = "Test";
        assertEquals(expectedResult, customerModel.getName());
    }

    @Test
    public void testCustomerNameIsSavedWhenLastNameProvided() {
        // Given
        addressData.setLastName("Test");

        // When
        pspCustomerNameFromAddressPopulator.populate(addressData, customerModel);

        // Then
        String expectedResult = "Test";
        assertEquals(expectedResult, customerModel.getName());
    }

    @Test
    public void testCustomerNameIsNotSavedOnEmptyInput() {
        // Given blank address data

        // When
        pspCustomerNameFromAddressPopulator.populate(addressData, customerModel);

        // Then
        assertNull(customerModel.getName());
    }

    @Test
    public void testCustomerNameIsNotSavedOnNoInput() {
        // Given no data

        // When
        pspCustomerNameFromAddressPopulator.populate(addressData, customerModel);

        // Then
        assertNull(customerModel.getName());
    }
}
