package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.junit.Before;
import org.junit.Test;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;

@UnitTest
public class PspOtherParametersPopulatorTest {

    private PspOtherParametersPopulator pspOtherParametersPopulator;

    private PspCreateSubscriptionRequest pspCreateSubscriptionRequest;

    private PaymentData paymentData;

    @Before
    public void setUp() {
        pspOtherParametersPopulator = new PspOtherParametersPopulator();
        pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void testPopulateWithValidData() {
        // Given
        pspCreateSubscriptionRequest.setPspId("test");
        pspCreateSubscriptionRequest.setParamPlus("ABC=DEF");

        // When
        pspOtherParametersPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_TOKENISATION_PSPID), is("test"));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_PARAM_PLUS), is("ABC=DEF"));
    }

    @Test
    public void testPopulateWithEmptyData() {
        // Given
        pspCreateSubscriptionRequest.setPspId("");
        pspCreateSubscriptionRequest.setParamPlus("");

        // When
        pspOtherParametersPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_TOKENISATION_PSPID), is(""));
        assertThat(paymentData.getParameters().get(PspConstants.KEY_PARAM_PLUS), is(""));
    }

    @Test
    public void testPopulateWithNoData() {
        // Given nothing

        // When
        pspOtherParametersPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertThat(paymentData.getParameters().size(), is(2));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_TOKENISATION_PSPID));
        assertNull(paymentData.getParameters().get(PspConstants.KEY_PARAM_PLUS));
    }
}
