package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentDataPopulatorTest {

    @InjectMocks
    private PspPaymentDataPopulator pspPaymentDataPopulator;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    private PspCreateSubscriptionRequest pspCreateSubscriptionRequest;

    private PaymentData paymentData;

    @Before
    public void setUp() {
        pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void testPopulateWithValidData() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_TOKENISATION_URL)).thenReturn("http://www.test.com");

        // When
        pspPaymentDataPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getPostUrl());
        assertEquals(paymentData.getPostUrl(), "http://www.test.com");
    }

    @Test
    public void testPopulateWithEmptyData() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_TOKENISATION_URL)).thenReturn(StringUtils.EMPTY);

        // When
        pspPaymentDataPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertEquals(paymentData.getPostUrl(), StringUtils.EMPTY);
    }

    @Test
    public void testPopulateWithNoData() {
        // Given
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_TOKENISATION_URL)).thenReturn(null);

        // When
        pspPaymentDataPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNull(paymentData.getPostUrl());
    }
}
