package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

@UnitTest
public class PspPaymentInfoPopulatorTest {

    private PspPaymentInfoPopulator pspPaymentInfoPopulator;

    private PspCreateSubscriptionRequest pspCreateSubscriptionRequest;

    private PaymentData paymentData;

    @Before
    public void setUp() {
        pspPaymentInfoPopulator = new PspPaymentInfoPopulator();
        pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        pspCreateSubscriptionRequest.setPaymentMethod(PspConstants.CREDIT_CARD);
        paymentData = new PaymentData();
    }

    @Test
    public void testPopulateWithValidData() {
        // Given nothing

        // When
        pspPaymentInfoPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertEquals(1, paymentData.getParameters().size());
        assertEquals(PspConstants.CREDIT_CARD, paymentData.getParameters().get(PspConstants.KEY_PAYMENT_METHOD));
    }

    @Test
    public void testPopulateWithEmptyData() {
        // Given
        pspCreateSubscriptionRequest.setPaymentMethod(StringUtils.EMPTY);

        // When
        pspPaymentInfoPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertEquals(2, paymentData.getParameters().size());
        assertEquals(StringUtils.EMPTY, paymentData.getParameters().get(PspConstants.KEY_PAYMENT_METHOD));
    }

    @Test
    public void testPopulateWithNonCreditCard() {
        // Given nothing
        pspCreateSubscriptionRequest.setPaymentMethod(PspConstants.PAYPAL);

        // When
        pspPaymentInfoPopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertEquals(2, paymentData.getParameters().size());
        assertNotNull(paymentData.getParameters().get(PspConstants.KEY_PAYMENT_METHOD));
    }
}
