package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

@UnitTest
public class PspSignaturePopulatorTest {

    private PspSignaturePopulator pspSignaturePopulator;

    private PspCreateSubscriptionRequest pspCreateSubscriptionRequest;

    private PaymentData paymentData;

    @Before
    public void setUp() {
        pspSignaturePopulator = new PspSignaturePopulator();
        pspCreateSubscriptionRequest = new PspCreateSubscriptionRequest();
        paymentData = new PaymentData();
    }

    @Test
    public void testPopulateWithValidData() {
        // Given
        pspCreateSubscriptionRequest.setShaSign("shasign");

        // When
        pspSignaturePopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        String expectedResult = "shasign";
        assertNotNull(paymentData.getParameters());
        assertEquals(1, paymentData.getParameters().size());
        assertEquals(expectedResult, paymentData.getParameters().get(PspConstants.KEY_SHA_SIGNATURE));
    }

    @Test
    public void testPopulateWithEmptyData() {
        // Given
        pspCreateSubscriptionRequest.setShaSign(StringUtils.EMPTY);

        // When
        pspSignaturePopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertEquals(1, paymentData.getParameters().size());
        assertEquals(StringUtils.EMPTY, paymentData.getParameters().get(PspConstants.KEY_SHA_SIGNATURE));
    }

    @Test
    public void testPopulateWithNoData() {
        // Given nothing

        // When
        pspSignaturePopulator.populate(pspCreateSubscriptionRequest, paymentData);

        // Then
        assertNotNull(paymentData.getParameters());
        assertEquals(1, paymentData.getParameters().size());
        assertNull(paymentData.getParameters().get(PspConstants.KEY_SHA_SIGNATURE));
    }
}
