package com.psp.payment.tokenisation.converters.populators.request;

import com.psp.constants.PspConstants;
import com.psp.payment.common.dto.PspPaymentRequest;
import com.psp.payment.common.utils.PspExtensionTrackingUtils;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspTrackingKeyPopulatorTest {

    @InjectMocks
    private PspTrackingKeyPopulator pspTrackingKeyPopulator;

    @Mock
    private PspExtensionTrackingUtils pspExtensionTrackingUtils;

    private PspPaymentRequest pspPaymentRequest;

    private PaymentData paymentData;

    @Before
    public void setup() {
        pspPaymentRequest = new PspPaymentRequest();
        paymentData = new PaymentData();
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn("HYB010317");
    }

    @Test
    public void trackingKeyIsPopulated() {
        // When
        pspTrackingKeyPopulator.populate(pspPaymentRequest, paymentData);

        // Then
        assertEquals(1, paymentData.getParameters().size());
        assertNotNull(paymentData.getParameters().get(PspConstants.KEY_TRACKING));
        assertEquals("HYB010317", paymentData.getParameters().get(PspConstants.KEY_TRACKING));
    }

    @Test
    public void trackingKeyIsNotPopulated() {
        // Given
        when(pspExtensionTrackingUtils.fetchTrackingDetails()).thenReturn(null);

        // When
        pspTrackingKeyPopulator.populate(pspPaymentRequest, paymentData);

        // Then
        assertEquals(0, paymentData.getParameters().size());
    }
}
