package com.psp.payment.tokenisation.converters.populators.response;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.enums.PspValidity;
import de.hybris.bootstrap.annotations.UnitTest;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCustomerInfoResultPopulatorTest {

    @InjectMocks
    private PspCustomerInfoResultPopulator pspCustomerInfoResultPopulator;

    private Map<String, String> source;

    private PspCreateSubscriptionResult target;

    @Before
    public void setup() {
        source = new TreeMap<>();
        target = new PspCreateSubscriptionResult();
    }

    @Test
    public void customerInfoIsPopulated() {
        // Given
        source.put(PspConstants.KEY_RESULT_ALIAS_ID, "12345");
        source.put(PspConstants.KEY_RESULT_STORE_PERMANENTLY, "Y");

        // When
        pspCustomerInfoResultPopulator.populate(source, target);

        // Then
        assertThat(target.getPaymentToken(), is("12345"));
        assertThat(target.getValidity(), is(PspValidity.INDEFINITE));
    }

    @Test
    public void customerInfoIsPopulatedForTemporaryValidity() {
        // Given
        source.put(PspConstants.KEY_RESULT_STORE_PERMANENTLY, "N");

        // When
        pspCustomerInfoResultPopulator.populate(source, target);

        // Then
        assertThat(target.getValidity(), is(PspValidity.TEMPORARY));
    }

    @Test
    public void customerInfoIsNotPopulatedWhenInputsAreNull() {
        // Given
        source.put(PspConstants.KEY_RESULT_ALIAS_ID, null);
        source.put(PspConstants.KEY_RESULT_STORE_PERMANENTLY, null);

        // When
        pspCustomerInfoResultPopulator.populate(source, target);

        // Then
        assertEquals(target.getPaymentToken(), null);
        assertEquals(target.getValidity(), null);
    }

    @Test
    public void customerInfoIsNotPopulatedWhenInputsAreEmpty() {
        // Given
        source.put(PspConstants.KEY_RESULT_ALIAS_ID, StringUtils.EMPTY);
        source.put(PspConstants.KEY_RESULT_STORE_PERMANENTLY, StringUtils.EMPTY);

        // When
        pspCustomerInfoResultPopulator.populate(source, target);

        // Then
        assertEquals(target.getPaymentToken(), null);
        assertEquals(target.getValidity(), null);
    }
}
