package com.psp.payment.tokenisation.converters.populators.response;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.dto.PspErrorData;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspErrorResultPopulatorTest {

    @InjectMocks
    private PspErrorResultPopulator pspErrorResultPopulator;

    private Map<String, String> source;

    private PspCreateSubscriptionResult target;

    @Before
    public void setup() {
        source = new TreeMap<>();
        target = new PspCreateSubscriptionResult();
    }

    @Test
    public void testPopulateSuccess() {
        source.put(PspConstants.KEY_RESULT_NCERROR, "50001111");
        source.put(PspConstants.KEY_RESULT_NCERRORCARDNO, "50001113");
        source.put(PspConstants.KEY_RESULT_NCERRORCN, "500011112");
        source.put(PspConstants.KEY_RESULT_NCERRORCVC, "50001114");
        source.put(PspConstants.KEY_RESULT_NCERRORED, "50001115");

        pspErrorResultPopulator.populate(source, target);

        PspErrorData data = target.getErrorData();

        assertThat(data.getNcError(), is("50001111"));
        assertThat(data.getNcErrorCardNo(), is("50001113"));
        assertThat(data.getNcErrorCN(), is("500011112"));
        assertThat(data.getNcErrorCVC(), is("50001114"));
        assertThat(data.getNcErrorED(), is("50001115"));
    }

    @Test
    public void testPopulateNullValue() {
        source.put(PspConstants.KEY_RESULT_NCERROR, null);
        source.put(PspConstants.KEY_RESULT_NCERRORCARDNO, null);
        source.put(PspConstants.KEY_RESULT_NCERRORCN, null);
        source.put(PspConstants.KEY_RESULT_NCERRORCVC, null);
        source.put(PspConstants.KEY_RESULT_NCERRORED, null);

        pspErrorResultPopulator.populate(source, target);

        PspErrorData data = target.getErrorData();

        assertEquals(data.getNcErrorCardNo(), null);
        assertEquals(data.getNcError(), null);
        assertEquals(data.getNcErrorCN(), null);
        assertEquals(data.getNcErrorCVC(), null);
        assertEquals(data.getNcErrorED(), null);
    }

    @Test
    public void testPopulateEmptyValue() {
        source.put(PspConstants.KEY_RESULT_NCERROR, "");
        source.put(PspConstants.KEY_RESULT_NCERRORCARDNO, "");
        source.put(PspConstants.KEY_RESULT_NCERRORCN, "");
        source.put(PspConstants.KEY_RESULT_NCERRORCVC, "");
        source.put(PspConstants.KEY_RESULT_NCERRORED, "");

        pspErrorResultPopulator.populate(source, target);

        PspErrorData data = target.getErrorData();

        assertEquals(data.getNcErrorCardNo(), null);
        assertEquals(data.getNcError(), null);
        assertEquals(data.getNcErrorCN(), null);
        assertEquals(data.getNcErrorCVC(), null);
        assertEquals(data.getNcErrorED(), null);
    }
}
