package com.psp.payment.tokenisation.converters.populators.response;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentInfoResultPopulatorTest {

    @InjectMocks
    private PspPaymentInfoResultPopulator populator;

    private Map<String, String> source;

    private PspCreateSubscriptionResult target;

    @Before
    public void setup() {
        source = new TreeMap<>();
        target = new PspCreateSubscriptionResult();
    }

    @Test
    public void testPopulateSuccess() {
        source.put(PspConstants.KEY_RESULT_CARD_NUMBER, "1234567890");
        source.put(PspConstants.KEY_RESULT_CARD_HOLDER, "Card Holder");
        source.put(PspConstants.KEY_RESULT_CARD_BRAND, "Visa");
        source.put(PspConstants.KEY_RESULT_CARD_CVC, "123");
        source.put(PspConstants.KEY_RESULT_CARD_EXPIRYDATE, "0220");

        populator.populate(source, target);

        final PaymentInfoData data = target.getPaymentInfoData();
        assertThat(data.getCardAccountNumber(), is("1234567890"));
        assertThat(data.getCardAccountHolderName(), is("Card Holder"));
        assertThat(data.getCardCardType(), is("Visa"));
        assertThat(data.getCardCvNumber(), is("123"));
        assertThat(data.getCardExpirationMonth(), is(2));
        assertThat(data.getCardExpirationYear(), is(20));
    }

    @Test
    public void testPopulateNullValue() {
        source.put(PspConstants.KEY_RESULT_CARD_NUMBER, null);
        source.put(PspConstants.KEY_RESULT_CARD_HOLDER, null);
        source.put(PspConstants.KEY_RESULT_CARD_BRAND, null);
        source.put(PspConstants.KEY_RESULT_CARD_CVC, null);
        source.put(PspConstants.KEY_RESULT_CARD_EXPIRYDATE, null);

        populator.populate(source, target);

        final PaymentInfoData data = target.getPaymentInfoData();
        assertEquals(data.getCardAccountNumber(), null);
        assertEquals(data.getCardAccountHolderName(), null);
        assertEquals(data.getCardCardType(), null);
        assertEquals(data.getCardCvNumber(), null);
        assertEquals(data.getCardExpirationMonth(), null);
        assertEquals(data.getCardExpirationYear(), null);
    }

    @Test
    public void testPopulateEmptyValue() {
        source.put(PspConstants.KEY_RESULT_CARD_NUMBER, "");
        source.put(PspConstants.KEY_RESULT_CARD_HOLDER, "");
        source.put(PspConstants.KEY_RESULT_CARD_BRAND, "");
        source.put(PspConstants.KEY_RESULT_CARD_CVC, "");
        source.put(PspConstants.KEY_RESULT_CARD_EXPIRYDATE, "");

        populator.populate(source, target);

        final PaymentInfoData data = target.getPaymentInfoData();
        assertEquals(data.getCardAccountNumber(), null);
        assertEquals(data.getCardAccountHolderName(), null);
        assertEquals(data.getCardCardType(), null);
        assertEquals(data.getCardCvNumber(), null);
        assertEquals(data.getCardExpirationMonth(), null);
        assertEquals(data.getCardExpirationYear(), null);
    }
}
