package com.psp.payment.tokenisation.converters.populators.response;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.enums.PspTokenisationStatus;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Map;
import java.util.TreeMap;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspStatusResultPopulatorTest {

    @InjectMocks
    private PspStatusResultPopulator pspStatusResultPopulator;

    private Map<String, String> source;

    private PspCreateSubscriptionResult target;

    @Before
    public void setup() {
        source = new TreeMap<>();
        target = new PspCreateSubscriptionResult();
    }

    @Test
    public void testPopulateStatusSuccess() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, "0");

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertThat(target.getTokenisationStatus(), is(PspTokenisationStatus.OK));
    }

    @Test
    public void testPopulateStatusErrorNOK() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, "1");

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertThat(target.getTokenisationStatus(), is(PspTokenisationStatus.NOK));
    }


    @Test
    public void testPopulateStatusErrorUpdated() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, "2");

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertThat(target.getTokenisationStatus(), is(PspTokenisationStatus.ALIAS_UPDATED));
    }

    @Test
    public void testPopulateStatusErrorCancelled() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, "3");

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertThat(target.getTokenisationStatus(), is(PspTokenisationStatus.CANCELLED_BY_USER));
    }

    @Test
    public void testPopulateStatusNullValue() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, null);

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertEquals(target.getTokenisationStatus(), null);
    }

    @Test
    public void testPopulateStatusEmptyValue() {
        // Given
        source.put(PspConstants.KEY_RESULT_STATUS, "");

        // When
        pspStatusResultPopulator.populate(source, target);

        // Then
        assertEquals(target.getTokenisationStatus(), null);
    }
}
