package com.psp.payment.tokenisation.factories;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.enums.PspTokenisationStatus;
import com.psp.payment.tokenisation.strategies.PspResponseStrategy;
import com.psp.payment.tokenisation.strategies.impl.PspErrorResponseStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspSucessResponseStrategyImpl;
import com.psp.payment.tokenisation.strategies.impl.PspUserCancelledResponseStrategyImpl;
import de.hybris.bootstrap.annotations.UnitTest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspResponseStrategyFactoryTest {

    @InjectMocks
    private PspResponseStrategyFactory pspResponseStrategyFactory;

    @Mock
    private PspCreateSubscriptionResult pspCreateSubscriptionResult;

    @Mock
    private PspSucessResponseStrategyImpl pspSucessResponseStrategy;

    @Mock
    private PspErrorResponseStrategyImpl pspErrorResponseStrategy;

    @Mock
    private PspUserCancelledResponseStrategyImpl pspUserCancelledResponseStrategy;

    @Before
    public void setup() {
        Map<PspTokenisationStatus, PspResponseStrategy> statusResponseMap = new HashMap<>();
        statusResponseMap.put(PspTokenisationStatus.OK, pspSucessResponseStrategy);
        statusResponseMap.put(PspTokenisationStatus.NOK, pspErrorResponseStrategy);
        statusResponseMap.put(PspTokenisationStatus.CANCELLED_BY_USER, pspUserCancelledResponseStrategy);

        pspResponseStrategyFactory.setPspStatusResponseMapping(statusResponseMap);
    }

    @Test
    public void successStrategyIsReturned() {
        // Given
        when(pspCreateSubscriptionResult.getTokenisationStatus()).thenReturn(PspTokenisationStatus.OK);

        // When
        PspResponseStrategy result = pspResponseStrategyFactory.getResponseStrategy(pspCreateSubscriptionResult);

        // Then
        assertEquals(pspSucessResponseStrategy, result);
    }

    @Test
    public void errorStrategyIsReturned() {
        // Given
        when(pspCreateSubscriptionResult.getTokenisationStatus()).thenReturn(PspTokenisationStatus.NOK);

        // When
        PspResponseStrategy result = pspResponseStrategyFactory.getResponseStrategy(pspCreateSubscriptionResult);

        // Then
        assertEquals(pspErrorResponseStrategy, result);
    }

    @Test
    public void userCancelledStrategyIsReturned() {
        // Given
        when(pspCreateSubscriptionResult.getTokenisationStatus()).thenReturn(PspTokenisationStatus.CANCELLED_BY_USER);

        // When
        PspResponseStrategy result = pspResponseStrategyFactory.getResponseStrategy(pspCreateSubscriptionResult);

        // Then
        assertEquals(pspUserCancelledResponseStrategy, result);
    }

    @Test(expected = IllegalArgumentException.class)
    public void noStrategyIsReturned() {
        // Given nothing

        // When
        pspResponseStrategyFactory.getResponseStrategy(null);

        // Then an exception is thrown
    }
}
