package com.psp.payment.tokenisation.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.payment.common.utils.PspUtils;
import com.psp.payment.tokenisation.dto.PspCreateSubscriptionRequest;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.site.BaseSiteService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Answers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static com.psp.constants.PspConstants.PARAM_ACCEPT_URL;
import static com.psp.constants.PspConstants.PARAM_EXCEPTION_URL;
import static java.lang.Boolean.TRUE;
import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.*;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCreateSubscriptionRequestStrategyImplTest {

    @InjectMocks
    private PspCreateSubscriptionRequestStrategyImpl pspCreateSubscriptionRequestStrategy;

    @Mock
    private PspUtils pspUtils;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private ConfigurationService configurationService;

    @Mock
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Mock
    private BaseSiteService baseSiteService;

    @Mock
    private CartModel cartModel;

    @Mock
    private CustomerModel customerModel;

    @Mock
    private PaymentModeModel paymentModeModel;

    @Mock
    private BaseSiteModel baseSiteModel;

    @Before
    public void setup() {
        when(paymentModeModel.getBrand()).thenReturn("CreditCard");
        when(paymentModeModel.getPaymentMethod()).thenReturn("CreditCard");
        when(cartModel.getPaymentMode()).thenReturn(paymentModeModel);
        when(cartModel.getCode()).thenReturn("100000");
        when(customerModel.getCustomerID()).thenReturn("1000");
        when(pspUtils.generateUniqueOrderId(cartModel)).thenReturn("100000-0");
        when(pspUtils.generateChecksum(anyMap(), anyString())).thenReturn("sha");
        when(baseSiteService.getCurrentBaseSite()).thenReturn(baseSiteModel);
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_ACCEPT_URL)).thenReturn("acceptUrl");
        when(siteBaseUrlResolutionService.getWebsiteUrlForSite(baseSiteModel, TRUE, PARAM_EXCEPTION_URL)).thenReturn("exceptionUrl");
        when(configurationService.getConfiguration().getString(PARAM_ACCEPT_URL)).thenReturn(PARAM_ACCEPT_URL);
        when(configurationService.getConfiguration().getString(PARAM_EXCEPTION_URL)).thenReturn(PARAM_EXCEPTION_URL);
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PSPID)).thenReturn("glhybris");
        when(configurationService.getConfiguration().getString(PspConstants.PARAM_PASSPHRASE)).thenReturn("passphrase");
        when(pspUtils.getWebsiteIdByCart(cartModel)).thenReturn("apparel-uk");
        when(checkoutCustomerStrategy.isAnonymousCheckout()).thenReturn(Boolean.FALSE);
    }

    @Test
    public void testCreateSubscriptionRequestIsValidForLoggedInUser() {
        // Given
        PspCreateSubscriptionRequest expectedResult = new PspCreateSubscriptionRequest();
        expectedResult.setOrderId("100000-0");
        expectedResult.setShaSign("sha");
        expectedResult.setExceptionUrl("exceptionUrl");
        expectedResult.setAcceptUrl("acceptUrl");
        expectedResult.setPaymentMethod("CreditCard");
        expectedResult.setStorePermanently("Y");
        expectedResult.setPspId("glhybris");

        // When
        PspCreateSubscriptionRequest result = pspCreateSubscriptionRequestStrategy.createSubscriptionRequest(cartModel, customerModel);

        // Then
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testCreateSubscriptionRequestIsValidForAnonymousCustomer() {
        // Given
        when(checkoutCustomerStrategy.isAnonymousCheckout()).thenReturn(TRUE);

        PspCreateSubscriptionRequest expectedResult = new PspCreateSubscriptionRequest();
        expectedResult.setOrderId("100000-0");
        expectedResult.setShaSign("sha");
        expectedResult.setExceptionUrl("exceptionUrl");
        expectedResult.setAcceptUrl("acceptUrl");
        expectedResult.setPaymentMethod("CreditCard");
        expectedResult.setStorePermanently("N");
        expectedResult.setPspId("glhybris");

        // When
        PspCreateSubscriptionRequest result = pspCreateSubscriptionRequestStrategy.createSubscriptionRequest(cartModel, customerModel);

        // Then
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testCreateSubscriptionRequestWithInvalidParameters() {
        // When
        PspCreateSubscriptionRequest result = pspCreateSubscriptionRequestStrategy.createSubscriptionRequest(null, null);

        // Then
        PspCreateSubscriptionRequest expectedResult = new PspCreateSubscriptionRequest();
        assertThat(expectedResult, is(result));
    }

    @Test
    public void testCreateSubscriptionRequestIsValidForLoggedInUserForDirectDebits() {
        // Given
        when(paymentModeModel.getBrand()).thenReturn("Direct debits");
        when(paymentModeModel.getPaymentMethod()).thenReturn("Direct debits");

        PspCreateSubscriptionRequest expectedResult = new PspCreateSubscriptionRequest();
        expectedResult.setOrderId("100000-0");
        expectedResult.setShaSign("sha");
        expectedResult.setExceptionUrl("exceptionUrl");
        expectedResult.setAcceptUrl("acceptUrl");
        expectedResult.setPaymentMethod("Direct debits");
        expectedResult.setBrand("Direct debits");
        expectedResult.setStorePermanently("Y");
        expectedResult.setPspId("glhybris");

        // When
        PspCreateSubscriptionRequest result = pspCreateSubscriptionRequestStrategy.createSubscriptionRequest(cartModel, customerModel);

        // Then
        assertThat(expectedResult, is(result));
    }
}