package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import com.psp.payment.common.enums.PspValidity;
import com.psp.payment.tokenisation.strategies.PspCreditCardTypeBrandMappingStrategy;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import de.hybris.platform.core.enums.CreditCardType;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCreditCardPaymentInfoCreateStrategyImplTest {

    @InjectMocks
    private PspCreditCardPaymentInfoCreateStrategyImpl pspCreditCardPaymentInfoCreateStrategy;

    @Mock
    PspCreditCardTypeBrandMappingStrategy pspCreditCardTypeBrandMappingStrategy;

    @Mock
    private ModelService modelService;

    @Mock
    private CartService cartService;

    @Mock
    private CartModel cartModel;

    private CustomerModel customerModel;

    private CreditCardPaymentInfoModel creditCardPaymentInfoModel;

    private PspCreateSubscriptionResult pspCreateSubscriptionResult;

    private AddressModel addressModel;

    @Before
    public void setUp() {
        pspCreateSubscriptionResult = new PspCreateSubscriptionResult();
        creditCardPaymentInfoModel = new CreditCardPaymentInfoModel();
        customerModel = new CustomerModel();
        customerModel.setUid("uid");
        addressModel = new AddressModel();

        setupMockingBehaviour();
    }

    private void setupMockingBehaviour() {
        when(modelService.create(CreditCardPaymentInfoModel.class)).thenReturn(creditCardPaymentInfoModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(cartModel.getPaymentAddress()).thenReturn(addressModel);
        when(pspCreditCardTypeBrandMappingStrategy.getCreditCardType("VISA")).thenReturn(CreditCardType.VISA);
    }

    @Test
    public void testSaveSubscriptionOnStorePaymentDetails() {
        // Given
        setupPspCreateSubscriptionResult(pspCreateSubscriptionResult);

        // When
        CreditCardPaymentInfoModel result = pspCreditCardPaymentInfoCreateStrategy.saveSubscription(customerModel, pspCreateSubscriptionResult);

        // Then
        CreditCardPaymentInfoModel expectedResult = new CreditCardPaymentInfoModel();
        expectedResult.setSubscriptionId("Token");
        expectedResult.setBillingAddress(addressModel);
        expectedResult.setCcOwner("Owner");
        expectedResult.setNumber("4111111111111111");
        expectedResult.setType(CreditCardType.VISA);
        expectedResult.setValidToMonth("2");
        expectedResult.setValidToYear("2018");

        verify(modelService).save(creditCardPaymentInfoModel);
        assertEquals(expectedResult.getSubscriptionId(), result.getSubscriptionId());
        assertEquals(expectedResult.getBillingAddress(), result.getBillingAddress());
        assertEquals(expectedResult.getCcOwner(), result.getCcOwner());
        assertEquals(expectedResult.getNumber(), result.getNumber());
        assertEquals(expectedResult.getType(), result.getType());
        assertEquals(expectedResult.getValidToMonth(), result.getValidToMonth());
        assertEquals(expectedResult.getValidToYear(), result.getValidToYear());
    }

    @Test(expected = IllegalArgumentException.class)
    public void testSaveSubscriptionForNullSubscriptionResult() {
        pspCreditCardPaymentInfoCreateStrategy.saveSubscription(customerModel, null);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testSaveSubscriptionForNullCustomer() {
        pspCreditCardPaymentInfoCreateStrategy.saveSubscription(null, new PspCreateSubscriptionResult());
    }

    @Test
    public void testSaveSubscriptionEmptyValue() {
        // Given empty result

        // When
        CreditCardPaymentInfoModel cardPaymentInfo = pspCreditCardPaymentInfoCreateStrategy.saveSubscription(customerModel, new PspCreateSubscriptionResult());

        // Then
        assertEquals(cardPaymentInfo, null);
    }

    private void setupPspCreateSubscriptionResult(PspCreateSubscriptionResult pspCreateSubscriptionResult) {
        pspCreateSubscriptionResult.setValidity(PspValidity.INDEFINITE);
        pspCreateSubscriptionResult.setPaymentToken("Token");

        PaymentInfoData paymentInfoData = new PaymentInfoData();
        paymentInfoData.setCardCardType("VISA");
        paymentInfoData.setCardExpirationMonth(2);
        paymentInfoData.setCardExpirationYear(2018);
        paymentInfoData.setCardAccountHolderName("Owner");
        paymentInfoData.setCardAccountNumber("4111111111111111");
        pspCreateSubscriptionResult.setPaymentInfoData(paymentInfoData);
    }
}
