package com.psp.payment.tokenisation.strategies.impl;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.core.enums.CreditCardType;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspCreditCardTypeBrandMappingStrategyImplTest {
    private PspCreditCardTypeBrandMappingStrategyImpl pspCreditCardTypeBrandMappingStrategy;

    @Before
    public void setUp() {
        pspCreditCardTypeBrandMappingStrategy = new PspCreditCardTypeBrandMappingStrategyImpl();
    }

    @Test
    public void testBrandMappingForVisa() {
        // Given
        String creditCardBrand = "VISA";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        CreditCardType expectedResult = CreditCardType.VISA;
        assertEquals(expectedResult, result);
    }

    @Test
    public void testBrandMappingForAmex() {
        // Given
        String creditCardBrand = "American Express";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        CreditCardType expectedResult = CreditCardType.AMEX;
        assertEquals(expectedResult, result);
    }

    @Test
    public void testBrandMappingForMasterCard() {
        // Given
        String creditCardBrand = "MasterCard";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        CreditCardType expectedResult = CreditCardType.MASTER;
        assertEquals(expectedResult, result);
    }

    @Test
    public void testBrandMappingForMaestro() {
        // Given
        String creditCardBrand = "MAESTRO";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        CreditCardType expectedResult = CreditCardType.MAESTRO;
        assertEquals(expectedResult, result);
    }

    @Test
    public void testBrandMappingForDiners() {
        // Given
        String creditCardBrand = "Diners Club";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        assertEquals(CreditCardType.DINERS, result);
    }

    @Test
    public void testBrandMappingForAnyOtherBrand() {
        // Given
        String creditCardBrand = "Tesco";

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(creditCardBrand);

        // Then
        assertNull(result);
    }

    @Test
    public void testBrandMappingForNull() {
        // Given nothing

        // When
        CreditCardType result = pspCreditCardTypeBrandMappingStrategy.getCreditCardType(null);

        // Then
        assertNull(result);
    }
}
