package com.psp.payment.tokenisation.strategies.impl;


import com.psp.data.PspCreateSubscriptionResult;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.payment.common.enums.PspValidity;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.acceleratorservices.payment.data.PaymentInfoData;
import de.hybris.platform.core.enums.CreditCardType;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspDirectDebitPaymentInfoCreateStrategyImplTest {
    @InjectMocks
    private PspDirectDebitPaymentInfoCreateStrategyImpl pspDirectDebitPaymentInfoCreateStrategy;

    @Mock
    private ModelService modelService;

    @Mock
    private CartService cartService;

    @Mock
    private CartModel cartModel;

    private CustomerModel customerModel;

    private DirectDebitPaymentInfoModel directDebitPaymentInfoModel;

    private PspCreateSubscriptionResult pspCreateSubscriptionResult;

    private AddressModel addressModel;

    @Before
    public void setUp() throws Exception {
        pspCreateSubscriptionResult = new PspCreateSubscriptionResult();
        directDebitPaymentInfoModel = new DirectDebitPaymentInfoModel();
        customerModel = new CustomerModel();
        customerModel.setUid("uid");
        addressModel = new AddressModel();

        when(modelService.create(DirectDebitPaymentInfoModel.class)).thenReturn(directDebitPaymentInfoModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
        when(cartModel.getPaymentAddress()).thenReturn(addressModel);
    }

    @Test
    public void testSaveSubscriptionOnStorePaymentDetails() {
        // Given
        setupPspCreateSubscriptionResult(pspCreateSubscriptionResult);

        // When
        DirectDebitPaymentInfoModel result = pspDirectDebitPaymentInfoCreateStrategy.saveSubscription(customerModel, pspCreateSubscriptionResult);

        // Then
        DirectDebitPaymentInfoModel expectedResult = new DirectDebitPaymentInfoModel();
        expectedResult.setSubscriptionId("Token");
        expectedResult.setBillingAddress(addressModel);
        expectedResult.setAccountName("name");
        expectedResult.setAccountNumber("4111111111111111");
        expectedResult.setBrand("Direct Debits");

        verify(modelService).save(directDebitPaymentInfoModel);
        assertEquals(expectedResult.getSubscriptionId(), result.getSubscriptionId());
        assertEquals(expectedResult.getBillingAddress(), result.getBillingAddress());
    }

    @Test(expected = IllegalArgumentException.class)
    public void testSaveSubscriptionForNullSubscriptionResult() {
        pspDirectDebitPaymentInfoCreateStrategy.saveSubscription(customerModel, null);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testSaveSubscriptionForNullCustomer() {
        pspDirectDebitPaymentInfoCreateStrategy.saveSubscription(null, new PspCreateSubscriptionResult());
    }

    @Test
    public void testSaveSubscriptionEmptyValue() {
        // Given empty result

        // When
        DirectDebitPaymentInfoModel cardPaymentInfo = pspDirectDebitPaymentInfoCreateStrategy.saveSubscription(customerModel, new PspCreateSubscriptionResult());

        // Then
        assertEquals(cardPaymentInfo, null);
    }

    private void setupPspCreateSubscriptionResult(PspCreateSubscriptionResult pspCreateSubscriptionResult) {
        pspCreateSubscriptionResult.setValidity(PspValidity.INDEFINITE);
        pspCreateSubscriptionResult.setPaymentToken("Token");

        PaymentInfoData paymentInfoData = new PaymentInfoData();
        paymentInfoData.setCardCardType("Direct Debits");
        paymentInfoData.setCardAccountHolderName("Owner");
        paymentInfoData.setCardAccountNumber("4111111111111111");
        pspCreateSubscriptionResult.setPaymentInfoData(paymentInfoData);
    }


}