package com.psp.payment.tokenisation.strategies.impl;

import com.psp.payment.common.dto.PspErrorData;
import de.hybris.bootstrap.annotations.UnitTest;
import org.apache.commons.lang.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspErrorResponseStrategyImplTest {

    @InjectMocks
    private PspErrorResponseStrategyImpl pspErrorResponseStrategyImpl;

    private PspErrorData errorData;

    @Before
    public void setup() {
        errorData = new PspErrorData();
    }

    @Test
    public void noErrorCodeIsReturned() {
        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals(StringUtils.EMPTY, result);
    }

    @Test
    public void cardNoErrorCodeIsReturned() {
        // Given
        errorData.setNcErrorCardNo("12");
        errorData.setNcErrorCN("34");
        errorData.setNcErrorCVC("56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("12", result);
    }

    @Test
    public void customerNameErrorCodeIsReturned() {
        // Given
        errorData.setNcErrorCardNo("0");
        errorData.setNcErrorCN("34");
        errorData.setNcErrorCVC("56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("34", result);
    }

    @Test
    public void cvcErrorCodeIsReturned() {
        // Given
        errorData.setNcErrorCardNo("0");
        errorData.setNcErrorCN("0");
        errorData.setNcErrorCVC("56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("56", result);
    }

    @Test
    public void expiryDateErrorCodeIsReturned() {
        // Given
        errorData.setNcErrorCardNo("0");
        errorData.setNcErrorCN("0");
        errorData.setNcErrorCVC("0");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("78", result);
    }

    @Test
    public void generalErrorCodeIsReturned() {
        // Given
        errorData.setNcErrorCardNo("0");
        errorData.setNcErrorCN("0");
        errorData.setNcErrorCVC("0");
        errorData.setNcErrorED("0");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("90", result);
    }

    @Test
    public void negativeValuesAreIgnoredForCardNo() {
        // Given
        errorData.setNcErrorCardNo("-12");
        errorData.setNcErrorCN("34");
        errorData.setNcErrorCVC("56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("34", result);
    }

    @Test
    public void negativeValuesAreIgnoredForCustomerName() {
        // Given
        errorData.setNcErrorCardNo("-12");
        errorData.setNcErrorCN("-34");
        errorData.setNcErrorCVC("56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("56", result);
    }

    @Test
    public void negativeValuesAreIgnoredForCVC() {
        // Given
        errorData.setNcErrorCardNo("-12");
        errorData.setNcErrorCN("-34");
        errorData.setNcErrorCVC("-56");
        errorData.setNcErrorED("78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("78", result);
    }

    @Test
    public void negativeValuesAreIgnoredForExpiryDate() {
        // Given
        errorData.setNcErrorCardNo("-12");
        errorData.setNcErrorCN("-34");
        errorData.setNcErrorCVC("-56");
        errorData.setNcErrorED("-78");
        errorData.setNcError("90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals("90", result);
    }

    @Test
    public void negativeValuesAreIgnoredForGeneralErrorCode() {
        // Given
        errorData.setNcErrorCardNo("-12");
        errorData.setNcErrorCN("-34");
        errorData.setNcErrorCVC("-56");
        errorData.setNcErrorED("-78");
        errorData.setNcError("-90");

        // When
        String result = pspErrorResponseStrategyImpl.getErrorCode(errorData);

        // Then
        assertEquals(StringUtils.EMPTY, result);
    }
}
