package com.psp.payment.tokenisation.strategies.impl;

import com.psp.data.PspCreateSubscriptionResult;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.HashMap;
import java.util.Map;

import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspPaymentResponseInterpretationStrategyImplTest {

    @InjectMocks
    private PspPaymentResponseInterpretationStrategyImpl pspPaymentResponseInterpretationStrategyImpl;

    @Mock
    private Converter<Map<String, String>, PspCreateSubscriptionResult> createSubscriptionResultConverter;

    private Map<String, String> parameters;

    @Before
    public void setup() {
        parameters = new HashMap<>();
    }

    @Test
    public void testInterpretResponseSuccess() {
        // Given
        setUpRequestMap(parameters);

        // When
        PspCreateSubscriptionResult result = pspPaymentResponseInterpretationStrategyImpl.interpretResponse(parameters);

        //Then
        verify(createSubscriptionResultConverter, times(1)).convert(parameters);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testInterpretResponseWithNullParameter() {
        // Given nothing

        // When
        PspCreateSubscriptionResult result = pspPaymentResponseInterpretationStrategyImpl.interpretResponse(null);

        // Then exception is thrown
    }

    @Test(expected = IllegalArgumentException.class)
    public void testInterpretResponseWithEmptyMap() {
        // Given empty input map

        // When
        PspCreateSubscriptionResult result = pspPaymentResponseInterpretationStrategyImpl.interpretResponse(parameters);

        // Then exception is thrown
    }

    private Map<String, String> setUpRequestMap(final Map<String, String> parameters) {
        parameters.put("ALIAS.ALIASID", "34F5302C-85D7-4F35-BDF5-103CCEC2FB61");
        parameters.put("ALIAS.NCERROR", "50001111");
        parameters.put("ALIAS.NCERRORCARDNO", "50001111");
        parameters.put("ALIAS.NCERRORCN", "50001111");
        parameters.put("ALIAS.NCERRORCVC", "50001111");
        parameters.put("ALIAS.NCERRORED", "50001111");
        parameters.put("ALIAS.ORDERID", "000001-10");
        parameters.put("ALIAS.STATUS", "0");
        parameters.put("ALIAS.STOREPERMANENTLY", "Y");
        parameters.put("CARD.BRAND", "Visa");
        parameters.put("CARD.CARDHOLDERNAME", "Card Holder Name");
        parameters.put("CARD.CARDNUMBER", "3717-111111-11111");
        parameters.put("CARD.CVC", "123");
        parameters.put("CARD.EXPIRYDATE", "0220");
        parameters.put("SHASIGN", "0DEDADB0CB36CB23E99C9C76AA0AE6A6D0E572D0344136FE5AF27D24FC7337C8");

        return parameters;
    }
}