package com.psp.payment.tokenisation.strategies.impl;

import com.psp.constants.PspConstants;
import com.psp.data.PspCreateSubscriptionResult;
import com.psp.data.PspPaymentSubscriptionResultItem;
import com.psp.model.DirectDebitPaymentInfoModel;
import com.psp.payment.common.factories.PspPaymentInfoConvertStrategyFactory;
import com.psp.payment.common.factories.PspPaymentInfoCreateStrategyFactory;
import com.psp.payment.common.strategies.PspPaymentInfoCreateStrategy;
import com.psp.payment.common.strategies.PspPaymentTransactionStrategy;
import com.psp.payment.tokenisation.strategies.PspCreditCardPaymentInfoCreateStrategy;
import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.strategies.CheckoutCustomerStrategy;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.type.ComposedTypeModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@UnitTest
@RunWith(MockitoJUnitRunner.class)
public class PspSucessResponseStrategyImplTest {

    @InjectMocks
    private PspSucessResponseStrategyImpl successStrategy;

    @Mock
    private PspPaymentTransactionStrategy pspPaymentTransactionStrategy;

    @Mock
    private CheckoutCustomerStrategy checkoutCustomerStrategy;

    @Mock
    private PspCreditCardPaymentInfoCreateStrategy pspCreditCardPaymentInfoCreateStrategy;

    @Mock
    private PspPaymentInfoConvertStrategyFactory pspPaymentInfoConvertStrategyFactory;

    @Mock
    private PspCreateSubscriptionResult result;

    @Mock
    private PaymentTransactionEntryModel transactionEntry;

    @Mock
    private CustomerModel customer;

    private PaymentModeModel paymentModeModel;

    @Mock
    private CartService cartService;

    private CartModel cartModel;

    @Mock
    private PspPaymentInfoCreateStrategyFactory pspPaymentInfoCreateStrategyFactory;

    @Mock
    private PspPaymentInfoCreateStrategy pspPaymentInfoCreateStrategy;

    @Mock
    private PspCreditCardPaymentInfoConvertStrategyImpl pspCreditCardPaymentInfoConvertStrategy;

    @Mock
    private PspDirectDebitPaymentInfoConvertStrategyImpl pspDirectDebitPaymentInfoConvertStrategy;

    private PaymentInfoModel paymentInfoModel;


    @Before
    public void setUp() {
        when(checkoutCustomerStrategy.getCurrentUserForCheckout()).thenReturn(customer);
        cartModel = new CartModel();
        paymentModeModel = new PaymentModeModel();
        paymentModeModel.setBrand(PspConstants.CREDIT_CARD);
        ComposedTypeModel composedTypeModel = new ComposedTypeModel();
        composedTypeModel.setCode("CreditCardPaymentInfo");
        paymentModeModel.setPaymentInfoType(composedTypeModel);
        cartModel.setPaymentMode(paymentModeModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);
    }

    @Test
    public void testHandleResponseSucces() {
        //Given
        CCPaymentInfoData paymentInfoData = new CCPaymentInfoData();
        when(result.getPaymentToken()).thenReturn("1234");
        when(pspPaymentTransactionStrategy.savePaymentTransactionEntry(customer, "1234")).thenReturn(transactionEntry);
        when(pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(any(ComposedTypeModel.class))).thenReturn(pspPaymentInfoCreateStrategy);
        paymentInfoModel = new CreditCardPaymentInfoModel();
        when(pspPaymentInfoCreateStrategy.saveSubscription(customer, result)).thenReturn(paymentInfoModel);
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(any(CreditCardPaymentInfoModel.class))).thenReturn(pspCreditCardPaymentInfoConvertStrategy);
        //When
        PspPaymentSubscriptionResultItem subscriptionResult = successStrategy.handleResponse(result);
        //Then
        verify(pspPaymentTransactionStrategy).savePaymentTransactionEntry(customer, "1234");
    }

    @Test(expected = IllegalArgumentException.class)
    public void testHandleResponseNullValue() {
        //Given nothing

        //When
        PspPaymentSubscriptionResultItem subscriptionResult = successStrategy.handleResponse(null);

        // Then Exception
    }

    @Test
    public void testHandleResponseEmptyValue() {
        //Given
        when(result.getPaymentToken()).thenReturn("1234");
        when(pspPaymentTransactionStrategy.savePaymentTransactionEntry(customer, "1234")).thenReturn(transactionEntry);
        when(pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(any(ComposedTypeModel.class))).thenReturn(pspPaymentInfoCreateStrategy);
        paymentInfoModel = new CreditCardPaymentInfoModel();
        when(pspPaymentInfoCreateStrategy.saveSubscription(customer, result)).thenReturn(paymentInfoModel);

        //When
        PspPaymentSubscriptionResultItem subscriptionResult = successStrategy.handleResponse(new PspCreateSubscriptionResult());

        //Then
        assertEquals(subscriptionResult.getStoredCard(), null);
    }

    @Test
    public void testHandleResponseSucceForDirectDebits() {
        //Given
        paymentModeModel.setBrand("DirectDebits");
        ComposedTypeModel composedTypeModel = new ComposedTypeModel();
        composedTypeModel.setCode("DirectDebitPaymentInfo");
        paymentModeModel.setPaymentInfoType(composedTypeModel);
        cartModel.setPaymentMode(paymentModeModel);
        when(cartService.getSessionCart()).thenReturn(cartModel);

        when(result.getPaymentToken()).thenReturn("1234");
        when(pspPaymentTransactionStrategy.savePaymentTransactionEntry(customer, "1234")).thenReturn(transactionEntry);
        when(pspPaymentInfoCreateStrategyFactory.getPaymentInfoCreateStrategy(any(ComposedTypeModel.class))).thenReturn(pspPaymentInfoCreateStrategy);
        when(pspPaymentInfoConvertStrategyFactory.getPaymentInfoConvertStrategy(any(DirectDebitPaymentInfoModel.class))).thenReturn(pspDirectDebitPaymentInfoConvertStrategy);
        paymentInfoModel = new DirectDebitPaymentInfoModel();
        when(pspPaymentInfoCreateStrategy.saveSubscription(customer, result)).thenReturn(paymentInfoModel);
        //When
        PspPaymentSubscriptionResultItem subscriptionResult = successStrategy.handleResponse(result);

        //Then
        verify(pspPaymentTransactionStrategy).savePaymentTransactionEntry(customer, "1234");
    }

}
