package com.psp.addon.checkout.steps.validation.impl;

import com.psp.payment.common.facades.order.PspCheckoutFacade;
import de.hybris.platform.acceleratorstorefrontcommons.checkout.steps.validation.AbstractCheckoutStepValidator;
import de.hybris.platform.acceleratorstorefrontcommons.checkout.steps.validation.ValidationResults;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.util.GlobalMessages;
import de.hybris.platform.commercefacades.order.data.CartData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;


public class ResponsiveSummaryCheckoutStepValidator extends AbstractCheckoutStepValidator {

    private static final Logger LOG = LoggerFactory.getLogger(ResponsiveSummaryCheckoutStepValidator.class);

    @Resource
    private PspCheckoutFacade pspCheckoutFacade;

    @Override
    public ValidationResults validateOnEnter(final RedirectAttributes redirectAttributes) {
        if (!getCheckoutFlowFacade().hasValidCart()) {
            LOG.info("Missing, empty or unsupported cart");
            return ValidationResults.REDIRECT_TO_CART;
        }

        if (getCheckoutFlowFacade().hasNoDeliveryAddress()) {
            GlobalMessages.addFlashMessage(redirectAttributes, GlobalMessages.INFO_MESSAGES_HOLDER,
                    "checkout.multi.deliveryAddress.notprovided");
            return ValidationResults.REDIRECT_TO_DELIVERY_ADDRESS;
        }

        if (getCheckoutFlowFacade().hasNoDeliveryMode()) {
            GlobalMessages.addFlashMessage(redirectAttributes, GlobalMessages.INFO_MESSAGES_HOLDER,
                    "checkout.multi.deliveryMethod.notprovided");
            return ValidationResults.REDIRECT_TO_DELIVERY_METHOD;
        }

        if (pspCheckoutFacade.hasNoPaymentInfo()) {
            GlobalMessages.addFlashMessage(redirectAttributes, GlobalMessages.INFO_MESSAGES_HOLDER,
                    "checkout.multi.paymentDetails.notprovided");
            return ValidationResults.REDIRECT_TO_PAYMENT_METHOD;
        }

        final CartData cartData = getCheckoutFacade().getCheckoutCart();

        if (!getCheckoutFacade().hasShippingItems()) {
            cartData.setDeliveryAddress(null);
        }

        if (!getCheckoutFacade().hasPickUpItems() && cartData.getDeliveryMode().getCode().equals("pickup")) {
            return ValidationResults.REDIRECT_TO_DELIVERY_ADDRESS;
        }

        return ValidationResults.SUCCESS;
    }
}