package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.controllers.PspaddonControllerConstants;
import com.psp.addon.forms.PSPAddressForm;
import com.psp.payment.common.services.PspAddressService;
import com.psp.payment.common.services.PspCustomerService;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.PreValidateCheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.PreValidateQuoteCheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.acceleratorstorefrontcommons.checkout.steps.CheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.checkout.steps.validation.ValidationResults;
import de.hybris.platform.acceleratorstorefrontcommons.constants.WebConstants;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.pages.checkout.steps.AbstractCheckoutStepController;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.util.GlobalMessages;
import de.hybris.platform.acceleratorstorefrontcommons.forms.AddressForm;
import de.hybris.platform.acceleratorstorefrontcommons.util.AddressDataUtil;
import de.hybris.platform.cms2.exceptions.CMSItemNotFoundException;
import de.hybris.platform.commercefacades.address.data.AddressVerificationResult;
import de.hybris.platform.commercefacades.order.data.CartData;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commercefacades.user.data.CountryData;
import de.hybris.platform.commercefacades.user.data.RegionData;
import de.hybris.platform.commerceservices.address.AddressVerificationDecision;
import de.hybris.platform.util.Config;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Validator;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;
import java.util.Set;

import static java.lang.Boolean.TRUE;

@Controller
@RequestMapping(value = "/checkout/multi/delivery-address")
public class DeliveryAddressCheckoutStepController extends AbstractCheckoutStepController {
    private static final String DELIVERY_ADDRESS = "delivery-address";

    private static final String SHOW_SAVE_TO_ADDRESS_BOOK_ATTR = "showSaveToAddressBook";

    @Resource
    private AddressDataUtil addressDataUtil;

    @Resource(name = "billingAddressValidator")
    private Validator billingAddressValidator;

    @Resource
    private PspAddressService pspAddressService;

    @Resource
    private PspCustomerService pspCustomerService;

    @Override
    @RequestMapping(value = "/add", method = RequestMethod.GET)
    @RequireHardLogIn
    @PreValidateQuoteCheckoutStep
    @PreValidateCheckoutStep(checkoutStep = DELIVERY_ADDRESS)
    public String enterStep(final Model model, final RedirectAttributes redirectAttributes) throws CMSItemNotFoundException {
        getCheckoutFacade().setDeliveryAddressIfAvailable();
        final CartData cartData = getCheckoutFacade().getCheckoutCart();
        populateCommonModelAttributes(model, cartData, new PSPAddressForm());

        return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
    }

    @RequestMapping(value = "/add", method = RequestMethod.POST)
    @RequireHardLogIn
    public String add(final PSPAddressForm addressForm, final BindingResult bindingResult, final Model model,
                      final RedirectAttributes redirectModel) throws CMSItemNotFoundException {
        final CartData cartData = getCheckoutFacade().getCheckoutCart();

        getAddressValidator().validate(addressForm, bindingResult);

        if (!addressForm.getUseDeliveryAddress()) {
            getBillingAddressValidator().validate(addressForm, bindingResult);
        }

        populateCommonModelAttributes(model, cartData, addressForm);

        if (bindingResult.hasErrors()) {
            GlobalMessages.addErrorMessage(model, "address.error.formentry.invalid");
            return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
        }

        final AddressData shippingAddress = getAddressData(addressForm, Boolean.FALSE);

        // Verify the address data.
        final AddressVerificationResult<AddressVerificationDecision> verificationResult = getAddressVerificationFacade()
                .verifyAddressData(shippingAddress);
        final boolean addressRequiresReview = getAddressVerificationResultHandler().handleResult(verificationResult, shippingAddress,
                model, redirectModel, bindingResult, getAddressVerificationFacade().isCustomerAllowedToIgnoreAddressSuggestions(),
                "checkout.multi.address.updated");

        if (addressRequiresReview) {
            return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
        }

        getUserFacade().addAddress(shippingAddress);

        final AddressData previousSelectedAddress = getCheckoutFacade().getCheckoutCart().getDeliveryAddress();
        // Set the new address as the selected checkout delivery address
        getCheckoutFacade().setDeliveryAddress(shippingAddress);

        // temporary address should be removed
        if (previousSelectedAddress != null && !previousSelectedAddress.isVisibleInAddressBook()) {
            getUserFacade().removeAddress(previousSelectedAddress);
        }

        // Add billing address to the cart
        AddressData billingAddress = shippingAddress;

        if (!addressForm.getUseDeliveryAddress()) {
            billingAddress = getAddressData(addressForm, TRUE);
        }

        getPspAddressService().saveBillingAddressToCurrentCart(billingAddress, addressForm.getUseDeliveryAddress());
        pspCustomerService.setCustomerNameFromAddress(billingAddress);

        return getCheckoutStep().nextStep();
    }

    private AddressData getAddressData(PSPAddressForm addressForm, boolean isBillingAddress) {
        AddressData newAddress = new AddressData();

        if (isBillingAddress) {
            newAddress.setTitleCode(addressForm.getBillTo_titleCode());
            newAddress.setFirstName(addressForm.getBillTo_firstName());
            newAddress.setLastName(addressForm.getBillTo_lastName());
            newAddress.setLine1(addressForm.getBillTo_line1());
            newAddress.setLine2(addressForm.getBillTo_line2());
            newAddress.setTown(addressForm.getBillTo_townCity());
            newAddress.setPostalCode(addressForm.getBillTo_postcode());
            newAddress.setBillingAddress(TRUE);
            newAddress.setShippingAddress(Boolean.FALSE);
            newAddress.setVisibleInAddressBook(Boolean.FALSE);
            newAddress.setPhone(addressForm.getBillTo_phone());

            if (addressForm.getBillTo_countryIso() != null) {
                final CountryData countryData = getI18NFacade().getCountryForIsocode(addressForm.getBillTo_countryIso());
                newAddress.setCountry(countryData);
            }

            if (StringUtils.isNotEmpty(addressForm.getBillTo_regionIso())) {
                final RegionData regionData = getI18NFacade().getRegion(addressForm.getBillTo_countryIso(),
                        addressForm.getBillTo_regionIso());
                newAddress.setRegion(regionData);
            }
        } else {
            newAddress = addressDataUtil.convertToAddressData(addressForm);
            processAddressVisibilityAndDefault(addressForm, newAddress);
        }

        return newAddress;
    }

    @RequestMapping(value = "/edit", method = RequestMethod.GET)
    @RequireHardLogIn
    public String editAddressForm(@RequestParam("editAddressCode") final String editAddressCode, final Model model,
                                  final RedirectAttributes redirectAttributes) throws CMSItemNotFoundException {
        final ValidationResults validationResults = getCheckoutStep().validate(redirectAttributes);
        if (getCheckoutStep().checkIfValidationErrors(validationResults)) {
            return getCheckoutStep().onValidation(validationResults);
        }

        AddressData addressData = null;
        if (StringUtils.isNotEmpty(editAddressCode)) {
            addressData = getCheckoutFacade().getDeliveryAddressForCode(editAddressCode);
        }

        final PSPAddressForm addressForm = new PSPAddressForm();
        final boolean hasAddressData = addressData != null;
        if (hasAddressData) {
            addressDataUtil.convert(addressData, addressForm);
        }

        final CartData cartData = getCheckoutFacade().getCheckoutCart();
        populateCommonModelAttributes(model, cartData, addressForm);

        if (addressData != null) {
            model.addAttribute(SHOW_SAVE_TO_ADDRESS_BOOK_ATTR, !addressData.isVisibleInAddressBook());
        }

        return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
    }

    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    @RequireHardLogIn
    public String edit(final PSPAddressForm addressForm, final BindingResult bindingResult, final Model model,
                       final RedirectAttributes redirectModel) throws CMSItemNotFoundException {
        getAddressValidator().validate(addressForm, bindingResult);

        final CartData cartData = getCheckoutFacade().getCheckoutCart();
        populateCommonModelAttributes(model, cartData, addressForm);

        if (bindingResult.hasErrors()) {
            GlobalMessages.addErrorMessage(model, "address.error.formentry.invalid");
            return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
        }

        final AddressData newAddress = addressDataUtil.convertToAddressData(addressForm);
        processAddressVisibility(addressForm, newAddress);

        newAddress.setDefaultAddress(CollectionUtils.isEmpty(getUserFacade().getAddressBook())
                || getUserFacade().getAddressBook().size() == 1
                || TRUE.equals(addressForm.getDefaultAddress()));

        // Verify the address data.
        final AddressVerificationResult<AddressVerificationDecision> verificationResult = getAddressVerificationFacade()
                .verifyAddressData(newAddress);
        final boolean addressRequiresReview = getAddressVerificationResultHandler().handleResult(verificationResult, newAddress,
                model, redirectModel, bindingResult, getAddressVerificationFacade().isCustomerAllowedToIgnoreAddressSuggestions(),
                "checkout.multi.address.updated");

        if (addressRequiresReview) {
            if (StringUtils.isNotEmpty(addressForm.getAddressId())) {
                final AddressData addressData = getCheckoutFacade().getDeliveryAddressForCode(addressForm.getAddressId());
                if (addressData != null) {
                    model.addAttribute(SHOW_SAVE_TO_ADDRESS_BOOK_ATTR, !addressData.isVisibleInAddressBook());
                    model.addAttribute("edit", TRUE);
                }
            }

            return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.AddEditDeliveryAddressPage;
        }

        getUserFacade().editAddress(newAddress);
        getCheckoutFacade().setDeliveryModeIfAvailable();
        getCheckoutFacade().setDeliveryAddress(newAddress);

        return getCheckoutStep().nextStep();
    }

    @RequestMapping(value = "/remove", method =
            {RequestMethod.GET, RequestMethod.POST})
    @RequireHardLogIn
    public String removeAddress(@RequestParam("addressCode") final String addressCode, final RedirectAttributes redirectModel,
                                final Model model) throws CMSItemNotFoundException {
        if (getCheckoutFacade().isRemoveAddressEnabledForCart()) {
            final AddressData addressData = new AddressData();
            addressData.setId(addressCode);
            getUserFacade().removeAddress(addressData);
            GlobalMessages.addFlashMessage(redirectModel, GlobalMessages.CONF_MESSAGES_HOLDER,
                    "account.confirmation.address.removed");
        }
        storeCmsPageInModel(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));
        setUpMetaDataForContentPage(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));
        model.addAttribute("addressForm", new AddressForm());

        return getCheckoutStep().currentStep();
    }

    @RequestMapping(value = "/select", method = RequestMethod.POST)
    @RequireHardLogIn
    public String doSelectSuggestedAddress(final AddressForm addressForm, final RedirectAttributes redirectModel) {
        final Set<String> resolveCountryRegions = org.springframework.util.StringUtils
                .commaDelimitedListToSet(Config.getParameter("resolve.country.regions"));

        final AddressData selectedAddress = addressDataUtil.convertToAddressData(addressForm);

        final CountryData countryData = selectedAddress.getCountry();

        if (!resolveCountryRegions.contains(countryData.getIsocode())) {
            selectedAddress.setRegion(null);
        }

        if (addressForm.getSaveInAddressBook() != null) {
            selectedAddress.setVisibleInAddressBook(addressForm.getSaveInAddressBook());
        }

        if (TRUE.equals(addressForm.getEditAddress())) {
            getUserFacade().editAddress(selectedAddress);
        } else {
            getUserFacade().addAddress(selectedAddress);
        }

        final AddressData previousSelectedAddress = getCheckoutFacade().getCheckoutCart().getDeliveryAddress();
        // Set the new address as the selected checkout delivery address
        getCheckoutFacade().setDeliveryAddress(selectedAddress);
        if (previousSelectedAddress != null && !previousSelectedAddress.isVisibleInAddressBook()) { // temporary address should be removed
            getUserFacade().removeAddress(previousSelectedAddress);
        }

        GlobalMessages.addFlashMessage(redirectModel, GlobalMessages.CONF_MESSAGES_HOLDER, "checkout.multi.address.added");

        return getCheckoutStep().nextStep();
    }


    /**
     * This method gets called when the "Use this Address" button is clicked. It sets the selected delivery address on
     * the checkout facade - if it has changed, and reloads the page highlighting the selected delivery address.
     *
     * @param selectedAddressCode - the id of the delivery address.
     * @return - a URL to the page to load.
     */
    @RequestMapping(value = "/select", method = RequestMethod.GET)
    @RequireHardLogIn
    public String doSelectDeliveryAddress(@RequestParam("selectedAddressCode") final String selectedAddressCode,
                                          final RedirectAttributes redirectAttributes) {
        final ValidationResults validationResults = getCheckoutStep().validate(redirectAttributes);
        if (getCheckoutStep().checkIfValidationErrors(validationResults)) {
            return getCheckoutStep().onValidation(validationResults);
        }
        if (StringUtils.isNotBlank(selectedAddressCode)) {
            final AddressData selectedAddressData = getCheckoutFacade().getDeliveryAddressForCode(selectedAddressCode);
            final boolean hasSelectedAddressData = selectedAddressData != null;
            if (hasSelectedAddressData) {
                setDeliveryAddress(selectedAddressData);
            }
        }
        return getCheckoutStep().nextStep();
    }

    @RequestMapping(value = "/back", method = RequestMethod.GET)
    @RequireHardLogIn
    @Override
    public String back(final RedirectAttributes redirectAttributes) {
        return getCheckoutStep().previousStep();
    }

    @RequestMapping(value = "/next", method = RequestMethod.GET)
    @RequireHardLogIn
    @Override
    public String next(final RedirectAttributes redirectAttributes) {
        return getCheckoutStep().nextStep();
    }

    protected String getBreadcrumbKey() {
        return "checkout.multi." + getCheckoutStep().getProgressBarId() + ".breadcrumb";
    }

    protected CheckoutStep getCheckoutStep() {
        return getCheckoutStep(DELIVERY_ADDRESS);
    }

    protected void populateCommonModelAttributes(final Model model, final CartData cartData, final PSPAddressForm addressForm)
            throws CMSItemNotFoundException {
        model.addAttribute("cartData", cartData);
        model.addAttribute("addressForm", addressForm);
        model.addAttribute("deliveryAddresses", getDeliveryAddresses(cartData.getDeliveryAddress()));
        model.addAttribute("noAddress", getCheckoutFlowFacade().hasNoDeliveryAddress());
        model.addAttribute("addressFormEnabled", getCheckoutFacade().isNewAddressEnabledForCart());
        model.addAttribute("removeAddressEnabled", getCheckoutFacade().isRemoveAddressEnabledForCart());
        model.addAttribute(SHOW_SAVE_TO_ADDRESS_BOOK_ATTR, TRUE);
        model.addAttribute(WebConstants.BREADCRUMBS_KEY, getResourceBreadcrumbBuilder().getBreadcrumbs(getBreadcrumbKey()));
        model.addAttribute("metaRobots", "noindex,nofollow");
        if (StringUtils.isNotBlank(addressForm.getCountryIso())) {
            model.addAttribute("regions", getI18NFacade().getRegionsForCountryIso(addressForm.getCountryIso()));
            model.addAttribute("country", addressForm.getCountryIso());
        }
        prepareDataForPage(model);
        storeCmsPageInModel(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));
        setUpMetaDataForContentPage(model, getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL));
        setCheckoutStepLinksForModel(model, getCheckoutStep());
    }

    protected void processAddressVisibilityAndDefault(final AddressForm addressForm, final AddressData newAddress) {
        if (addressForm.getSaveInAddressBook() != null) {
            newAddress.setVisibleInAddressBook(addressForm.getSaveInAddressBook());
            if (addressForm.getSaveInAddressBook() && CollectionUtils.isEmpty(getUserFacade().getAddressBook())) {
                newAddress.setDefaultAddress(true);
            }
        } else if (getCheckoutCustomerStrategy().isAnonymousCheckout()) {
            newAddress.setDefaultAddress(true);
            newAddress.setVisibleInAddressBook(true);
        }
    }

    protected void processAddressVisibility(final AddressForm addressForm, final AddressData newAddress) {
        if (addressForm.getSaveInAddressBook() == null) {
            newAddress.setVisibleInAddressBook(true);
        } else {
            newAddress.setVisibleInAddressBook(TRUE.equals(addressForm.getSaveInAddressBook()));
        }
    }

    protected void setDeliveryAddress(final AddressData selectedAddressData) {
        final AddressData cartCheckoutDeliveryAddress = getCheckoutFacade().getCheckoutCart().getDeliveryAddress();
        if (isAddressIdChanged(cartCheckoutDeliveryAddress, selectedAddressData)) {
            getCheckoutFacade().setDeliveryAddress(selectedAddressData);
            if (cartCheckoutDeliveryAddress != null && !cartCheckoutDeliveryAddress.isVisibleInAddressBook()) { // temporary address should be removed
                getUserFacade().removeAddress(cartCheckoutDeliveryAddress);
            }
        }
    }

    public Validator getBillingAddressValidator() {
        return billingAddressValidator;
    }

    public void setBillingAddressValidator(Validator billingAddressValidator) {
        this.billingAddressValidator = billingAddressValidator;
    }

    public PspAddressService getPspAddressService() {
        return pspAddressService;
    }

    public void setPspAddressService(PspAddressService pspAddressService) {
        this.pspAddressService = pspAddressService;
    }
}
