package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.constants.PspaddonWebConstants;
import com.psp.constants.PspConstants;
import com.psp.payment.common.services.PspPaymentService;
import com.psp.payment.common.strategies.PspSignatureValidationStrategy;
import com.psp.payment.directlink.services.PspDirectLinkService;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.commercefacades.order.data.OrderData;
import de.hybris.platform.order.InvalidCartException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Map;


@Controller
@RequestMapping(value = "/checkout/multi/psp/3dsecure")
public class DirectLink3DSecureCallbackController extends PaymentMethodCheckoutStepController {

    private static final Logger LOGGER = LoggerFactory.getLogger(DirectLink3DSecureCallbackController.class);

    @Resource
    private PspDirectLinkService pspDirectLinkService;

    @Resource
    private PspSignatureValidationStrategy pspSignatureValidationStrategy;

    @Resource
    private PspPaymentService pspPaymentService;

    /**
     * This method handles the 3D Secure callback from PSP. Any changes to the
     * mapped URL should also be reflected in the configuration properties
     * psp.conf.3dsecure.accept.url, psp.conf.3dsecure.decline.url and
     * psp.conf.3dsecure.exception.url.
     *
     * @param request            the http request
     * @param model              the spring mvc model
     * @param redirectAttributes spring redirect attributes
     * @return The review step or confirmation on successful transaction
     */
    @RequestMapping(value = "/response", method = {RequestMethod.POST, RequestMethod.GET})
    @RequireHardLogIn
    public String handleResponse(final HttpServletRequest request, final Model model, RedirectAttributes redirectAttributes) {
        Map<String, String> parameters = getParameterMap(request);

        // if the status is not successful then do not create the order
        if (pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_DL_SHA_SIGNATURE)
                && parameters.containsKey(PspConstants.KEY_STATUS)) {
            boolean isValidResponse = pspPaymentService.validateCartDetails(parameters, PspConstants.KEY_DL_ORDERID, PspConstants.KEY_RESULT_AMOUNT);

            if (isValidResponse) {
                pspDirectLinkService.createPaymentTransactionAndEntry(parameters);

                if ((getStatus(parameters) == PspPaymentStatus.STATUS_PAYMENT_REQUESTED
                        || getStatus(parameters) == PspPaymentStatus.STATUS_AUTHORISED)) {
                    final OrderData orderData;

                    try {
                        orderData = getCheckoutFacade().placeOrder();
                        return redirectToOrderConfirmationPage(orderData);
                    } catch (InvalidCartException ice) {
                        LOGGER.error("Failed to place Order", ice);
                    }
                } else {
                    String errorCode = getErrorCode(parameters);
                    redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, errorCode);
                    return getCheckoutStep().currentStep();
                }
            }
        }

        redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, PspaddonWebConstants.KEY_AUTHORISATION_FAILED);
        return getCheckoutStep().currentStep();
    }
}
