package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.constants.PspaddonWebConstants;
import com.psp.addon.controllers.util.PspGlobalMessages;
import com.psp.constants.PspConstants;
import com.psp.payment.common.services.PspPaymentService;
import com.psp.payment.common.strategies.PspSignatureValidationStrategy;
import com.psp.payment.directlink.services.PspDirectLinkService;
import com.psp.payment.integration.response.status.PspPaymentStatus;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.commercefacades.order.data.OrderData;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Map;

@Controller
@RequestMapping(value = "checkout/multi/psp/flex")
public class FlexPaymentsCallbackController extends PaymentMethodCheckoutStepController {

    private static final Logger LOGGER = Logger.getLogger(FlexPaymentsCallbackController.class);

    @Resource
    private PspPaymentService pspPaymentService;

    @Resource
    private PspDirectLinkService pspDirectLinkService;

    @Resource
    private PspSignatureValidationStrategy pspSignatureValidationStrategy;

    @RequestMapping(value = "/response", method = {RequestMethod.POST, RequestMethod.GET})
    @RequireHardLogIn
    public String handleResponse(final HttpServletRequest request, final Model model, RedirectAttributes redirectAttributes) {
        Map<String, String> parameters = getParameterMap(request);

        if (pspSignatureValidationStrategy.validateSignature(parameters, PspConstants.KEY_DL_SHA_SIGNATURE)
                && parameters.containsKey(PspConstants.KEY_STATUS)) {
            boolean isValidResponse = pspPaymentService.validateCartDetails(parameters, PspConstants.KEY_DL_ORDERID, PspConstants.KEY_RESULT_AMOUNT);

            if (isValidResponse) {
                pspPaymentService.createPaymentInfo(parameters);
                pspDirectLinkService.createPaymentTransactionAndEntry(parameters);

                if (getStatus(parameters) == PspPaymentStatus.STATUS_PAYMENT_REQUESTED
                        || getStatus(parameters) == PspPaymentStatus.STATUS_PAYMENT_PROCESSING
                        || getStatus(parameters) == PspPaymentStatus.STATUS_AUTHORISED
                        || getStatus(parameters) == PspPaymentStatus.STATUS_AUTHORISATION_WAITING
                        || getStatus(parameters) == PspPaymentStatus.STATUS_AUTHORISATION_NOT_KNOWN
                        || getStatus(parameters) == PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN) {

                    if (getStatus(parameters) == PspPaymentStatus.STATUS_PAYMENT_UNCERTAIN
                            || getStatus(parameters) == PspPaymentStatus.STATUS_AUTHORISATION_NOT_KNOWN) {
                        redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, PspConstants.ERROR_CODE_PAYMENT_UNCERTAIN);
                    }

                    final OrderData orderData;

                    try {
                        orderData = getCheckoutFacade().placeOrder();
                        return redirectToOrderConfirmationPage(orderData);
                    } catch (final Exception e) {
                        LOGGER.error("Failed to place Order", e);
                    }
                } else {
                    String errorCode = getErrorCode(parameters);
                    redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, errorCode);
                    return getCheckoutStep().currentStep();
                }
            }
        }

        redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, PspaddonWebConstants.KEY_AUTHORISATION_FAILED);
        return getCheckoutStep().currentStep();
    }

    @RequestMapping(value = "/callback", method = {RequestMethod.POST, RequestMethod.GET})
    @RequireHardLogIn
    public String handleCallback(final Model model, RedirectAttributes redirectAttributes) {
        redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE, PspConstants.ERROR_CODE_CANCELLED_BY_USER);
        return getCheckoutStep().currentStep();
    }
}
