package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.constants.PspaddonWebConstants;
import com.psp.addon.controllers.PspaddonControllerConstants;
import com.psp.addon.controllers.util.PspGlobalMessages;
import com.psp.addon.forms.SelectPaymentMethodForm;
import com.psp.constants.PspConstants;
import com.psp.payment.common.dto.PspPaymentModeData;
import com.psp.payment.common.facades.order.PspCheckoutFacade;
import com.psp.payment.common.facades.payment.PspPaymentModeFacade;
import com.psp.payment.common.services.PspErrorMessageService;
import com.psp.payment.directlink.dto.Psp3DSecureDTO;
import de.hybris.platform.acceleratorservices.payment.data.PaymentData;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.PreValidateCheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.PreValidateQuoteCheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.annotations.RequireHardLogIn;
import de.hybris.platform.acceleratorstorefrontcommons.checkout.steps.CheckoutStep;
import de.hybris.platform.acceleratorstorefrontcommons.constants.WebConstants;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.pages.checkout.steps.AbstractCheckoutStepController;
import de.hybris.platform.acceleratorstorefrontcommons.forms.PlaceOrderForm;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.cms2.exceptions.CMSItemNotFoundException;
import de.hybris.platform.cms2.model.pages.ContentPageModel;
import de.hybris.platform.commercefacades.order.data.CartData;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.session.SessionService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.support.BindingAwareModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import static com.psp.addon.constants.PspaddonConstants.PARAM_HOSTED_PAYMENT_FRAME_URL;
import static java.lang.Boolean.TRUE;


/**
 * PSP specific implementation for the payment method checkout step.
 */
@Controller
@RequestMapping(value = "/checkout/multi/payment-method")
public class PaymentMethodCheckoutStepController extends AbstractCheckoutStepController {

    private static final Logger LOGGER = Logger.getLogger(PaymentMethodCheckoutStepController.class);

    private static final String PAYMENT_METHOD = "payment-method";

    private static final String CART_DATA_ATTR = "cartData";

    private static final String ACCEPT_HEADER = "accept";

    private static final String USER_AGENT_HEADER = "user-agent";

    private static final String PSP_3D_SECURE_DTO = "psp3DSecureDTO";

    private static final String CHALLENGE_WINDOW_SIZE = "01";

    @Resource
    private PspCheckoutFacade pspCheckoutFacade;

    @Resource
    private PspPaymentModeFacade pspPaymentModeFacade;

    @Resource
    private PspErrorMessageService pspErrorMessageService;

    @Resource
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private SessionService sessionService;

    @Override
    @RequestMapping(value = "/add", method = RequestMethod.GET)
    @RequireHardLogIn
    @PreValidateQuoteCheckoutStep
    @PreValidateCheckoutStep(checkoutStep = PAYMENT_METHOD)
    public String enterStep(final Model model, final RedirectAttributes redirectAttributes) throws CMSItemNotFoundException {
        getCheckoutFacade().setDeliveryModeIfAvailable();
        pspCheckoutFacade.setPaymentModeIfAvailable();

        final CartData cartData = getCheckoutFacade().getCheckoutCart();
        model.addAttribute(CART_DATA_ATTR, cartData);
        model.addAttribute( new SelectPaymentMethodForm());

        setupAddPaymentPage(model);
        setupPaymentMethods(model);
        setCheckoutStepLinksForModel(model, getCheckoutStep());
        Object errorMessageObject = ((BindingAwareModelMap) model).get(PspConstants.ATT_ERROR_MESSAGE);

        if (errorMessageObject instanceof String) {
            String errorCode = (String) errorMessageObject;
            String errorMessage = pspErrorMessageService.fetchErrorMessageByCode(errorCode);
            PspGlobalMessages.addErrorMessage(model, errorMessage);
        }

        return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.PaymentMethodsPage;
    }

    @RequestMapping(value = "/select", method = RequestMethod.POST)
    @RequireHardLogIn
    public String choosePaymentMode(@ModelAttribute("selectPaymentMethodForm") final SelectPaymentMethodForm selectPaymentMethodForm,
                                    @RequestParam(value = "paymentMode", required = false) String paymentMode,
                                    @RequestParam(value = "selectedBrand", required = false) String selectedBrand,
                                    final HttpServletRequest request, RedirectAttributes redirectAttributes, final Model model)
            throws CMSItemNotFoundException {
        if (StringUtils.isNotEmpty(paymentMode)) {
            if (StringUtils.equals(PspConstants.MULTIPLE_BRANDS, paymentMode)) {
                paymentMode = selectedBrand;
            }

            PspPaymentModeData pspPaymentModeData = pspPaymentModeFacade.getPaymentModeForCode(paymentMode);

            if (pspPaymentModeData != null && CollectionUtils.isNotEmpty(pspPaymentModeData.getPaymentModes())) {
                pspCheckoutFacade.setPaymentMode(paymentMode);

                sessionService.getCurrentSession().setAttribute(PSP_3D_SECURE_DTO,populatePsp3DSecureDTO(request, selectPaymentMethodForm));

                if (!pspPaymentModeData.getPaymentModes().get(0).getTokenised()) {
                    pspCheckoutFacade.saveSubscription(pspPaymentModeData);
                    return next(redirectAttributes);
                } else {
                    setupAddPaymentPage(model);
                    final CartData cartData = getCheckoutFacade().getCheckoutCart();
                    model.addAttribute(CART_DATA_ATTR, cartData);

                    BaseSiteModel currentBaseSite = getBaseSiteService().getCurrentBaseSite();
                    String frameUrl = siteBaseUrlResolutionService
                            .getWebsiteUrlForSite(currentBaseSite, TRUE, configurationService.getConfiguration().getString(PARAM_HOSTED_PAYMENT_FRAME_URL));
                    model.addAttribute("hostedOrderFrame", frameUrl);
                    return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.HostedOrderPostPage;
                }
            } else {
                LOGGER.error(paymentMode + " payment mode is either inactive or not found in the system.");
                redirectAttributes.addFlashAttribute(PspConstants.ATT_ERROR_MESSAGE,
                        PspaddonWebConstants.KEY_AUTHORISATION_FAILED);
            }
        }

        // No payment mode is selected, return to payment options
        return getCheckoutStep().currentStep();
    }

    /**
     * Displays the HOP payment page. This is called from within an iframe and
     * will redirect to the PSP hosted form. Any changes to the mapped URL
     * should also be reflected in configuration property psp.conf.frame.url.
     *
     * @param model the model
     * @return the view name
     */
    @RequestMapping(value = {"/showFrame"}, method = RequestMethod.GET)
    public String processPaymentFrame(final Model model) {
        final PaymentData hostedOrderPageData = getPaymentFacade().beginHopCreateSubscription("/checkout/multi/hop/response",
                "/integration/merchant_callback");
        model.addAttribute("hostedOrderPageData", hostedOrderPageData);

        return PspaddonControllerConstants.Views.Pages.MultiStepCheckout.HostedOrderFrame;
    }

    @RequestMapping(value = "/back", method = RequestMethod.GET)
    @RequireHardLogIn
    @Override
    public String back(final RedirectAttributes redirectAttributes) {
        return getCheckoutStep().previousStep();
    }

    @RequestMapping(value = "/next", method = RequestMethod.GET)
    @RequireHardLogIn
    @Override
    public String next(final RedirectAttributes redirectAttributes) {
        return getCheckoutStep().nextStep();
    }

    private void setupPaymentMethods(Model model) {
        PspPaymentModeData paymentModeData = pspPaymentModeFacade.getPaymentModes();
        model.addAttribute("paymentModeData", paymentModeData);
    }

    protected void setupAddPaymentPage(final Model model) throws CMSItemNotFoundException {
        model.addAttribute("metaRobots", "noindex,nofollow");
        model.addAttribute("hasNoPaymentInfo", Boolean.valueOf(pspCheckoutFacade.hasNoPaymentInfo()));
        prepareDataForPage(model);
        model.addAttribute(WebConstants.BREADCRUMBS_KEY,
                getResourceBreadcrumbBuilder().getBreadcrumbs("checkout.multi.paymentMethod.breadcrumb"));
        final ContentPageModel contentPage = getContentPageForLabelOrId(MULTI_CHECKOUT_SUMMARY_CMS_PAGE_LABEL);
        storeCmsPageInModel(model, contentPage);
        setUpMetaDataForContentPage(model, contentPage);
        setCheckoutStepLinksForModel(model, getCheckoutStep());
    }

    protected CheckoutStep getCheckoutStep() {
        return getCheckoutStep(PAYMENT_METHOD);
    }

    /**
     * Gets the request parameters.
     *
     * @param request
     * @return parameters as name value pair
     */
    protected Map<String, String> getParameterMap(final HttpServletRequest request) {
        final Map<String, String> map = new HashMap<>();
        final Enumeration<String> myEnum = request.getParameterNames();
        LOGGER.info("***PSP (3D Secure or Flex Payment) Response Parameters***");

        while (myEnum.hasMoreElements()) {
            final String paramName = myEnum.nextElement();
            final String paramValue = request.getParameter(paramName);
            map.put(StringUtils.upperCase(paramName), paramValue);
            LOGGER.info(paramName + " -> " + paramValue);
        }

        return map;
    }

    private Psp3DSecureDTO populatePsp3DSecureDTO(HttpServletRequest request, SelectPaymentMethodForm placeOrderForm){
        Psp3DSecureDTO psp3DSecureDTO = new Psp3DSecureDTO();

        psp3DSecureDTO.setBrowserAcceptHeader(request.getHeader(ACCEPT_HEADER));
        psp3DSecureDTO.setBrowserUserAgent(request.getHeader(USER_AGENT_HEADER));
        psp3DSecureDTO.setBrowserColorDepth(placeOrderForm.getBrowserColorDepth());
        psp3DSecureDTO.setBrowserLanguage(placeOrderForm.getBrowserLanguage());
        psp3DSecureDTO.setBrowserScreenHeight(placeOrderForm.getBrowserScreenHeight());
        psp3DSecureDTO.setBrowserScreenWidth(placeOrderForm.getBrowserScreenWidth());
        psp3DSecureDTO.setBrowserTimeZone(placeOrderForm.getBrowserTimeZone());
        psp3DSecureDTO.setBrowserJavaEnabled(placeOrderForm.getBrowserJavaEnabled());
        psp3DSecureDTO.setChallengeWindowSize(CHALLENGE_WINDOW_SIZE);

        return psp3DSecureDTO;
    }

    protected int getStatus(Map<String, String> parameters) {
        return MapUtils.getIntValue(parameters, PspConstants.KEY_STATUS);
    }

    protected String getErrorCode(Map<String, String> parameters) {
        return MapUtils.getString(parameters, PspConstants.KEY_NCERROR);
    }
}
