package com.psp.addon.controllers.pages.checkout.steps;

import com.psp.addon.controllers.PspaddonControllerConstants;
import com.psp.addon.forms.PSPAddressForm;
import de.hybris.platform.commercefacades.i18n.I18NFacade;
import de.hybris.platform.commercefacades.order.CheckoutFacade;
import de.hybris.platform.commercefacades.user.UserFacade;
import de.hybris.platform.commercefacades.user.data.AddressData;
import de.hybris.platform.commercefacades.user.data.CountryData;
import de.hybris.platform.commercefacades.user.data.TitleData;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import java.util.Collection;


@Controller
@RequestMapping(value = "/checkout/multi/sop")
public class SopPaymentResponseController {
    @Resource
    private I18NFacade i18NFacade;

    @Resource
    private CheckoutFacade checkoutFacade;

    @Resource
    private UserFacade userFacade;

    @ModelAttribute("titles")
    public Collection<TitleData> getTitles() {
        return getUserFacade().getTitles();
    }

    @RequestMapping(value = "/billingaddressform", method = RequestMethod.GET)
    public String getCountryAddressForm(@RequestParam("countryIsoCode") final String countryIsoCode,
                                        @RequestParam("useDeliveryAddress") final boolean useDeliveryAddress, final Model model) {
        model.addAttribute("supportedCountries", getCountries());
        model.addAttribute("regions", getI18NFacade().getRegionsForCountryIso(countryIsoCode));
        model.addAttribute("country", countryIsoCode);

        final PSPAddressForm pspAddressForm = new PSPAddressForm();
        model.addAttribute("pspAddressForm", pspAddressForm);
        if (useDeliveryAddress) {
            final AddressData deliveryAddress = getCheckoutFacade().getCheckoutCart().getDeliveryAddress();

            if (deliveryAddress.getRegion() != null && !StringUtils.isEmpty(deliveryAddress.getRegion().getIsocode())) {
                pspAddressForm.setBillTo_regionIso(deliveryAddress.getRegion().getIsocodeShort());
            }

            pspAddressForm.setBillTo_titleCode(deliveryAddress.getTitleCode());
            pspAddressForm.setBillTo_firstName(deliveryAddress.getFirstName());
            pspAddressForm.setBillTo_lastName(deliveryAddress.getLastName());
            pspAddressForm.setBillTo_line1(deliveryAddress.getLine1());
            pspAddressForm.setBillTo_line2(deliveryAddress.getLine2());
            pspAddressForm.setBillTo_townCity(deliveryAddress.getTown());
            pspAddressForm.setBillTo_postcode(deliveryAddress.getPostalCode());
            pspAddressForm.setBillTo_countryIso(deliveryAddress.getCountry().getIsocode());
            pspAddressForm.setBillTo_phone(deliveryAddress.getPhone());
        }
        return PspaddonControllerConstants.Views.Fragments.Checkout.BillingAddressForm;
    }

    @ModelAttribute("countries")
    public Collection<CountryData> getCountries() {
        return this.getCheckoutFacade().getDeliveryCountries();
    }

    public I18NFacade getI18NFacade() {
        return i18NFacade;
    }

    public void setI18NFacade(I18NFacade i18NFacade) {
        this.i18NFacade = i18NFacade;
    }

    public CheckoutFacade getCheckoutFacade() {
        return checkoutFacade;
    }

    public void setCheckoutFacade(CheckoutFacade checkoutFacade) {
        this.checkoutFacade = checkoutFacade;
    }

    public UserFacade getUserFacade() {
        return userFacade;
    }

    public void setUserFacade(UserFacade userFacade) {
        this.userFacade = userFacade;
    }
}
